@@ -23,8 +23,8 @@ Annotate::Annotate(Git* parent, QObject* guiObj) : QObject(parent) {
 
 	processingTime.start();
 
-	connect(&patchProc, SIGNAL(readyReadStdout()), this, SLOT(patchProcReadFromStdout()));
-	connect(&patchProc, SIGNAL(processExited()), this, SLOT(patchProcExited()));
+	connect(&patchProc, SIGNAL(readyReadStdout()), this, SLOT(on_patchProc_readFromStdout()));
+	connect(&patchProc, SIGNAL(processExited()), this, SLOT(on_patchProc_processExited()));
 }
 
 const FileAnnotation* Annotate::lookupAnnotation(SCRef sha, SCRef fn) {
@@ -51,7 +51,7 @@ const FileAnnotation* Annotate::lookupAnnotation(SCRef sha, SCRef fn) {
 
 bool Annotate::startPatchProc(SCRef buf, SCRef fileName) {
 
-	const QString cmd("git-diff-tree -r -m --patch-with-raw "
+	const QString cmd("git diff-tree -r -m --patch-with-raw "
 	                  "--no-commit-id --stdin -- " + fileName);
 	patchProc.setArguments(QStringList::split(' ', cmd));
 	patchProc.setWorkingDirectory(git->workDir);
@@ -59,7 +59,7 @@ bool Annotate::startPatchProc(SCRef buf, SCRef fileName) {
 	return patchProc.launch(buf);
 }
 
-void Annotate::patchProcReadFromStdout() {
+void Annotate::on_patchProc_readFromStdout() {
 
 	const QString tmp(patchProc.readStdout());
 	annFilesNum += tmp.contains("diff --git ");
@@ -79,7 +79,7 @@ bool Annotate::stop() {
 	return !annotateRunning;
 }
 
-void Annotate::slotProgressInfo() {
+void Annotate::on_progressTimer_timeout() {
 
 	if (!cancelingAnnotate && !isError) {
 		const QString n(QString::number(annFilesNum));
@@ -87,11 +87,12 @@ void Annotate::slotProgressInfo() {
 	}
 }
 
-bool Annotate::start(SCRef _fileName, const StrVect& _histRevOrder) {
+bool Annotate::start(const FileHistory* _fh) {
 
 	// could change during annotation, so save them
-	fileName = _fileName;
-	histRevOrder = _histRevOrder;
+	fh = _fh;
+	fileName = fh->fileName;
+	histRevOrder = fh->revOrder;
 
 	if (histRevOrder.isEmpty()) {
 		valid = false;
@@ -109,30 +110,30 @@ bool Annotate::start(SCRef _fileName, const StrVect& _histRevOrder) {
 	while (++it != histRevOrder.constEnd());
 
 	// annotation is splitted in two parts.
-	// first we get the list of sha's to feed git-diff-tree.
+	// first we get the list of sha's to feed git diff-tree.
 	// This is very fast.
 	isError = false;
 	annotateFileHistory(fileName, true);
 
 	if (isError || cancelingAnnotate) {
-		computeDiffs(); // clean-up in case of error/canceling
+		slotComputeDiffs(); // clean-up in case of error/canceling
 		return false;
 	}
-	connect(&progressTimer, SIGNAL(timeout()), this, SLOT(slotProgressInfo()));
+	connect(&progressTimer, SIGNAL(timeout()), this, SLOT(on_progressTimer_timeout()));
 	progressTimer.start(500);
 
 	// now we get the patches with an async call. This is the slowest part.
 	return startPatchProc(patchScript, fileName);
 }
 
-void Annotate::patchProcExited() {
+void Annotate::on_patchProc_processExited() {
 
 	// start computing diffs only on return from event handler
-	QTimer::singleShot(1, this, SLOT(computeDiffs()));
+	QTimer::singleShot(1, this, SLOT(slotComputeDiffs()));
 	progressTimer.stop();
 }
 
-void Annotate::computeDiffs() {
+void Annotate::slotComputeDiffs() {
 
 	// when all the patches are loaded we compute the annotation.
 	// this part is normally faster then getting the patches.
@@ -181,7 +182,7 @@ void Annotate::doAnnotate(SCRef fileName, SCRef sha, bool buildPatchScript) {
 	if (fa == NULL || fa->isValid || isError)
 		return;
 
-	const Rev* r = git->revLookup(sha, true); // historyRevs
+	const Rev* r = git->revLookup(sha, fh); // historyRevs
 	if (r == NULL) {
 		dbp("ASSERT doAnnotate: no revision %1", sha);
 		isError = true;
@@ -256,14 +257,17 @@ void Annotate::setInitialAnnotation(SCRef fileName, SCRef sha, FileAnnotation* f
 
 	QString f;
 	SCRef fileSha(git->getFileSha(fileName, sha));
-	if (fileSha.isEmpty())
+	if (fileSha.isEmpty()) {
 		dbp("ASSERT in setInitialAnnotation: empty file of initial rev %1", sha);
-
+		isError = true;
+		return;
+	}
 	ah[sha].fileSha = fileSha;
 	git->getFile(fileName, sha, NULL, &f); // blocking call, calls Qt event loop
 	int lineNum = f.contains('\n');
 	if (!f.endsWith("\n")) // No newline at end of file
 		lineNum++;
+
 	fa->lines.insert(fa->lines.end(), lineNum, "");
 }
 
@@ -383,7 +387,7 @@ const QString Annotate::getNextPatch(QString& patchFile, SCRef fileName, SCRef s
 	if (sha == QGit::ZERO_SHA) {
 		// diff-tree --stdin doesn't work with working dir so get it
 		// directly. We don't need file sha info because is ZERO_SHA
-		QString runCmd("git-diff-index -r -m -p HEAD -- " + fileName), runOutput;
+		QString runCmd("git diff-index -r -m -p HEAD -- " + fileName), runOutput;
 		git->run(runCmd, &runOutput);
 
 		// restore removed head line of next patch
@@ -412,7 +416,7 @@ const QString Annotate::getNextPatch(QString& patchFile, SCRef fileName, SCRef s
 
 	int start = diff.find('@');
 	// handle a possible file mode only change and remove header
-	diff = (start != -1) ? diff.right(diff.length() - start) : "";
+	diff = (start != -1) ? diff.mid(start) : "";
 
 	int i = 0;
 	while (diffMap.contains(Key(sha, i)))
@@ -724,7 +728,7 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart,
 
 	// going back in history, to oldest following first parent lane
 	const QString oldest(histRevOrder.last()); // causes a detach!
-	const Rev* curRev = git->revLookup(ancestor, true); // historyRevs
+	const Rev* curRev = git->revLookup(ancestor, fh); // historyRevs
 	QString curRevSha(curRev->sha());
 	while (curRevSha != oldest) {
 
@@ -755,7 +759,7 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart,
 		if (curRev->parentsCount() == 0)
 			break;
 
-		curRev = git->revLookup(curRev->parent(0), true);
+		curRev = git->revLookup(curRev->parent(0), fh);
 		curRevSha = curRev->sha();
 		rangeMap.insert(curRevSha, r);
 
@@ -771,7 +775,7 @@ const QString Annotate::computeRanges(SCRef sha, int rangeStart,
 
 		if (!rangeMap.contains(sha)) {
 
-			curRev = git->revLookup(sha, true);
+			curRev = git->revLookup(sha, fh);
 
 			if (curRev->parentsCount() == 0) {
 				// the start of an indipendent branch is found in this case
