@@ -14,8 +14,8 @@
 
 using namespace QGit;
 
-bool Cache::update(const QString& workDir, const RevFileMap& rf,
-                   const StrVect& dirs, const StrVect& files) {
+bool Cache::save(const QString& workDir, const RevFileMap& rf,
+                 const StrVect& dirs, const StrVect& files) {
 
 	if (workDir.isEmpty() || rf.isEmpty())
 		return false;
@@ -51,28 +51,44 @@ bool Cache::update(const QString& workDir, const RevFileMap& rf,
 	for (uint i = 0; i < files.count(); ++i)
 		stream << files[i];
 
-	loop(RevFileMap, it, rf) {
+	// to achieve a better compression we save the sha's as
+	// one very long string instead of feeding the stream with
+	// each one. With this trick we gain a 15% size reduction
+	// in the final compressed file. The save/load speed is
+	// almost the same.
+	uint bufSize = rf.count() * 40 + 1000; // a little bit more space then required
+	stream << (Q_INT32)bufSize;
 
-		if (it.key() == ZERO_SHA)
+	QString buf;
+	buf.reserve(bufSize);
+	QDictIterator<RevFile> it(rf);
+	for ( ; it.current(); ++it) {
+		if (   it.currentKey() == ZERO_SHA
+		    || it.currentKey() == CUSTOM_SHA
+		    || it.currentKey().startsWith("A")) // ALL_MERGE_FILES + rev sha
 			continue;
 
-		if (it.key() == CUSTOM_SHA)
+		buf.append(it.currentKey());
+	}
+	stream << buf;
+
+	for (it.toFirst() ; it.current(); ++it) {
+		if (   it.currentKey() == ZERO_SHA
+		    || it.currentKey() == CUSTOM_SHA
+		    || it.currentKey().startsWith("A")) // ALL_MERGE_FILES + rev sha
 			continue;
 
-		if (it.key().startsWith("A")) // ALL_MERGE_FILES + rev sha
-			continue;
-
-		stream << it.key();
-		stream << (*it).names;
-		stream << (*it).dirs;
-		stream << (*it).status;
-		stream << (*it).mergeParent;
+		stream << it.current()->names;
+		stream << it.current()->dirs;
+		stream << it.current()->status;
+		stream << it.current()->mergeParent;
 	}
 	dbs("Compressing data...");
-	f.writeBlock(qCompress(data)); // no need to encode with QDataStream compressed data
+	f.writeBlock(qCompress(data)); // no need to encode with compressed data
 	f.close();
 
-	if (dir.exists(path)) { // rename C_DAT_FILE + BAK_EXT -> C_DAT_FILE
+	// rename C_DAT_FILE + BAK_EXT -> C_DAT_FILE
+	if (dir.exists(path)) {
 		if(!dir.remove(path)) {
 			dbs("access denied to " + path);
 			dir.remove(tmpPath);
@@ -106,10 +122,9 @@ bool Cache::load(const QString& workDir, RevFileMap& rf,
 		return false;
 
 	QDataStream* stream = new QDataStream(qUncompress(f.readAll()), IO_ReadOnly);
-
 	Q_UINT32 magic;
 	Q_INT32 version;
-	Q_INT32 dirsNum, filesNum;
+	Q_INT32 dirsNum, filesNum, bufSize;
 	*stream >> magic;
 	*stream >> version;
 	if (magic != C_MAGIC || version != C_VERSION) {
@@ -128,14 +143,23 @@ bool Cache::load(const QString& workDir, RevFileMap& rf,
 	for (int i = 0; i < filesNum; ++i)
 		*stream >> files[i];
 
-	QString sha;
+	*stream >> bufSize;
+	QString buf;
+	buf.reserve(bufSize);
+	*stream >> buf;
+
+	uint bufIdx = 0;
 	while (!stream->atEnd()) {
-		*stream >> sha;
-		RevFileMap::Iterator it = rf.insert(sha, RevFile());
-		*stream >> (*it).names;
-		*stream >> (*it).dirs;
-		*stream >> (*it).status;
-		*stream >> (*it).mergeParent;
+
+		SCRef sha(buf.mid(bufIdx, 40));
+		bufIdx += 40;
+
+		rf.insert(sha, new RevFile());
+		RevFile* p = rf[sha];
+		*stream >> p->names;
+		*stream >> p->dirs;
+		*stream >> p->status;
+		*stream >> p->mergeParent;
 	}
 	f.close();
 	delete stream;
