@@ -13,8 +13,8 @@ class Cache : public QObject {
 Q_OBJECT
 public:
 	Cache(QObject *par = 0) : QObject(par) {};
-	bool update(const QString& workDir, const RevFileMap& rf,
-	            const StrVect& dirs, const StrVect& files);
+	bool save(const QString& workDir, const RevFileMap& rf,
+	          const StrVect& dirs, const StrVect& files);
 	bool load(const QString& workDir, RevFileMap& rf,
 	          StrVect& dirs, StrVect& files);
 };
