@@ -8,8 +8,6 @@
    Part of this code is taken from Fredrik Kuivinen "Gct"
    tool. I have just translated from Python to C++
 */
-
-#include <stdlib.h>
 #include <qlistview.h>
 #include <qsettings.h>
 #include <qlabel.h>
@@ -96,13 +94,12 @@ CommitImpl::CommitImpl(Git* g, bool* ok) : CommitBase(0, 0, Qt::WDestructiveClos
 	const RevFile* files = git->getFiles(ZERO_SHA);
 	for (uint i = 0; i < files->names.count(); ++i) {
 
-		SCRef st(files->status[i]);
-		SCRef lbl((st.startsWith(CACHED_FILE)) ? "In sync" : "Out of sync");
+		SCRef lbl(files->isInIndex(i) ? "In sync" : "Out of sync");
 
 		CheckListFileItem* item = new CheckListFileItem(listViewFiles,
-		                              git->filePath(*files, i), st.right(1), lbl);
+		                          git->filePath(*files, i), files->getStatus(i), lbl);
 
-		item->setOn(!st.endsWith(UNKNOWN));
+		item->setOn(!files->statusCmp(i, UNKNOWN));
 	}
 	// setup textEditMsg with default value
 	QString status(git->getDefCommitMsg());
@@ -270,7 +267,7 @@ void CommitImpl::pushButtonOk_clicked() {
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 	EM_PROCESS_EVENTS; // to close message box
 	if (git->isStGITStack())
-		*commitOK = git->stgCommit(selFiles, msg, patchName, false);
+		*commitOK = git->stgCommit(selFiles, msg, patchName, !Git::optFold);
 	else
 		*commitOK = git->commitFiles(selFiles, msg);
 
@@ -297,7 +294,7 @@ void CommitImpl::pushButtonUpdateCache_clicked() {
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 	EM_PROCESS_EVENTS; // to close message box
 	if (git->isStGITStack())
-		*commitOK = git->stgCommit(selFiles, msg, "", true);
+		*commitOK = git->stgCommit(selFiles, msg, "", Git::optFold);
 	else
 		*commitOK = git->updateIndex(selFiles);
 
