@@ -32,9 +32,9 @@
                        for ( ; x != y.constEnd(); ++x)
 
 // type shortcuts
-typedef const QString& SCRef;
-typedef QStringList& SList;
-typedef const QStringList& SCList;
+typedef const QString&        SCRef;
+typedef QStringList&          SList;
+typedef const QStringList&    SCList;
 typedef QValueVector<QString> StrVect;
 
 namespace QGit {
@@ -120,7 +120,8 @@ namespace QGit {
 		POPUP_TREE_EV = 65435,
 		MSG_EV        = 65436,
 		ANN_PRG_EV    = 65437,
-		UPD_DOMAIN_EV = 65438
+		UPD_DM_EV     = 65438,
+		UPD_DM_MST_EV = 65439
 	};
 
 	// list views columns
@@ -136,7 +137,7 @@ namespace QGit {
 		SHA_MAP_COL = 99 // dummy col used when filter output is a set of matching sha
 	};
 
-	inline bool isInfoCol(int x){ return (x == TIME_COL || x == LOG_COL || x == AUTH_COL); }
+	inline bool isInfoCol(int x) { return (x == TIME_COL || x == LOG_COL || x == AUTH_COL); }
 
 	// default list view widths
 	const int DEF_GRAPH_COL_WIDTH = 80;
@@ -164,13 +165,15 @@ namespace QGit {
 	const QChar MODIFIED = 'M';
 	const QChar DELETED  = 'D';
 	const QChar NEW      = 'A';
+	const QChar RENAMED  = 'R';
+	const QChar COPIED   = 'C';
 	const QChar UNKNOWN  = '?';
 
 	// git index parameters
 	const QString ZERO_SHA        = "0000000000000000000000000000000000000000";
 	const QString CUSTOM_SHA      = "CUSTOM";
-	const QChar CACHED_FILE       = 'C';
-	const QChar NO_CACHED_FILE    = 'N';
+	const QChar IN_INDEX          = 'I';
+	const QChar NOT_IN_INDEX      = 'N';
 	const QString ALL_MERGE_FILES = "ALL_MERGE_FILES";
 
 	// settings keys
@@ -226,7 +229,7 @@ namespace QGit {
 
 	// cache file
 	const uint C_MAGIC  = 0xA0B0C0D0;
-	const int C_VERSION = 10;
+	const int C_VERSION = 12;
 
 	const QString BAK_EXT          = ".bak";
 	const QString C_DAT_FILE       = "/qgit_cache.dat";
@@ -234,6 +237,7 @@ namespace QGit {
 	const QString C_OLD_DAT_FILE_Z = "/.qgit_cache.dat.z";
 
 	// misc
+	const int MAX_DICT_SIZE    = 100003; // must be a prime number see QDict docs
 	const int MAX_MENU_ENTRIES = 15;
 	const int MAX_RECENT_REPOS = 5;
 	const QString QUOTE_CHAR   = "$";
@@ -241,7 +245,7 @@ namespace QGit {
 
 class Rev {
 public:
-	Rev() {};
+	Rev() {}
 	Rev(const QString& data, int idx) : orderIdx(idx), d(data), indexed(false) {
 		isTag = isBranch = isRef = isCurrentBranch = false;
 		isDiffCache = isApplied = isUnApplied = false,
@@ -250,20 +254,20 @@ public:
 		// these fields must be valid from beginning
 		isBoundary = (data[0] == '-'); // public field
 		parentsCnt = (data.find('\n', 39) - (int)isBoundary) / 41;
-	};
+	}
 	const QString parent(int idx) const;
 	const QStringList parents() const;
-	const QString sha() const { return isBoundary ? d.mid(1, 40) : d.left(40); };
-	uint parentsCount() const { return parentsCnt; };
-	const QString author() const { init(); return d.mid(autStart, autLen); };
-	const QString authorDate() const { init();return d.mid(autDateStart, autDateLen);};
-	const QString shortLog() const { init(); return d.mid(sLogStart, sLogLen); };
-	const QString longLog() const { init(); return d.right(lLogLen); };
+	const QString sha() const { return isBoundary ? d.mid(1, 40) : d.left(40); }
+	uint parentsCount() const { return parentsCnt; }
+	const QString author() const { init(); return d.mid(autStart, autLen); }
+	const QString authorDate() const { init(); return d.mid(autDateStart, autDateLen); }
+	const QString shortLog() const { init(); return d.mid(sLogStart, sLogLen); }
+	const QString longLog() const { init(); return d.right(lLogLen); }
 
 	QValueVector<int> lanes, childs;
 	QString tag, branch, ref, tagMsg;
 	bool isTag, isBranch, isRef, isCurrentBranch;
-	bool isDiffCache, isApplied, isUnApplied,isBoundary;
+	bool isDiffCache, isApplied, isUnApplied, isBoundary;
 	int orderIdx;
 	QValueVector<int> descRefs;     // list of descendant refs index, normally tags
 	QValueVector<int> ancRefs;      // list of ancestor refs index, normally tags
@@ -273,74 +277,88 @@ public:
 	int descBrnMaster;  // by corresponding index xxxMaster
 
 private:
+	void init() const { if (!indexed) indexData(); }
 	void indexData() const;
-	void init() const {
-		if (indexed)
-			return;
-		indexData();
-		indexed = true;
-	};
+
 	const QString d;
-	mutable bool indexed;
 	uint parentsCnt;
+	mutable bool indexed;
 	mutable int autStart, autLen, autDateStart;
 	mutable int autDateLen, sLogStart, sLogLen, lLogLen;
 };
+typedef QDict<Rev> RevMap; // faster then a map
 
 class RevFile {
+
+	friend class Cache; // to directly load status
+	friend class Git;
+
+	// 'status' is one ore more comma separated fields. First field
+	// is always the status returned by 'git diff-tree' without -C option.
+	// If file is renamed or copied a second field is added with the status
+	// returned by 'git diff-tree -C' plus source and destination files info.
+	// In case of a working dir file a third field is added with info about
+	// git index status of the file (CACHED_FILE or NO_CACHED_FILE)
+	QValueVector<QString> status;
+
 public:
-	RevFile() {};
-	QValueVector<int> dirs; // index in a dict vector
+	RevFile() {}
+
+	QValueVector<int> dirs; // index of a string vector
 	QValueVector<int> names;
 	QValueVector<int> mergeParent;
-	QValueVector<QString> status;
+
+	// status helper functions
+	QChar getStatus(int idx) const { return status[idx].at(0); }
+	bool statusCmp(int idx, const QChar& st) const { return status[idx].at(0) == st; }
+	bool isExtendedStatus(int idx) const { return status[idx].length() != 1; }
+	bool isInIndex(int idx) const {
+
+		return (isExtendedStatus(idx) && status[idx].section(',', 2, 2) == QGit::IN_INDEX);
+	}
+	QString getExtendedStatus(int idx) const {
+
+		return (isExtendedStatus(idx) ? status[idx].section(',', 1, 1) : "");
+	}
 };
-typedef QMap<QString, RevFile> RevFileMap;
+typedef QDict<RevFile> RevFileMap;
 
 class FileAnnotation {
 public:
-	FileAnnotation(int id) { annId = id; isValid = false; };
-	FileAnnotation() { isValid = false; };
+	explicit FileAnnotation(int id) { annId = id; isValid = false; }
+	FileAnnotation() { isValid = false; }
 	QStringList lines;
 	bool isValid;
 	int annId;
 	QString fileSha;
 };
 
-// payload custom events
-class DataEvent: public QCustomEvent {
+class BaseEvent: public QCustomEvent {
 public:
-	DataEvent(const QString& d, int id) : QCustomEvent(id), payLoad(d) {}
-	const QString data() { return payLoad; }
+	BaseEvent(SCRef d, int id) : QCustomEvent(id), payLoad(d) {}
+	const QString data() const { return payLoad; }
 private:
 	const QString payLoad; // passed by copy
 };
 
-class DeferredPopupEvent : public DataEvent {
+#define DEF_EVENT(X, T) class X : public BaseEvent { public: \
+                        X (SCRef d) : BaseEvent(d, T) {} }
+
+DEF_EVENT(MessageEvent, QGit::MSG_EV);
+DEF_EVENT(AnnotateProgressEvent, QGit::ANN_PRG_EV);
+
+class DeferredPopupEvent : public BaseEvent {
 public:
-	DeferredPopupEvent(const QString& msg, int type) : DataEvent(msg, type) {}
+	DeferredPopupEvent(SCRef msg, int type) : BaseEvent(msg, type) {}
 };
 
-class MessageEvent : public DataEvent {
+class MainExecErrorEvent : public BaseEvent {
 public:
-	MessageEvent(const QString& msg) : DataEvent(msg, QGit::MSG_EV) {}
-};
-
-class AnnotateProgressEvent : public DataEvent {
-public:
-	AnnotateProgressEvent(const QString& n) : DataEvent(n, QGit::ANN_PRG_EV) {}
-};
-
-// specialized custom events
-class MainExecErrorEvent : public QCustomEvent {
-public:
-	MainExecErrorEvent(const QString& c, const QString& e)
-	: QCustomEvent(QGit::ERROR_EV), cmd(c), errorDesc(e) {}
-	const QString command() { return cmd; }
-	const QString report() { return errorDesc; }
+	MainExecErrorEvent(SCRef c, SCRef e) : BaseEvent("", QGit::ERROR_EV), cmd(c), err(e) {}
+	const QString command() const { return cmd; }
+	const QString report() const { return err; }
 private:
-	const QString cmd;
-	const QString errorDesc;
+	const QString cmd, err;
 };
 
 #endif
