@@ -22,7 +22,7 @@ public:
 	bool start(const QString& cmd,const QString& args);
 
 signals:
-	void macro_exited(const QString& name);
+	void customAction_exited(const QString& name);
 
 public slots:
 	void on_procDataReady(const QString& data);
