@@ -9,15 +9,15 @@
 #include "git.h"
 #include "dataloader.h"
 
-#define  GUI_UPDATE_INTERVAL	500
-#define  BUF_RING_SIZE		50000
+#define  GUI_UPDATE_INTERVAL 500
+#define  BUF_RING_SIZE       50000
 
-DataLoader::DataLoader(Git* g, bool h) : QObject((QObject*)g), git(g), isHistory(h) {
+DataLoader::DataLoader(Git* g, FileHistory* f)
+           : QObject((QObject*)g), git(g), fh(f) {
 
 	canceling = false;
 
 	buffersRing.setAutoDelete(false); // manual memory management
-	buffersRing.clear();
 	buffersRing.resize(BUF_RING_SIZE);
 
 	connect(git, SIGNAL(cancelAllProcesses()), this, SLOT(on_cancel()));
@@ -37,6 +37,12 @@ void DataLoader::on_cancel() {
 	proc.tryTerminate();
 }
 
+void DataLoader::on_cancel(const FileHistory* f) {
+
+	if (f == fh)
+		on_cancel();
+}
+
 bool DataLoader::start(SCRef cmd, SCRef wd) {
 
 	buffersRingHead = buffersRingTail = grandTot = 0;
@@ -48,7 +54,7 @@ bool DataLoader::start(SCRef cmd, SCRef wd) {
 
 	loadTime.start();
 
-	if (!proc.launch(QByteArray())) {
+	if (!proc.start()) {
 		deleteLater();
 		return false;
 	}
@@ -87,31 +93,28 @@ void DataLoader::on_processExited() {
 void DataLoader::on_timeout() {
 
 	parsing = true;
-	// process could exit while we are processing so save the flag now.
+	// process could exit while we are processing so save the flag now
 	bool lastBuffer = isProcExited;
 
-	uint bsize = 0;
 	while (buffersRingTail != buffersRingHead) {
 
 		const QByteArray* ba(buffersRing.at(buffersRingTail));
 		parseSingleBuffer(*ba); // about 25% of total time
-		bsize += ba->size();
+		grandTot += ba->size();
 		delete ba;
 
 		if (++buffersRingTail == BUF_RING_SIZE)
 			buffersRingTail = 0;
 	}
-	grandTot += bsize;
-
 	if (!canceling)
-		emit newDataReady(); // inserting in list view is about 5% of total time
+		emit newDataReady(fh); // inserting in list view is about 5% of total time
 
 	if (lastBuffer) {
-		QString ds(proc.readStderr());
-		QString cmd(proc.arguments().join(" "));
+		const QString ds(proc.readStderr());
+		const QString cmd(proc.arguments().join(" "));
 		if (!canceling)
-			emit loaded(grandTot, loadTime, proc.normalExit(), cmd, ds);
-
+			emit loaded(fh, grandTot, loadTime.elapsed(),
+			            proc.normalExit(), cmd, ds);
 		deleteLater();
 	} else
 		guiUpdateTimer.start(GUI_UPDATE_INTERVAL, true);
@@ -139,10 +142,10 @@ void DataLoader::parseSingleBuffer(const QByteArray& ba) {
 		const QString& chunk(QString::fromAscii(&(ba[prevIdx]), idx - prevIdx));
 
 		if (halfChunks.isEmpty())
-			git->addChunk(isHistory, chunk); // about 20% of total time
+			git->addChunk(fh, chunk); // about 20% of total time
 		else {
 			// unlikely path (10% of cases), we can accept a detach
-			git->addChunk(isHistory, halfChunks + chunk);
+			git->addChunk(fh, halfChunks.append(chunk));
 			halfChunks = "";
 		}
 		updateIndex(&idx, &prevIdx, ba);
