@@ -15,20 +15,22 @@
 #include <qptrvector.h>
 
 class Git;
+class FileHistory;
 
 class DataLoader : public QObject {
 Q_OBJECT
 public:
-	DataLoader(Git* g, bool isHistory);
+	DataLoader(Git* g, FileHistory* f);
 	~DataLoader();
 	bool start(const QString& cmd, const QString& wd);
 
 signals:
-	void newDataReady();
-	void loaded(ulong,const QTime&,bool,const QString&,const QString&);
+	void newDataReady(const FileHistory*);
+	void loaded(const FileHistory*,ulong,int,bool,const QString&,const QString&);
 
 private slots:
 	void on_cancel();
+	void on_cancel(const FileHistory*);
 	void on_readyReadStdout();
 	void on_processExited();
 	void on_timeout();
@@ -38,7 +40,7 @@ private:
 	void updateIndex(int* idx, int* prevIdx, const QByteArray& ba);
 
 	Git* git;
-	bool isHistory;
+	FileHistory* fh;
 	QProcess proc;
 	QString halfChunks;
 	QPtrVector<QByteArray> buffersRing;
