@@ -86,10 +86,23 @@ bool StateInfo::operator!=(const StateInfo& newState) const {
 
 Domain::Domain(MainImpl* m, Git* g) : QObject(m), git(g) {
 
-	d = this;
 	st.clear();
-	busy = readyToDrag = dragging = dropping = false;
+	busy = readyToDrag = dragging = dropping = linked = false;
 	popupType = 0;
+	tabPosition = -1;
+
+	connect(m, SIGNAL(tabClosed(int)), this, SLOT(on_tabClosed(int)));
+}
+
+void Domain::on_closeAllTabs() {
+
+	delete this; // must be sync, deleteLater() does not work
+}
+
+void Domain::on_tabClosed(int tabPos) {
+
+	if (tabPosition > tabPos)
+		tabPosition--;
 }
 
 MainImpl* Domain::m() const { return static_cast<MainImpl*>(parent()); }
@@ -115,11 +128,32 @@ bool Domain::setDragging(bool b) {
 	return dragging;
 }
 
+void Domain::unlinkDomain(Domain* d) {
+
+	d->linked = false;
+	while (d->disconnect(SIGNAL(updateRequested(StateInfo)), this))
+		;// a signal is emitted for every connection you make,
+		 // so if you duplicate a connection, two signals will be emitted.
+}
+
+void Domain::linkDomain(Domain* d) {
+
+	unlinkDomain(d); // be sure only one connection is active
+	connect(d, SIGNAL(updateRequested(StateInfo)), this, SLOT(on_updateRequested(StateInfo)));
+	d->linked = true;
+}
+
+void Domain::on_updateRequested(StateInfo newSt) {
+
+	st = newSt;
+	UPDATE();
+}
+
 bool Domain::flushQueue() {
 // during dragging any state update is queued, so try to flush pending now
 
 	if (!busy && st.flushQueue()) {
-		UPDATE_DOMAIN(d);
+		UPDATE();
 		return true;
 	}
 	return false;
@@ -127,9 +161,14 @@ bool Domain::flushQueue() {
 
 void Domain::customEvent(QCustomEvent* e) {
 
+	bool fromMaster = false;
+
 	switch (e->type()) {
-	case UPD_DOMAIN_EV:
-		update();
+	case UPD_DM_MST_EV:
+		fromMaster = true;
+		// fall through
+	case UPD_DM_EV:
+		update(fromMaster);
 		break;
 	case MSG_EV:
 		if (!busy && !st.requestPending())
@@ -150,7 +189,7 @@ void Domain::populateState() {
 		st.setIsMerge(r->parentsCount() > 1);
 }
 
-void Domain::update() {
+void Domain::update(bool fromMaster) {
 
 	if (st.sha().isEmpty()) {
 		dbs("ASSERT update called with empty sha");
@@ -159,6 +198,13 @@ void Domain::update() {
 	if (busy || dragging)
 		return;
 
+	if (linked && !fromMaster) {
+		// in this case let the update to fall down from master domain
+		StateInfo tmp(st);
+		st.rollBack(); // we don't want to filter out next update sent from master
+		emit updateRequested(tmp);
+		return;
+	}
 	busy = true;
 
 	setReadyToDrag(false); // do not start any drag while updating
