@@ -10,7 +10,9 @@
 #include <qobject.h>
 #include "common.h"
 
-#define UPDATE_DOMAIN(x) QApplication::postEvent(x, new UpdateDomainEvent())
+#define UPDATE_DOMAIN(x)    QApplication::postEvent(x, new UpdateDomainEvent(false))
+#define UPDATE()            QApplication::postEvent(this, new UpdateDomainEvent(false))
+#define UPDATE_DM_MASTER(x) QApplication::postEvent(x, new UpdateDomainEvent(true))
 
 class Domain;
 class MainImpl;
@@ -18,7 +20,8 @@ class Git;
 
 class UpdateDomainEvent : public QCustomEvent {
 public:
-	UpdateDomainEvent() : QCustomEvent(QGit::UPD_DOMAIN_EV) {}
+	UpdateDomainEvent(bool fromMaster)
+	: QCustomEvent(fromMaster ? QGit::UPD_DM_MST_EV : QGit::UPD_DM_EV) {}
 };
 
 class StateInfo {
@@ -85,6 +88,7 @@ private:
 class Domain: public QObject {
 Q_OBJECT
 public:
+	Domain() {}
 	Domain(MainImpl* m, Git* git);
 	MainImpl* m() const;
 	bool isReadyToDrag() const { return readyToDrag; }
@@ -93,29 +97,43 @@ public:
 	bool setDragging(bool b);
 	bool isDropping() const { return dropping; }
 	void setDropping(bool b) { dropping = b; }
+	bool isLinked() const { return linked; }
+	int tabPos() const { return tabPosition; }
+	virtual QWidget* tab() = 0;
 
 	StateInfo st;
 
+signals:
+	void updateRequested(StateInfo newSt);
+
+public slots:
+	void on_closeAllTabs();
+
 protected slots:
-	void on_contextMenu(const QString&, int);
+	virtual void on_contextMenu(const QString&, int);
+	void on_tabClosed(int);
+	void on_updateRequested(StateInfo newSt);
 
 protected:
 	virtual void customEvent(QCustomEvent* e);
 	virtual bool doUpdate() = 0;
+	void linkDomain(Domain* d);
+	void unlinkDomain(Domain* d);
 
-	Domain* d; // to use UPDATE_DOMAIN() macro
 	Git* git;
 	bool busy;
+	int tabPosition;
 
 private:
 	void populateState();
-	void update();
+	void update(bool fromMaster);
 	bool flushQueue();
 	void sendPopupEvent();
 
 	bool readyToDrag;
 	bool dragging;
 	bool dropping;
+	bool linked;
 	int popupType;
 	QString popupData;
 	QString statusBarRequest;
