@@ -23,7 +23,7 @@ void ExceptionManager::init(int* excp, const QString& desc) {
 	descriptions.append(desc);
 }
 
-QString ExceptionManager::desc(int excpId) {
+const QString ExceptionManager::desc(int excpId) {
 
 	return descriptions[excpId];
 }
@@ -60,7 +60,7 @@ void ExceptionManager::remove(int excpId) {
 		return;
 	}
 	// remove from region.
-	ThrowableSet::Iterator itReg = regionThrowableSet.begin();
+	SetIt itReg(regionThrowableSet.begin());
 	if ((*itReg).excpId != excpId) {
 		qDebug("ASSERT in remove: %i is not the first in list", excpId);
 		return;
@@ -68,7 +68,7 @@ void ExceptionManager::remove(int excpId) {
 	regionThrowableSet.remove(itReg);
 
 	// remove from total.
-	ThrowableSet::Iterator itTot = totalThrowableSet.begin();
+	SetIt itTot(totalThrowableSet.begin());
 	if ((*itTot).excpId != excpId) {
 		qDebug("ASSERT in remove: %i is not the first in list", excpId);
 		return;
@@ -121,7 +121,7 @@ int ExceptionManager::saveThrowableSet() {
 // regionThrowableSet is saved with the current region index.
 // then current region is changed to a new and never used index
 
-	int ret = currentRegionId;
+	int oldCurrentRegionId = currentRegionId;
 	throwableSetMap.insert(currentRegionId, regionThrowableSet);
 	currentRegionId = ++regionId;
 
@@ -130,7 +130,7 @@ int ExceptionManager::saveThrowableSet() {
 	// have totalThrowableSet to catch any request.
 	regionThrowableSet.clear();
 
-	return ret;
+	return oldCurrentRegionId;
 }
 
 void ExceptionManager::restoreThrowableSet(int regionId) {
@@ -148,12 +148,12 @@ void ExceptionManager::throwPending() {
 	if (regionThrowableSet.isEmpty())
 		return;
 
-	SetIt it(regionThrowableSet.begin());
-	for ( ; it != regionThrowableSet.end(); ++it)
+	ThrowableSet::const_iterator it(regionThrowableSet.constBegin());
+	for ( ; it != regionThrowableSet.constEnd(); ++it)
 		if ((*it).isRaised)
 			break;
 
-	if (it == regionThrowableSet.end())
+	if (it == regionThrowableSet.constEnd())
 		return;
 
 	int excpToThrow = (*it).excpId;
