@@ -23,22 +23,28 @@
 
 #define MAX_LINE_NUM 5
 
+const QString FileContent::HTML_HEAD       = "<font color=\"#C0C0C0\">"; // light gray
+const QString FileContent::HTML_TAIL       = "</font>";
+const QString FileContent::HTML_FILE_START = "<pre><tt>";
+const QString FileContent::HTML_FILE_END   = "</tt></pre>";
+
 class FileHighlighter : public QSyntaxHighlighter {
 public:
 	FileHighlighter(QTextEdit* te, FileContent* fc) : QSyntaxHighlighter(te), f(fc) {};
 	int highlightParagraph(const QString& p, int) {
 
+		if (f->isHtmlSource)
+			return 0;
+
 		if (!f->isRangeFilterActive)
 			setFormat(0, p.length(), textEdit()->currentFont());
 
-		if (f->isShowAnnotate)
-			setFormat(0, f->annoLen + MAX_LINE_NUM, Qt::lightGray);
-		else
-			setFormat(0, MAX_LINE_NUM, Qt::lightGray);
+		int headLen = f->isAnnotationAppended ? f->annoLen + MAX_LINE_NUM : MAX_LINE_NUM;
+		setFormat(0, headLen, Qt::lightGray);
 
 		if (f->isRangeFilterActive && f->rangeInfo->start != 0)
-			if (f->rangeInfo->start - 1 <= currentParagraph() &&
-			    f->rangeInfo->end - 1 >= currentParagraph()) {
+			if (   f->rangeInfo->start - 1 <= currentParagraph()
+			    && f->rangeInfo->end - 1 >= currentParagraph()) {
 
 				QFont fn(textEdit()->currentFont());
 				fn.setBold(true);
@@ -50,24 +56,23 @@ private:
 	FileContent* f;
 };
 
-FileContent::FileContent(Domain* dm, Git* g, QTextEdit* f) :
-                         d(dm), git(g), ft(f) {
+FileContent::FileContent(Domain* dm, Git* g, QTextEdit* f) : d(dm), git(g), ft(f) {
 
 	st = &(d->st);
 
 	// init native types
+	isRangeFilterActive = isHtmlSource = false;
 	isShowAnnotate = true;
-	isFileAvailable = isRangeFilterActive = false;
-	isAnnotateAvailable = isAnnotationAppended = false;
-	annoLen = 0;
-	line = 1;
+	clearAnnotate();
+	clearText(false);
 
 	rangeInfo = new RangeInfo();
 	fileHighlighter = new FileHighlighter(ft, this);
 
-	connect(ft, SIGNAL(doubleClicked(int,int)),this, SLOT(on_doubleClicked(int,int)));
+	connect(ft, SIGNAL(doubleClicked(int,int)), this, SLOT(on_doubleClicked(int,int)));
+
 	connect(git, SIGNAL(annotateReady(Annotate*, const QString&, bool, const QString&)),
-	        this, SLOT(on_annotateReady(Annotate*,const QString&,bool,const QString&)));
+	        this, SLOT(on_annotateReady(Annotate*,const QString&,bool, const QString&)));
 }
 
 FileContent::~FileContent() {
@@ -81,9 +86,9 @@ void FileContent::clearAnnotate() {
 
 	git->cancelAnnotate(annotateObj);
 	annotateObj = NULL;
-	annoLen = 0;
 	curAnn = NULL;
-	isAnnotateAvailable = false;
+	annoLen = 0;
+	isAnnotationAvailable = false;
 	emit annotationAvailable(false);
 }
 
@@ -91,17 +96,51 @@ void FileContent::clearText(bool emitSignal) {
 
 	git->cancelProcess(proc);
 	ft->clear();
-	fileData = pending = "";
+	fileRowData = fileProcessedData = halfLine = "";
 	isFileAvailable = isAnnotationAppended = false;
-	line = 1;
+	curLine = 1;
+	if (curAnn)
+		curAnnIt = curAnn->lines.constBegin();
+
 	if (emitSignal)
 		emit fileAvailable(false);
 }
 
 void FileContent::clear() {
 
-	clearText();
 	clearAnnotate();
+	clearText();
+}
+
+void FileContent::setShowAnnotate(bool b) {
+// add an annotation if is available and still not appended, this
+// can happen if annotation became available while loading the file.
+// If isShowAnnotate is unset try to remove any annotation.
+
+	isShowAnnotate = b;
+
+	if (   !isFileAvailable
+	    || (curAnn == NULL && isShowAnnotate)
+	    || (isAnnotationAppended == isShowAnnotate))
+		return;
+
+	// re-feed with file content processData()
+	saveScreenState();
+	const QString tmp(fileRowData);
+	clearText(false); // emitSignal = false
+	processData(tmp);
+	on_eof(false); // print and call restoreScreenState()
+}
+
+void FileContent::setHighlightSource(bool b) {
+
+	if (b && !git->isTextHighlighter()) {
+		dbs("ASSERT in setHighlightSource: no highlighter found");
+		return;
+	}
+	ft->setTextFormat(b ? Qt::RichText : Qt::PlainText);
+	isHtmlSource = b;
+	update(true);
 }
 
 void FileContent::update(bool force) {
@@ -119,24 +158,26 @@ void FileContent::update(bool force) {
 	else
 		clearText();
 
-	lookupAnnotation(); // before file loading
-	proc = git->getFile(st->fileName(), st->sha(), this, NULL); // non blocking
+	if (!force)
+		lookupAnnotation(); // before file loading
+
+	if (isHtmlSource) // both calls bound on_eof() and on_procDataReady() slots
+		proc = git->getHighlightedFile(st->fileName(), st->sha(), this, NULL);
+	else
+		proc = git->getFile(st->fileName(), st->sha(), this, NULL); // non blocking
 
 	if (curAnn) {
 		// call seekPosition() while loading the file so to shadow the compute time
-		if (ss.hasSelectedText)
-			ss.isValid = (annotateObj->seekPosition(ss.paraFrom, ss.paraTo,
-			              st->sha(false), st->sha()));
-		else
-			ss.isValid = (annotateObj->seekPosition(ss.topPara, ss.topPara,
-			              st->sha(false), st->sha()));
+		int from = ss.hasSelectedText ? ss.paraFrom : ss.topPara;
+		int to = ss.hasSelectedText ? ss.paraTo : ss.topPara;
+		ss.isValid = (annotateObj->seekPosition(from, to, st->sha(false), st->sha(true)));
 	} else
 		ss.isValid = false;
 }
 
-void FileContent::startAnnotate() {
+void FileContent::startAnnotate(FileHistory* fh) {
 
-	annotateObj = git->startAnnotate(d); // non blocking
+	annotateObj = git->startAnnotate(fh, d); // non blocking
 }
 
 uint FileContent::annotateLength(const FileAnnotation* annFile) {
@@ -159,10 +200,13 @@ bool FileContent::getRange(SCRef sha, RangeInfo* r) {
 
 void FileContent::goToAnnotation(int revId) {
 
-	if (!isAnnotationAppended || !curAnn || revId == 0)
+	if (   !isAnnotationAppended
+	    || !curAnn
+	    || (revId == 0)
+	    || (ft->textFormat() == Qt::RichText)) // setSelection() fails in this case
 		return;
 
-	const QString firstLine(QString::number(revId) + ".");
+	const QString firstLine = QString::number(revId) + ".";
 	int idx = 0;
 	QStringList::const_iterator it(curAnn->lines.constBegin());
 
@@ -178,7 +222,9 @@ void FileContent::goToAnnotation(int revId) {
 
 bool FileContent::goToRangeStart() {
 
-	if (!isRangeFilterActive || rangeInfo->start == 0 || !curAnn)
+	if (   !isRangeFilterActive
+	    || !curAnn
+	    || (rangeInfo->start == 0))
 		return false;
 
 	// scroll the viewport so that range is at top
@@ -193,25 +239,26 @@ void FileContent::copySelection() {
 	if (!ft->hasSelectedText())
 		return;
 
-	int headLen = annoLen + MAX_LINE_NUM + 1; // 1 is for space after line number
-	if (!isShowAnnotate)
-		headLen -= annoLen;
+	int headLen = isAnnotationAppended ? annoLen + MAX_LINE_NUM : MAX_LINE_NUM;
+	headLen++; // to count the space after line number
+
+	TextFormat tf = ft->textFormat();
+	ft->setTextFormat(Qt::PlainText); // we want text without formatting tags, this
+	QString sel(ft->selectedText());  // trick does not work with getSelection()
+	ft->setTextFormat(tf);
 
 	int indexFrom, dummy;
 	ft->getSelection(&dummy, &indexFrom, &dummy, &dummy);
-
-	QString sel(ft->selectedText());
 	QClipboard* cb = QApplication::clipboard();
 
-	// check if the selection starts in the middle of an header
-	if (indexFrom < headLen) {
+	if (indexFrom < headLen && (tf != Qt::RichText)) {
 		sel.remove(0, headLen - indexFrom);
 		if (sel.isEmpty()) { // an header part, like the author name, was selected
 			cb->setText(ft->selectedText(), QClipboard::Clipboard);
 			return;
 		}
 	}
-	QRegExp re(QRegExp("\n.{0," + QString::number(headLen) + "}"));
+	QRegExp re("\n.{0," + QString::number(headLen) + "}");
 	sel.replace(re, "\n");
 	cb->setText(sel, QClipboard::Clipboard);
 }
@@ -226,7 +273,7 @@ bool FileContent::rangeFilter(bool b) {
 			return false;
 		}
 		int indexFrom, paraFrom, paraTo, dummy;
-		ft->getSelection(&paraFrom, &indexFrom, &paraTo, &dummy);
+		ft->getSelection(&paraFrom, &indexFrom, &paraTo, &dummy); // does not work with RichText
 
 		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 		EM_PROCESS_EVENTS_NO_INPUT;
@@ -256,15 +303,18 @@ bool FileContent::rangeFilter(bool b) {
 
 bool FileContent::lookupAnnotation() {
 
-	if (st->sha().isEmpty() || st->fileName().isEmpty() ||
-		!isAnnotateAvailable || !annotateObj)
+	if (    st->sha().isEmpty()
+	    ||  st->fileName().isEmpty()
+	    || !isAnnotationAvailable
+	    || !annotateObj)
 		return false;
 
 	curAnn = git->lookupAnnotation(annotateObj, st->fileName(), st->sha());
 
-	if (curAnn)
+	if (curAnn) {
 		annoLen = annotateLength(curAnn);
-	else {
+		curAnnIt = curAnn->lines.constBegin();
+	} else {
 		dbp("ASSERT in lookupAnnotation: no annotation for %1", st->fileName());
 		clearAnnotate();
 	}
@@ -273,13 +323,17 @@ bool FileContent::lookupAnnotation() {
 
 void FileContent::saveScreenState() {
 
-	ss.isValid = true;
+	// getSelection() does not work with RichText
+	ss.isValid = (ft->textFormat() != Qt::RichText);
+	if (!ss.isValid)
+		return;
+
 	ss.topPara = ft->paragraphAt(QPoint(ft->contentsX(), ft->contentsY()));
 	ss.hasSelectedText = ft->hasSelectedText();
 	if (ss.hasSelectedText) {
 		ft->getSelection(&ss.paraFrom, &ss.indexFrom, &ss.paraTo, &ss.indexTo);
 		ss.annoLen = annoLen;
-		ss.isShowAnnotate = isShowAnnotate;
+		ss.isAnnotationAppended = isAnnotationAppended;
 	}
 }
 
@@ -290,12 +344,12 @@ void FileContent::restoreScreenState() {
 
 	if (ss.hasSelectedText) {
 		// index without previous annotation
-		ss.indexFrom -= (ss.isShowAnnotate) ? ss.annoLen : 0;
-		ss.indexTo -= (ss.isShowAnnotate) ? ss.annoLen : 0;
+		ss.indexFrom -= (ss.isAnnotationAppended) ? ss.annoLen : 0;
+		ss.indexTo -= (ss.isAnnotationAppended) ? ss.annoLen : 0;
 
 		// index with current annotation
-		ss.indexFrom += (isShowAnnotate) ? annoLen : 0;
-		ss.indexTo += (isShowAnnotate) ? annoLen : 0;
+		ss.indexFrom += (isAnnotationAppended) ? annoLen : 0;
+		ss.indexTo += (isAnnotationAppended) ? annoLen : 0;
 		ss.indexFrom = QMAX(ss.indexFrom, 0);
 		ss.indexTo = QMAX(ss.indexTo, 0);
 		ft->setSelection(ss.paraFrom, ss.indexFrom, ss.paraTo, ss.indexTo); // slow
@@ -304,78 +358,13 @@ void FileContent::restoreScreenState() {
 		int t = ft->paragraphRect(ss.topPara).bottom(); // slow for big files
 		ft->setContentsPos(0, t);
 	}
-	ss.isShowAnnotate = isShowAnnotate; // leave ss in a consistent state
+	// leave ss in a consistent state with current screen settings
+	ss.isAnnotationAppended = isAnnotationAppended;
 	ss.annoLen = annoLen;
 }
 
-void FileContent::addAnnotation() {
-// annotation was not ready at the beginning of file content reading,
-// so feed again on_procDataReady() to add also annotation data
-
-	if (!isFileAvailable || !isAnnotateAvailable ||
-		(isAnnotationAppended == isShowAnnotate))
-		return;
-
-	const QString tmp(fileData);
-
-	isShowAnnotate = !isShowAnnotate;
-	saveScreenState();
-	isShowAnnotate = !isShowAnnotate;
-
-	clearText(false);
-	on_procDataReady(tmp);
-	on_eof(false); // will call restoreScreenState()
-}
-
 // ************************************ SLOTS ********************************
 
-void FileContent::setShowAnnotate(bool b)  {
-
-	isShowAnnotate = b;
-	addAnnotation();
-}
-
-void FileContent::on_annotateReady(Annotate* readyAnn, const QString& fileName,
-						   bool ok, const QString& msg) {
-	if (!ok) {
-		d->m()->statusBar()->message("Sorry, annotation not available for this file.");
-		return;
-	}
-	if (readyAnn != annotateObj) {
-		dbs("ASSERT in on_annotateReady: arrived different annotation from expected");
-		return;
-	}
-	if (st->fileName() != fileName) {
-		dbs("ASSERT arrived annotation of a different file");
-		return;
-	}
-	d->m()->statusBar()->message(msg, 7000);
-
-	isAnnotateAvailable = true;
-	lookupAnnotation();
-	emit annotationAvailable(true);
-}
-
-void FileContent::on_eof(bool emitSignal) {
-
-	if (!pending.isEmpty())
-		on_procDataReady(QString("\n")); // No newline at end of file, fake one
-
-	if (!pending.isEmpty())
-		dbs("ASSERT in FileContent::on_eof(): there is still pending data");
-
-	pending = "";
-	isFileAvailable = true;
-	line = 1;
-
-	if (ss.isValid)
-		restoreScreenState(); // could be slow for big files
-
-	QApplication::restoreOverrideCursor();
-	if (emitSignal)
-		emit fileAvailable(true);
-}
-
 void FileContent::on_doubleClicked(int para, int) {
 
 	QString id(ft->text(para));
@@ -383,69 +372,122 @@ void FileContent::on_doubleClicked(int para, int) {
 	emit revIdSelected(id.toInt());
 }
 
+void FileContent::on_annotateReady(Annotate* readyAnn, const QString& fileName,
+                                   bool ok, const QString& msg) {
+
+	if (readyAnn != annotateObj) // Git::annotateReady() is sent to all receivers
+		return;
+
+	if (!ok) {
+		d->m()->statusBar()->message("Sorry, annotation not available for this file.");
+		return;
+	}
+	if (st->fileName() != fileName) {
+		dbp("ASSERT arrived annotation of wrong file <%1>", fileName);
+		return;
+	}
+	d->m()->statusBar()->message(msg, 7000);
+
+	isAnnotationAvailable = true;
+	if (lookupAnnotation())
+		emit annotationAvailable(true);
+}
+
 void FileContent::on_procDataReady(const QString& fileChunk) {
-// called to add data to file view
 
-	if (fileData.isEmpty()) // optimize common case, the whole file in one big chunk
-		fileData = fileChunk;
-	else
-		fileData.append(fileChunk);
+	processData(fileChunk);
+}
 
-	if (pending.isEmpty()) // optimize common case, pending is almost always empty
-		pending = fileChunk;
-	else
-		pending.append(fileChunk); // add to previous half lines
+void FileContent::on_eof(bool emitSignal) {
 
-	int nextEOL = pending.find('\n');
-	int lastEOL = -1;
-	int X = ft->contentsX();
-	int Y = ft->contentsY();
-	uint annLineNum = 0;
+	if (!fileRowData.endsWith("\n"))
+		processData(QString("\n")); // fake a trailing new line
 
-	if (ft->text().isEmpty() && isShowAnnotate) { // at the beginning only
+	ft->setText(fileProcessedData); // much faster then ft->append()
+	isFileAvailable = true;
+	if (ss.isValid)
+		restoreScreenState(); // could be slow for big files
+
+	if (emitSignal)
+		emit fileAvailable(true);
+}
+
+uint FileContent::processData(const QString& fileChunk) {
+
+	if (fileChunk.isEmpty())
+		return 0;
+
+	fileRowData.append(fileChunk);
+
+	QStringList sl = QStringList::split('\n', fileChunk, true); // allowEmptyEntries
+
+	if (!halfLine.isEmpty())
+		sl.first().prepend(halfLine);	// !fileChunk.isEmpty() => sl.first() is defined
+
+	halfLine = sl.last(); // empty if fileChunk ends with '\n'
+	sl.pop_back();
+
+	if (fileProcessedData.isEmpty() && isShowAnnotate) { // one shot at the beginning
 
 		// check if it is possible to add annotation while appending data
 		// if annotation is not available we will defer this in a separated
-		// step, calling addAnnotation() at proper time
+		// step, calling setShowAnnotate() at proper time
 		isAnnotationAppended = (curAnn != NULL);
 	}
-	QStringList::const_iterator it;
-	if (isAnnotationAppended && curAnn) { // curAnn could disappear while loading
-		it = curAnn->lines.at(line - 1);
-		annLineNum = curAnn->lines.count();
-	}
-	QString newStuff;
-	while (nextEOL != -1) {
+	bool isHtmlHeader = (isHtmlSource && curLine == 1);
+	bool isHtmlFirstContentLine = false;
 
-		// first add annotation...
-		if (isAnnotationAppended && curAnn) {
+	loopList(it, sl) {
 
-			if (line > annLineNum) {
-				dbs("ASSERT bad annotate in FileContent::on_procDataReady");
-				clearAnnotate();
-				return;
+		if (isHtmlHeader) {
+			if ((*it).startsWith(HTML_FILE_START)) {
+				isHtmlHeader = false;
+				isHtmlFirstContentLine = true;
+			} else {
+				fileProcessedData.append(*it).append('\n');
+				continue;
 			}
-			newStuff.append(QString("%1").arg(*it, -annoLen)); // left-aligned author
-			++it;
 		}
-		// ...then line number...
-		newStuff.append(QString("%1 ").arg(line, MAX_LINE_NUM));
+		// add HTML page header
+		if (isHtmlFirstContentLine)
+			fileProcessedData.append(HTML_FILE_START);
 
-		// ...then content
-		newStuff.append(pending.mid(lastEOL + 1, nextEOL - lastEOL));
-		lastEOL = nextEOL;
-		nextEOL = pending.find('\n', lastEOL + 1);
-		++line;
+		// add color tag head
+		if (isHtmlSource)
+			fileProcessedData.append(HTML_HEAD);
+
+		// add annotation
+		if (isAnnotationAppended && curAnn) { // curAnn can change while loading
+
+			if (curAnnIt == curAnn->lines.constEnd()) {
+
+				if (isHtmlSource && (*it) == HTML_FILE_END) {
+					fileProcessedData.append(HTML_TAIL).append(*it).append('\n');
+					continue;
+				} else {
+					dbs("ASSERT in FileContent::processData: bad annotate");
+					clearAnnotate();
+					return 0;
+				}
+			}
+			fileProcessedData.append((*curAnnIt).leftJustify(annoLen));
+			++curAnnIt;
+		}
+		// add line number
+		fileProcessedData.append(QString("%1 ").arg(curLine++, MAX_LINE_NUM));
+
+		// add color tag tail
+		if (isHtmlSource)
+			fileProcessedData.append(HTML_TAIL);
+
+		// finally add content
+		if (isHtmlFirstContentLine) {
+			isHtmlFirstContentLine = false;
+			fileProcessedData.append((*it).section(HTML_FILE_START, 1));
+		} else
+			fileProcessedData.append(*it);
+
+		fileProcessedData.append('\n'); // removed by QStringList::split()
 	}
-	if (lastEOL != -1)
-		pending.remove(0, lastEOL + 1);
-
-	// check for partial paragraphs. In patch loading sometimes happen.
-	// if it is the case also here we may have to add some more logic
-	if (!ft->text().isEmpty() && !ft->text().endsWith("\n"))
-		dbp("ASSERT partial file paragraph arrived <%1>", newStuff.left(30));
-
-	newStuff.prepend(ft->text());
-	ft->setText(newStuff); // much faster then ft->append()
-	ft->setContentsPos(X, Y);
+	return sl.count();
 }
