@@ -29,13 +29,12 @@ void ListBoxFileItem::paint(QPainter* p) {
 
 ListBoxFiles::ListBoxFiles(Domain* dm, Git* g, QListBox* l) :
                            QObject(dm), d(dm), git(g), lb(l) {
-
 	st = &(d->st);
 
 	connect(lb, SIGNAL(currentChanged(QListBoxItem*)),
-		this, SLOT(on_currentChanged(QListBoxItem*)));
+	        this, SLOT(on_currentChanged(QListBoxItem*)));
 	connect(lb, SIGNAL(contextMenuRequested(QListBoxItem*,const QPoint&)),
-		this, SLOT(on_contextMenuRequested(QListBoxItem*)));
+	        this, SLOT(on_contextMenuRequested(QListBoxItem*)));
 	connect(lb, SIGNAL(mouseButtonPressed(int,QListBoxItem*,const QPoint&)),
 	        this, SLOT(on_mouseButtonPressed(int,QListBoxItem*,const QPoint&)));
 	connect(lb, SIGNAL(onItem(QListBoxItem*)),this,SLOT(on_onItem(QListBoxItem*)));
@@ -48,11 +47,6 @@ void ListBoxFiles::clear() {
 	lb->clear();
 }
 
-bool ListBoxFiles::isMergeHeader(QListBoxItem* c) const {
-
-	return ((lb->index(c) == 0) && st->isMerge());
-}
-
 void ListBoxFiles::update(const RevFile* files) {
 
 	if (!st->diffToSha().isEmpty()) {
@@ -64,7 +58,10 @@ void ListBoxFiles::update(const RevFile* files) {
 	if (files)
 		insertFiles(files);
 
-	if (lb->currentText() && (lb->currentText() == st->fileName())) {
+	QString fileName(lb->currentText());
+	git->removeExtraFileInfo(&fileName); // could be a renamed/copied file
+
+	if (fileName && (fileName == st->fileName())) {
 		lb->setSelected(lb->currentItem(), st->selectItem()); // just a refresh
 		return;
 	}
@@ -73,7 +70,13 @@ void ListBoxFiles::update(const RevFile* files) {
 	if (st->fileName().isEmpty())
 		return;
 
-	QListBoxItem* c = lb->findItem(st->fileName(), Qt::ExactMatch); // could be null
+	QListBoxItem* c = lb->findItem(st->fileName(), Qt::ExactMatch);
+	if (c == NULL) { // could be a renamed/copied file, try harder
+
+		fileName = st->fileName();
+		git->addExtraFileInfo(&fileName, st->sha(), st->diffToSha(), st->allMergeFiles());
+		c = lb->findItem(fileName, Qt::ExactMatch);
+	}
 	lb->setSelected(c, st->selectItem()); // calls current changed
 }
 
@@ -86,8 +89,8 @@ void ListBoxFiles::insertFiles(const RevFile* files) {
 
 	if (st->isMerge()) {
 		QString header((st->allMergeFiles()) ?
-				"Click to view only interesting files" :
-				"Click to view all merge files");
+		                "Click to view only interesting files" :
+		                "Click to view all merge files");
 		new ListBoxFileItem(lb, header, Qt::blue);
 	}
 	if (files->names.empty())
@@ -96,7 +99,7 @@ void ListBoxFiles::insertFiles(const RevFile* files) {
 	int prevPar = files->mergeParent[0];
 	for (uint i = 0; i < files->names.count(); ++i) {
 
-		SCRef status(files->status[i].right(1));
+		QChar status(files->getStatus(i));
 		if (status == QGit::UNKNOWN)
 			continue;
 
@@ -106,11 +109,26 @@ void ListBoxFiles::insertFiles(const RevFile* files) {
 			new ListBoxFileItem(lb, "", clr);
 			new ListBoxFileItem(lb, "", clr);
 		}
-		if (status == QGit::NEW)
-			clr = Qt::darkGreen;
-		if (status == QGit::DELETED)
-			clr = Qt::red;
+		QString extSt(files->getExtendedStatus(i));
+		if (extSt.isEmpty()) {
+			if (status == QGit::MODIFIED)
+				; // common case
+			else if (status == QGit::NEW)
+				clr = Qt::darkGreen;
+			else if (status == QGit::DELETED)
+				clr = Qt::red;
+		} else {
+			clr = Qt::darkBlue;
+			// in case of rename deleted file is not shown and...
+			if (status == QGit::DELETED)
+				continue;
 
+			// ...new file is shown with extended info
+			if (status == QGit::NEW) {
+				new ListBoxFileItem(lb, extSt, clr);
+				continue;
+			}
+		}
 		new ListBoxFileItem(lb, git->filePath(*files, i), clr);
 	}
 }
@@ -119,20 +137,41 @@ void ListBoxFiles::on_currentChanged(QListBoxItem* item) {
 
 	if (item) {
 		if (st->isMerge() && item == lb->firstItem()) { // header clicked
+
+			// In a listbox without current item, as soon as the box
+			// gains focus the first item becomes the current item
+			// and a spurious currentChanged() signal is sent.
+			// In case of a merge the signal arrives here and fakes
+			// the user clicking on the header.
+			//
+			// The problem arise when user clicks on a merge header,
+			// then list box gains focus and current item becomes null
+			// because the content of the list is cleared and updated.
+			//
+			// If now tab is changed list box loose the focus and,
+			// upon changing back again the tab the signal triggers
+			// because Qt gives back the focus to the listbox.
+			//
+			// The workaround here is to give the focus away as soon
+			// as the user clicks on the merge header. Note that a
+			// lb->clearFocus() is not enough, we really need to
+			// reassign the focus to someone else.
+			d->tab()->setFocus();
+
 			st->setAllMergeFiles(!st->allMergeFiles());
 			st->setSelectItem(true);
 			UPDATE_DOMAIN(d);
 			return;
 		}
+		QString fileName(item->text());
+		git->removeExtraFileInfo(&fileName);
 		// if we are called by updateFileList() fileName is already updated
-		if (st->fileName() != item->text()) { // avoid loops
-			st->setFileName(item->text());
+		if (st->fileName() != fileName) { // avoid loops
+			st->setFileName(fileName);
 			st->setSelectItem(true);
 			UPDATE_DOMAIN(d);
 		}
 	}
-	if (!isMergeHeader(item))
-		emit fileSelected(item ? item->text() : "");
 }
 
 void ListBoxFiles::on_contextMenuRequested(QListBoxItem* item) {
