@@ -32,7 +32,6 @@ public:
 	void update(const RevFile* files);
 
 signals:
-	void fileSelected(const QString&);
 	void contextMenu(const QString&, int);
 
 private slots:
@@ -46,7 +45,6 @@ private slots:
 private:
 	void insertFiles(const RevFile* files);
 	void mouseMoved();
-	bool isMergeHeader(QListBoxItem* c) const;
 
 	Domain* d;
 	Git* git;
