@@ -7,7 +7,6 @@
 
 */
 #include <qtextedit.h>
-#include <qsyntaxhighlighter.h>
 #include <qlistview.h>
 #include <qspinbox.h>
 #include <qlineedit.h>
@@ -17,7 +16,6 @@
 #include <qlabel.h>
 #include <qsplitter.h>
 #include <qapplication.h>
-#include <qeventloop.h>
 #include <qcursor.h>
 #include <qregexp.h>
 #include <qclipboard.h>
@@ -27,34 +25,42 @@
 #include "git.h"
 #include "annotate.h"
 #include "listview.h"
-#include "fileview.h"
 #include "filecontent.h"
 #include "filebase.h"
+#include "fileview.h"
 
 #define MAX_LINE_NUM 5
 
-FileView::FileView(MainImpl* mi, Git* g, TabFile* t) :
-                   Domain(mi, g), fileTab(t) {
+FileView::FileView(MainImpl* mi, Git* g) : Domain(mi, g) {
 
 	EM_INIT(exDtorCalled, "Closing file viewer");
 
-	histListView = new ListView(this, git, fileTab->histListView, true);
+	fileTab = new TabFile(m());
+	m()->tabWdg->addTab(fileTab, "File");
+	tabPosition = m()->tabWdg->count() - 1;
+
+	fh = new FileHistory();
+	histListView = new ListView(this, git, fileTab->histListView, fh);
 	textEditFile = new FileContent(this, git, fileTab->textEditFile);
 
 	// cannot be set directly in the .ui file
 	fileTab->spinBoxRevision->setSpecialValueText(" ");
 
-	connect(git, SIGNAL(histLoadCompleted()),this, SLOT(on_histLoadCompleted()));
-	connect(git, SIGNAL(newHistRevsAdded(const QValueVector<QString>&)),
-	        histListView, SLOT(on_newRevsAdded(const QValueVector<QString>&)));
+	connect(git, SIGNAL(loadCompleted(const FileHistory*, const QString&)),
+	        this, SLOT(on_loadCompleted(const FileHistory*, const QString&)));
+
+	connect(git, SIGNAL(newRevsAdded(const FileHistory*, const QValueVector<QString>&)),
+	histListView, SLOT(on_newRevsAdded(const FileHistory*, const QValueVector<QString>&)));
 
 	connect(histListView, SIGNAL(contextMenu(const QString&, int)),
-	        d, SLOT(on_contextMenu(const QString&, int)));
+	        this, SLOT(on_contextMenu(const QString&, int)));
 
 	connect(textEditFile, SIGNAL(annotationAvailable(bool)),
 	        this, SLOT(on_annotationAvailable(bool)));
+
 	connect(textEditFile, SIGNAL(fileAvailable(bool)),
 	        this, SLOT(on_fileAvailable(bool)));
+
 	connect(textEditFile, SIGNAL(revIdSelected(int)),
 	        this, SLOT(on_revIdSelected(int)));
 
@@ -68,21 +74,29 @@ FileView::FileView(MainImpl* mi, Git* g, TabFile* t) :
 	        this, SLOT(on_toolButtonFindAnnotate_toggled(bool)));
 
     	connect(fileTab->toolButtonRangeFilter, SIGNAL(toggled(bool)),
-	        this, SLOT(toolButtonRangeFilter_toggled(bool)));
+	        this, SLOT(on_toolButtonRangeFilter_toggled(bool)));
 
 	connect(fileTab->toolButtonPin, SIGNAL(toggled(bool)),
 	        this, SLOT(on_toolButtonPin_toggled(bool)));
 
+	connect(fileTab->toolButtonHighlightText, SIGNAL(toggled(bool)),
+	        this, SLOT(on_toolButtonHighlightText_toggled(bool)));
+
 	connect(fileTab->spinBoxRevision, SIGNAL(valueChanged(int)),
 	        this, SLOT(on_spinBoxRevision_valueChanged(int)));
 }
 
 FileView::~FileView() {
 
+	if (!parent())
+		return;
+
 	EM_RAISE(exDtorCalled);
 
-	delete histListView;
-	delete textEditFile;
+	delete textEditFile; // delete now, without waiting base QObject d'tor
+	delete histListView; // to do the job for us, we need them out before
+	delete fh;           // Domain d'tor is called.
+	delete fileTab;
 
 	m()->statusBar()->clear(); // cleanup any pending progress info
 	QApplication::restoreOverrideCursor();
@@ -98,6 +112,7 @@ void FileView::clear(bool complete) {
 	histListView->clear();
 	textEditFile->clear();
 
+	fileTab->toolButtonHighlightText->setEnabled(false);
 	fileTab->toolButtonPin->setEnabled(false);
 	fileTab->toolButtonPin->setOn(false); // should not trigger an update
 	fileTab->spinBoxRevision->setEnabled(false);
@@ -119,7 +134,7 @@ void FileView::updateSpinBoxValue() {
 		return;
 
 	int id = item->text(QGit::ANN_ID_COL).toInt();
-	fileTab->spinBoxRevision->setValue(id); // triggers on_spinBoxRevision_valueChanged
+	fileTab->spinBoxRevision->setValue(id); // triggers on_spinBoxRevision_valueChanged()
 }
 
 bool FileView::doUpdate() {
@@ -132,7 +147,7 @@ bool FileView::doUpdate() {
 
 		if (st.fileName(true) != st.fileName(false)) {
 
-			clear(false); // blocking
+			clear(false);
 
 			m()->tabWdg->changeTab(fileTab, st.fileName());
 
@@ -141,13 +156,14 @@ bool FileView::doUpdate() {
 
 			m()->statusBar()->message("Retrieving history of '" +
 			                          st.fileName() + "'...");
+			fh->reset(st.fileName());
+			git->startFileHistory(fh);
 
-			git->getFileHistory(st.fileName()); // non blocking
 		} else {
 			bool found = histListView->update();
 			if (found) {
 				updateSpinBoxValue();
-				m()->statusBar()->message(git->getRevInfo(st.sha(), true));
+				m()->statusBar()->message(git->getRevInfo(st.sha(), fh));
 			}
 		}
 		if (!fileTab->toolButtonPin->isOn())
@@ -180,9 +196,8 @@ void FileView::on_toolButtonCopy_clicked() {
 
 void FileView::on_toolButtonShowAnnotate_toggled(bool b) {
 
-	textEditFile->setShowAnnotate(b);
-
 	fileTab->toolButtonFindAnnotate->setEnabled(b);
+	textEditFile->setShowAnnotate(b);
 
 	if (b && fileTab->toolButtonFindAnnotate->isOn())
 		goToCurrentAnnotation();
@@ -205,22 +220,50 @@ void FileView::on_toolButtonPin_toggled(bool b) {
 	}
 }
 
+void FileView::on_toolButtonRangeFilter_toggled(bool b) {
+
+	fileTab->toolButtonHighlightText->setEnabled(!b);
+	if (b) {
+		if (!textEditFile->annotateAvailable()) {
+			dbs("ASSERT in on_toolButtonRangeFilter_toggled: annotate not available");
+			return;
+		}
+ 		if (!fileTab->textEditFile->hasSelectedText()) {
+ 			m()->statusBar()->message("Please select some text to activate range filter");
+ 			return;
+ 		}
+ 	}
+	bool rangeFilterActive = textEditFile->rangeFilter(b);
+	filterOnRange(rangeFilterActive);
+}
+
+void FileView::on_toolButtonHighlightText_toggled(bool b) {
+
+	fileTab->toolButtonRangeFilter->setEnabled(!b);
+	textEditFile->setHighlightSource(b);
+}
+
 void FileView::on_spinBoxRevision_valueChanged(int id) {
 
 	if (id != fileTab->spinBoxRevision->minValue()) {
 
 		SCRef selRev(histListView->getSha(id));
+
 		if (st.sha() != selRev) { // to avoid looping
 			st.setSha(selRev);
 			st.setSelectItem(true);
-			UPDATE_DOMAIN(d);
+			UPDATE();
 		}
 	}
 }
 
-void FileView::on_histLoadCompleted() {
+void FileView::on_loadCompleted(const FileHistory* f, const QString&) {
 
 	QApplication::restoreOverrideCursor();
+
+	if (f != fh)
+		return;
+
 	histListView->updateIdValues();
 	int maxId = fileTab->histListView->childCount();
 	if (maxId == 0) {
@@ -231,29 +274,30 @@ void FileView::on_histLoadCompleted() {
 	fileTab->toolButtonPin->setEnabled(true);
 	fileTab->spinBoxRevision->setEnabled(true);
 
-	UPDATE_DOMAIN(d);
+	UPDATE();
 
 	updateProgressBar(0);
-	textEditFile->startAnnotate();
+	textEditFile->startAnnotate(fh);
 }
 
-void FileView::addAnnotation() {
+void FileView::showAnnotation() {
 
 	if (  !fileTab->toolButtonPin->isOn()
 	    && fileTab->toolButtonShowAnnotate->isEnabled()
 	    && fileTab->toolButtonShowAnnotate->isOn()) {
 
-		textEditFile->addAnnotation();
+		textEditFile->setShowAnnotate(true);
+
+		if (   fileTab->toolButtonFindAnnotate->isEnabled()
+		    && fileTab->toolButtonFindAnnotate->isOn())
 
-		if (fileTab->toolButtonFindAnnotate->isEnabled() &&
-		    fileTab->toolButtonFindAnnotate->isOn())
 			goToCurrentAnnotation();
 	}
 }
 
 void FileView::on_annotationAvailable(bool b) {
 
-	fileTab->toolButtonRangeFilter->setEnabled(b);
+	fileTab->toolButtonRangeFilter->setEnabled(b && !fileTab->toolButtonHighlightText->isOn());
 
 	if (!b && fileTab->toolButtonRangeFilter->isOn())
 		fileTab->toolButtonRangeFilter->toggle();
@@ -262,45 +306,28 @@ void FileView::on_annotationAvailable(bool b) {
 	fileTab->toolButtonFindAnnotate->setEnabled(b && fileTab->toolButtonShowAnnotate->isOn());
 
 	if (b)
-		addAnnotation(); // in case annotation arrived after file
+		showAnnotation(); // in case annotation got ready after file
 }
 
 void FileView::on_fileAvailable(bool b) {
 
-	if (b) { // avoid flickering when disable only temporary
+	if (b) {
 		fileTab->toolButtonCopy->setEnabled(b);
-
+		fileTab->toolButtonHighlightText->setEnabled(!fileTab->toolButtonRangeFilter->isOn()
+		                                             && git->isTextHighlighter());
 		// code range is indipendent from annotation
 		if (fileTab->toolButtonRangeFilter->isOn())
 			textEditFile->goToRangeStart();
 
-		addAnnotation(); // in case file arrived after annotation
-	}
+		showAnnotation(); // in case file got ready after annotation
+	} else
+		; // avoid flickering disabling only temporary
 }
 
 void FileView::on_revIdSelected(int id) {
 
-	if (id != 0) {
-		if (fileTab->spinBoxRevision->isEnabled())
-			fileTab->spinBoxRevision->setValue(id);
-	}
-}
-
-void FileView::toolButtonRangeFilter_toggled(bool b) {
-
-	if (b) {
-		if (!textEditFile->annotateAvailable()) {
-			dbs("ASSERT in toolButtonRangeFilter_toggled: annotate not available");
-			return;
-		}
- 		if (!fileTab->textEditFile->hasSelectedText()) {
- 			m()->statusBar()->message("Please select some text "
-			                          "to activate range filter");
- 			return;
- 		}
- 	}
-	bool rangeFilterActive = textEditFile->rangeFilter(b);
-	filterOnRange(rangeFilterActive);
+	if (id != 0 && fileTab->spinBoxRevision->isEnabled())
+		fileTab->spinBoxRevision->setValue(id);
 }
 
 // ******************************* data events ****************************
