@@ -6,8 +6,6 @@
 	Copyright: See COPYING file that comes with this distribution
 
 */
-#include <unistd.h>    // usleep()
-#include <stdlib.h>    // used by getenv(), setenv()
 #include <sys/types.h> // used by chmod()
 #include <sys/stat.h>  // used by chmod()
 #include <qapplication.h>
@@ -38,44 +36,35 @@ Git::Git(QWidget* p, const char* n) : QObject(p, n), par(p) {
 	errorReportingEnabled = true; // report errors if run() fails
 	runningProcesses = 0;
 
+	revs.resize(MAX_DICT_SIZE);
+	revsFiles.resize(MAX_DICT_SIZE);
 	revs.setAutoDelete(true);
-	revs.clear();
-	revs.resize(100003); // must be a prime number see QDict docs
+	revsFiles.setAutoDelete(true);
 
 	cache = new Cache(this);
 
-	// get git bins path
-	QString execPath;
-	if (run("git --exec-path", &execPath)) { // workDir is empty now
-		execPath = execPath.section('\n', 0, 0).append(':');
-		QString curPath(getenv("PATH"));
-		if (!curPath.contains(execPath)) {
-			curPath.prepend(execPath);
-			setenv("PATH", curPath.latin1(), 1);
-		}
-	}
 	// control git version
-	QString gitVersion;
-	if (run("git --version", &gitVersion)) {
+	QString version;
+	if (run("git --version", &version)) {
 
-		gitVersion = gitVersion.section(' ', -1, -1).section('.', 0, 2);
-		if (gitVersion < GIT_VERSION) {
+		version = version.section(' ', -1, -1).section('.', 0, 2);
+		if (version < GIT_VERSION) {
 
 			// simply send information, the 'not compatible version'
 			// policy should be implemented upstream
-			const QString cmd("Current git version is " + gitVersion +
-				" but is required " + GIT_VERSION + " or better");
+			const QString cmd("Current git version is " + version +
+			      " but is required " + GIT_VERSION + " or better");
 			const QString errorDesc("Your installed git is too old."
-				"\nPlease upgrade to avoid possible misbehaviours.");
+			      "\nPlease upgrade to avoid possible misbehaviours.");
 			MainExecErrorEvent* e = new MainExecErrorEvent(cmd, errorDesc);
 			QApplication::postEvent(p, e);
 		}
 	}
-}
-
-Git::~Git() {
-
-	clearRevs();
+	errorReportingEnabled = false;
+	isTextHighlighterFound = run("source-highlight -V", &version);
+	if (isTextHighlighterFound)
+		dbp("Found %1", version.section('\n', 0, 0));
+	errorReportingEnabled = true;
 }
 
 void Git::setTextCodec(QTextCodec* tc) {
@@ -88,7 +77,7 @@ void Git::setTextCodec(QTextCodec* tc) {
 	if (mimeName == "Big5-HKSCS")
 		mimeName = "Big5";
 
-	run("git-repo-config i18n.commitencoding " + mimeName);
+	run("git repo-config i18n.commitencoding " + mimeName);
 }
 
 QTextCodec* Git::getTextCodec(bool* isGitArchive) {
@@ -98,7 +87,7 @@ QTextCodec* Git::getTextCodec(bool* isGitArchive) {
 		return NULL;
 
 	QString runOutput;
-	if (!run("git-repo-config --get i18n.commitencoding", &runOutput))
+	if (!run("git repo-config --get i18n.commitencoding", &runOutput))
 		return NULL;
 
 	if (runOutput.isEmpty()) // git docs says default is utf-8
@@ -114,9 +103,9 @@ const QString Git::getLocalDate(SCRef gitDate) {
 	return d.toString(Qt::LocalDate);
 }
 
-const QString Git::getRevInfo(SCRef sha, bool isHistory) {
+const QString Git::getRevInfo(SCRef sha, FileHistory* fh) {
 
-	const Rev* c = revLookup(sha, isHistory);
+	const Rev* c = revLookup(sha, fh);
 	if (c == NULL)
 		return "";
 
@@ -157,7 +146,7 @@ const QString Git::getRefSha(SCRef refName) {
 
 	// if a ref was not found perhaps is an abbreviated form
 	QString runOutput;
-	if (!run("git-rev-parse " + refName, &runOutput))
+	if (!run("git rev-parse " + refName, &runOutput))
 		return "";
 
 	return runOutput.stripWhiteSpace();
@@ -176,10 +165,55 @@ bool Git::isStGITStack() { return isStGIT; }
 
 bool Git::isUnapplied(SCRef sha) { return unAppliedSHA.contains(sha); }
 
-Annotate* Git::startAnnotate(QObject* guiObj) { // non blocking
+void Git::addExtraFileInfo(QString* rowName, SCRef sha, SCRef diffToSha, bool allMergeFiles) {
+
+	const RevFile* files;
+	if ((files = getFiles(sha, diffToSha, allMergeFiles)) == NULL)
+		return;
+
+	int idx;
+	if ((idx = findFileIndex(*files, *rowName)) == -1)
+		return;
+
+	QString extSt(files->getExtendedStatus(idx));
+	if (extSt.isEmpty())
+		return;
+
+	*rowName = extSt;
+}
+
+void Git::removeExtraFileInfo(QString* rowName) {
+
+	if (rowName->contains(" --> ")) // return destination file name
+		*rowName = rowName->section(" --> ", 1, 1).section(" (", 0, 0);
+}
+
+void Git::formatPatchFileHeader(QString* rowName, SCRef sha, SCRef diffToSha,
+                                bool combined, bool allMergeFiles) {
+
+	// return git patch format: diff --git a/<origFile> b/<destFile>
+	QString prefix((combined) ? "diff --combined " : "diff --git a/");
+
+	if (combined) { // TODO rename/copy still not supported in this case
+		*rowName = prefix + *rowName;
+		return;
+	}
+	// let's see if it's a rename/copy...
+	addExtraFileInfo(rowName, sha, diffToSha, allMergeFiles);
+
+	if (rowName->contains(" --> ")) { // ...it is!
+
+		SCRef destFile(rowName->section(" --> ", 1, 1).section(" (", 0, 0));
+		SCRef origFile(rowName->section(" --> ", 0, 0));
+		*rowName = prefix + origFile + " b/" + destFile;
+	} else
+		*rowName = prefix + *rowName + " b/" + *rowName;
+}
+
+Annotate* Git::startAnnotate(FileHistory* fh, QObject* guiObj) { // non blocking
 
 	Annotate* ann = new Annotate(this, guiObj);
-	ann->start(histFileName, histRevOrder); // non blocking call
+	ann->start(fh); // non blocking call
 	return ann;
 }
 
@@ -197,12 +231,10 @@ void Git::annotateExited(Annotate* ann) {
 		return; // do not emit anything
 	}
 	// Annotate object will be deleted by cancelAnnotate()
-	if (ann->isValid()) {
-		const QString msg(QString("Annotated %1 files in %2 ms")
-				.arg(histRevOrder.count()).arg(ann->elapsed()));
-		emit annotateReady(ann, ann->file(), ann->isValid(), msg);
-	} else
-		emit annotateReady(ann, ann->file(), ann->isValid(), "");
+	const QString msg = QString("Annotated %1 files in %2 ms")
+	                   .arg(ann->count()).arg(ann->elapsed());
+
+	emit annotateReady(ann, ann->file(), ann->isValid(), msg);
 }
 
 const FileAnnotation* Git::lookupAnnotation(Annotate* ann, SCRef fileName, SCRef sha) {
@@ -213,15 +245,15 @@ const FileAnnotation* Git::lookupAnnotation(Annotate* ann, SCRef fileName, SCRef
 	return ann->lookupAnnotation(sha, fileName);
 }
 
-void Git::cancelFileHistoryLoading() {
+void Git::cancelDataLoading(const FileHistory* fh) {
 // normally called when closing file viewer
 
-	emit cancelHistLoading(); // non blocking
+	emit cancelLoading(fh); // non blocking
 }
 
-const Rev* Git::revLookup(SCRef sha, bool isHistoryView) {
+const Rev* Git::revLookup(SCRef sha, const FileHistory* fh) {
 
-	const RevMap& r = (isHistoryView) ? histRevs : revs;
+	const RevMap& r = (fh ? fh->revs : revs);
 	return r[sha];
 }
 
@@ -263,7 +295,7 @@ void Git::on_runAsScript_eof() {
 void Git::cancelProcess(MyProcess* p) {
 
 	if (p)
-		p->cancel(); // non blocking call
+		p->on_cancel(); // non blocking call
 }
 
 int Git::findFileIndex(const RevFile& rf, SCRef name) {
@@ -273,7 +305,7 @@ int Git::findFileIndex(const RevFile& rf, SCRef name) {
 
 	int idx = name.findRev('/') + 1;
 	SCRef dr = name.left(idx);
-	SCRef nm = name.right(name.length() - idx);
+	SCRef nm = name.mid(idx);
 
 	for (uint i = 0; i < rf.names.count(); ++i) {
 		if (fileNamesVec[rf.names[i]] == nm && dirNamesVec[rf.dirs[i]] == dr)
@@ -286,25 +318,28 @@ const QString Git::getLaneParent(SCRef fromSHA, uint laneNum) {
 
 	const Rev* rs = revLookup(fromSHA);
 	if (!rs)
-		return QString();
+		return "";
 
 	for (int idx = rs->orderIdx - 1; idx >= 0; idx--) {
 
 		const Rev* r = revLookup(revOrder[idx]);
 		if (laneNum >= r->lanes.count())
-			return QString();
+			return "";
 
 		if (!isFreeLane(r->lanes[laneNum])) {
+
 			int type = r->lanes[laneNum], parNum = 0;
 			while (!isMerge(type) && type != ACTIVE) {
+
 				if (isHead(type))
 					parNum++;
+
 				type = r->lanes[--laneNum];
 			}
 			return r->parent(parNum);
 		}
 	}
-	return QString();
+	return "";
 }
 
 const QStringList Git::getChilds(SCRef parent) {
@@ -333,7 +368,7 @@ const QStringList Git::getChilds(SCRef parent) {
 const QString Git::getShortLog(SCRef sha) {
 
 	const Rev* r = revLookup(sha);
-	return (r == NULL) ? "" : r->shortLog();
+	return (r ? r->shortLog() : "");
 }
 
 const QStringList Git::getTagNames(bool onlyLoaded) {
@@ -350,7 +385,7 @@ const QString Git::getTagMsg(SCRef sha) {
 
 	Rev* c = const_cast<Rev*>(revLookup(sha));
 	if (!c)
-		return QString();
+		return "";
 
 	if (!c->tagMsg.isEmpty())
 		return c->tagMsg;
@@ -358,8 +393,10 @@ const QString Git::getTagMsg(SCRef sha) {
 	QRegExp pgp = QRegExp("-----BEGIN PGP SIGNATURE*END PGP SIGNATURE-----", true, true);
 
 	if (tagsObj.contains(sha)) {
+
 		QString runOutput;
-		if (run("git-cat-file tag " + tagsObj[sha], &runOutput)) {
+		if (run("git cat-file tag " + tagsObj[sha], &runOutput)) {
+
 			c->tagMsg = runOutput.section("\n\n", 1);
 			if (!c->tagMsg.isEmpty())
 				c->tagMsg = c->tagMsg.remove(pgp).stripWhiteSpace();
@@ -372,20 +409,12 @@ const QString Git::getTagMsg(SCRef sha) {
 MyProcess* Git::getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool combined) {
 
 	QString runCmd;
-	if (sha == ZERO_SHA) {
-		runCmd = "git-diff-index -r -m --patch-with-stat HEAD";
-		return runAsync(runCmd, receiver);
-	}
-	// workaround: --patch-with-stat is still not working well with merges
-	const Rev* c = revLookup(sha);
-	bool isMerge = (c ? c->parentsCount() > 1 : false);
-	const QString opt(isMerge ? "-p " : "--patch-with-stat ");
-	runCmd = (combined) ? "git-diff-tree -r -c " + opt : "git-diff-tree -r -m " + opt;
-
-	if (diffToSha.isEmpty())
-		runCmd.append(sha);
-	else
-		runCmd.append(diffToSha + " " + sha);
+	if (sha != ZERO_SHA) {
+		runCmd = "git diff-tree -r --patch-with-stat ";
+		runCmd.append(combined ? "-c " : "-C -m "); // TODO rename for combined
+		runCmd.append(diffToSha + " " + sha); // diffToSha could be empty
+	} else
+		runCmd = "git diff-index -r -m --patch-with-stat HEAD";
 
 	return runAsync(runCmd, receiver);
 }
@@ -393,9 +422,9 @@ MyProcess* Git::getDiff(SCRef sha, QObject* receiver, SCRef diffToSha, bool comb
 const QString Git::getFileSha(SCRef file, SCRef revSha) {
 
 	const QString sha(revSha == ZERO_SHA ? "HEAD" : revSha);
-	QString runCmd("git-ls-tree -r " + sha + " " + file), runOutput;
-	run(runCmd, &runOutput);
-	if (runOutput.isEmpty()) // deleted file case
+	QString runCmd("git ls-tree -r " + sha + " " + file), runOutput;
+	bool ok = run(runCmd, &runOutput);
+	if (!ok || runOutput.isEmpty()) // deleted file case
 		return "";
 
 	return runOutput.mid(12, 40);
@@ -409,9 +438,9 @@ MyProcess* Git::getFile(SCRef file, SCRef revSha, QObject* receiver, QString* re
 	else {
 		SCRef fileSha(getFileSha(file, revSha));
 		if (!fileSha.isEmpty())
-			runCmd = "git-cat-file blob " + fileSha;
+			runCmd = "git cat-file blob " + fileSha;
 		else
-			runCmd = "git-diff-tree HEAD HEAD"; // fake an empty file reading
+			runCmd = "git diff-tree HEAD HEAD"; // fake an empty file reading
 	}
 	if (receiver == NULL) {
 		run(runCmd, result);
@@ -420,14 +449,53 @@ MyProcess* Git::getFile(SCRef file, SCRef revSha, QObject* receiver, QString* re
 	return runAsync(runCmd, receiver);
 }
 
+MyProcess* Git::getHighlightedFile(SCRef file, SCRef sha, QObject* receiver, QString* result){
+
+	if (!isTextHighlighter()) {
+		dbs("ASSERT in getHighlightedFile: highlighter not found");
+		return NULL;
+	}
+	QString ext(file.section('.', -1, -1, QString::SectionIncludeLeadingSep));
+	QString inputFile(workDir + "/qgit_hlght_input" + ext);
+	if (!saveFile(file, sha, inputFile))
+		return NULL;
+
+	QString runCmd("source-highlight --failsafe -f html -i " + inputFile);
+
+	if (receiver == NULL) {
+		run(runCmd, result);
+		on_getHighlightedFile_eof();
+		return NULL; // in case of sync call we ignore run() return value
+	}
+	MyProcess* p = runAsync(runCmd, receiver);
+	if (p)
+		connect(p, SIGNAL(eof()), this, SLOT(on_getHighlightedFile_eof()));
+	return p;
+}
+
+void Git::on_getHighlightedFile_eof() {
+
+	QDir dir(workDir);
+	QStringList sl(dir.entryList("qgit_hlght_input*"));
+	loopList(it, sl)
+		dir.remove(*it);
+}
+
+bool Git::saveFile(SCRef file, SCRef sha, SCRef path) {
+
+	QString fileData;
+	getFile(file, sha, NULL, &fileData); // sync call
+	return writeToFile(path, fileData);
+}
+
 void Git::getTree(SCRef treeSha, SList names, SList shas,
                   SList types, bool isWorkingDir, SCRef treePath) {
 
-	QStringList newFiles, delFiles, dummy, tmp;
+	QStringList newFiles, unfiles, delFiles, dummy;
 	if (isWorkingDir) { // retrieve unknown and deleted files under treePath
 
-		getWorkDirFiles(UNKNOWN, tmp, dummy);
-		loopList(it, tmp) { // don't add unknown files under other directories
+		getWorkDirFiles(UNKNOWN, unfiles, dummy);
+		loopList(it, unfiles) { // don't add unknown files under other directories
 			QFileInfo f(*it);
 			SCRef d(f.dirPath(false));
 			if (d == treePath || (treePath.isEmpty() && d == "."))
@@ -438,7 +506,7 @@ void Git::getTree(SCRef treeSha, SList names, SList shas,
 	// if needed fake a working directory tree starting from HEAD tree
 	const QString tree(treeSha == ZERO_SHA ? "HEAD" : treeSha);
 	QString runOutput;
-	if (!run("git-ls-tree " + tree, &runOutput))
+	if (!run("git ls-tree " + tree, &runOutput))
 		return;
 
 	const QStringList sl(QStringList::split('\n', runOutput));
@@ -468,7 +536,7 @@ void Git::getTree(SCRef treeSha, SList names, SList shas,
 	}
 }
 
-void Git::getWorkDirFiles(SCRef status, SList files, SList dirs) {
+void Git::getWorkDirFiles(const QChar& status, SList files, SList dirs) {
 
 	files.clear();
 	dirs.clear();
@@ -477,10 +545,12 @@ void Git::getWorkDirFiles(SCRef status, SList files, SList dirs) {
 		return;
 
 	for (uint i = 0; i < f->names.count(); i++) {
-		if (f->status[i].endsWith(status)) {
+
+		if (f->statusCmp(i, status)) {
 
 			SCRef fp(filePath(*f, i));
 			files.append(fp);
+
 			for (int j = 0; j < fp.contains('/'); j++) {
 
 				SCRef dir(fp.section('/', 0, j));
@@ -491,17 +561,15 @@ void Git::getWorkDirFiles(SCRef status, SList files, SList dirs) {
 	}
 }
 
-bool Git::isSameTree(SCRef sha) {
+bool Git::isTreeModified(SCRef sha) {
 
-	const RevFile* files = getFiles(sha);
-	if (files) {
-		for (uint i = 0; i < files->names.count(); ++i) {
-			SCRef st(files->status[i].right(1));
-			if (st == DELETED || st == NEW)
-				return false;
-		}
+	const RevFile* f = getFiles(sha);
+	if (f) {
+		for (uint i = 0; i < f->names.count(); ++i)
+			if (f->statusCmp(i, DELETED) || f->statusCmp(i, NEW))
+				return true;
 	}
-	return true;
+	return false;
 }
 
 bool Git::isParentOf(SCRef par, SCRef child) {
@@ -517,25 +585,23 @@ bool Git::isSameFiles(SCRef tree1Sha, SCRef tree2Sha) {
 
 	// early skip common case of browsing with up and down arrows, i.e.
 	// going from parent(child) to child(parent). In this case we can
-	// check RevFileMap and skip a costly git-diff-tree call.
+	// check RevFileMap and skip a costly 'git diff-tree' call.
 	if (isParentOf(tree1Sha, tree2Sha))
-		return isSameTree(tree2Sha);
+		return !isTreeModified(tree2Sha);
 
 	if (isParentOf(tree2Sha, tree1Sha))
-		return isSameTree(tree1Sha);
+		return !isTreeModified(tree1Sha);
 
-	const QString runCmd("git-diff-tree -r -c " + tree1Sha + " " + tree2Sha);
+	const QString runCmd("git diff-tree -r " + tree1Sha + " " + tree2Sha);
 	QString runOutput;
 	if (!run(runCmd, &runOutput))
 		return false;
 
-	if (runOutput.find(" A\t") != -1 || runOutput.find(" D\t") != -1)
-		return false;
-
-	return true;
+	bool isChanged = (runOutput.find(" A\t") != -1 || runOutput.find(" D\t") != -1);
+	return !isChanged;
 }
 
-const QStringList Git::getDescBranches(SCRef sha) {
+const QStringList Git::getDescendantBranches(SCRef sha) {
 
 	QStringList tl;
 	const Rev* r = revLookup(sha);
@@ -557,36 +623,33 @@ const QStringList Git::getDescBranches(SCRef sha) {
 	return tl;
 }
 
-const QStringList Git::getNearTags(bool down, SCRef sha) {
+const QStringList Git::getNearTags(bool goDown, SCRef sha) {
 
 	QStringList tl;
 	const Rev* r = revLookup(sha);
 	if (!r)
 		return tl;
 
-	int nearRefsMaster = (down) ? r->descRefsMaster : r->ancRefsMaster;
+	int nearRefsMaster = (goDown ? r->descRefsMaster : r->ancRefsMaster);
 	if (nearRefsMaster == -1)
 		return tl;
 
-	const QValueVector<int>& nr = down ? revLookup(revOrder[nearRefsMaster])->descRefs :
-	                                     revLookup(revOrder[nearRefsMaster])->ancRefs;
+	const QValueVector<int>& nr = goDown ? revLookup(revOrder[nearRefsMaster])->descRefs :
+	                                       revLookup(revOrder[nearRefsMaster])->ancRefs;
 
 	for (uint i = 0; i < nr.count(); i++) {
 
 		SCRef sha = revOrder[nr[i]];
 		int idx = tagsSHA.findIndex(sha);
-		if (idx != -1) {
-			SCRef tagName = tags[idx];
-			const QString t = tagName + " (" + sha + ")";
-			tl.append(t);
-		}
+		if (idx != -1)
+			tl.append(tags[idx] + " (" + sha + ")");
 	}
 	return tl;
 }
 
 const QString Git::getDefCommitMsg() {
 
-	SCRef sha((appliedSHA.empty()) ? ZERO_SHA : appliedSHA.last());
+	SCRef sha(appliedSHA.empty() ? ZERO_SHA : appliedSHA.last());
 	const Rev* c = revLookup(sha);
 	if (c == NULL) {
 		dbp("ASSERT: getDefCommitMsg sha <%1> not found", sha);
@@ -621,15 +684,15 @@ const QString Git::colorMatch(SCRef txt, QRegExp& regExp) {
 const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
 
 	if (sha.isEmpty())
-		return QString();
+		return "";
 
 	const Rev* c = revLookup(sha);
 	if (c == NULL) {
-		dbp("ASSERT: getDesc sha <%1> not found", sha);
-		return QString();
+		dbp("ASSERT in Git::getDesc: sha <%1> not found", sha);
+		return "";
 	}
 
-	// set Latin-1 to avoid using QString::fromLatin1() everywhere
+	// set temporary Latin-1 to avoid using QString::fromLatin1() everywhere
 	QTextCodec* tc = QTextCodec::codecForCStrings();
 	QTextCodec::setCodecForCStrings(0);
 
@@ -642,21 +705,21 @@ const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
 		if (!c->isUnApplied) {
 			text.append("\nParent: ").append(c->parents().join("\nParent: "));
 
-			QStringList ch = getChilds(sha);
-			if (!ch.isEmpty())
-				text.append("\nChild: ").append(ch.join("\nChild: "));
+			QStringList sl = getChilds(sha);
+			if (!sl.isEmpty())
+				text.append("\nChild: ").append(sl.join("\nChild: "));
 
-			QStringList tg = getDescBranches(sha);
-			if (!tg.empty())
-				text.append("\nBranch: ").append(tg.join("\nBranch: "));
+			sl = getDescendantBranches(sha);
+			if (!sl.empty())
+				text.append("\nBranch: ").append(sl.join("\nBranch: "));
 
-			tg = getNearTags(false, sha);
-			if (!tg.isEmpty())
-				text.append("\nFollows: ").append(tg.join(", "));
+			sl = getNearTags(!optGoDown, sha);
+			if (!sl.isEmpty())
+				text.append("\nFollows: ").append(sl.join(", "));
 
-			tg = getNearTags(true, sha);
-			if (!tg.isEmpty())
-				text.append("\nPrecedes: ").append(tg.join(", "));
+			sl = getNearTags(optGoDown, sha);
+			if (!sl.isEmpty())
+				text.append("\nPrecedes: ").append(sl.join(", "));
 		}
 		text.append("\n\n    " + colorMatch(c->shortLog(), shortLogRE) +
 		            '\n' + colorMatch(c->longLog(), longLogRE));
@@ -671,7 +734,7 @@ const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
 
 		SCRef sha(reSHA.cap(0));
 		const Rev* r = revLookup(sha);
-		QString slog((r == NULL) ? sha : r->shortLog());
+		QString slog(r ? r->shortLog() : sha);
 		if (slog.isEmpty()) // very rare but possible
 			slog = sha;
 		if (slog.length() > 60)
@@ -692,16 +755,17 @@ const QString Git::getDesc(SCRef sha, QRegExp& shortLogRE, QRegExp& longLogRE) {
 const RevFile* Git::getAllMergeFiles(const Rev* r) {
 
 	SCRef mySha(ALL_MERGE_FILES + r->sha());
-	RevFileMap::Iterator rf = revsFiles.find(mySha);
-	if (rf == revsFiles.end()) {
-		QString runCmd("git-diff-tree -r -m " + r->sha()), runOutput;
+	RevFile* rf = revsFiles[mySha];
+	if (rf == NULL) {
+		QString runCmd("git diff-tree -r -m -C " + r->sha()), runOutput;
 		if (!run(runCmd, &runOutput))
 			return NULL;
 
-		rf = revsFiles.insert(mySha, RevFile());
+		revsFiles.insert(mySha, new RevFile());
+		rf = revsFiles[mySha];
 		parseDiffFormat(*rf, runOutput);
 	}
-	return &(*rf);
+	return rf;
 }
 
 const RevFile* Git::getFiles(SCRef sha, SCRef diffToSha, bool allFiles, SCRef path) {
@@ -717,64 +781,63 @@ const RevFile* Git::getFiles(SCRef sha, SCRef diffToSha, bool allFiles, SCRef pa
 		return getAllMergeFiles(r);
 
 	if (!diffToSha.isEmpty() && (sha != ZERO_SHA)) {
-		QString runCmd("git-diff-tree -r -m ");
+
+		QString runCmd("git diff-tree -r -m -C ");
 		runCmd.append(diffToSha + " " + sha);
 		if (!path.isEmpty())
 			runCmd.append(" " + path);
+
 		QString runOutput;
 		if (!run(runCmd, &runOutput))
 			return NULL;
+
 		// we insert a dummy revision file object. It will be
 		// overwritten at each request but we don't care.
-		RevFileMap::Iterator rf = revsFiles.insert(CUSTOM_SHA, RevFile());
+		revsFiles.insert(CUSTOM_SHA, new RevFile());
+		RevFile* rf = revsFiles[CUSTOM_SHA];
 		parseDiffFormat(*rf, runOutput);
-		return &(*rf);
+		return rf;
 	}
-	RevFileMap::Iterator rf = revsFiles.find(sha); // ZERO_SHA search arrives here
-	if (rf == revsFiles.end()) {
+	RevFile* rf = revsFiles[sha]; // ZERO_SHA search arrives here
+	if (rf == NULL) {
 
 		if (sha == ZERO_SHA) {
 			dbs("ASSERT in Git::getFiles, ZERO_SHA not found");
 			return NULL;
 		}
-		const QString runCmd("git-diff-tree -r -c " + sha);
-		QString runOutput;
+		QString runCmd("git diff-tree -r -c -C " + sha), runOutput;
 		if (!run(runCmd, &runOutput))
 			return false;
 
-		if (!revsFiles.contains(sha)) { // has been created in the mean time?
-			rf = revsFiles.insert(sha, RevFile());
+		if (!revsFiles.find(sha)) { // has been created in the mean time?
+			revsFiles.insert(sha, new RevFile());
+			rf = revsFiles[sha];
 			parseDiffFormat(*rf, runOutput);
 			cacheNeedsUpdate = true;
 		} else
-			return &(*revsFiles.find(sha));
+			return revsFiles[sha];
 	}
-	return &(*rf);
+	return rf;
 }
 
-void Git::getFileHistory(SCRef name) {
+void Git::startFileHistory(FileHistory* fh) {
 
-	histRevs.clear();
-	histRevOrder.clear();
-	firstHistFreeLane = 0;
-	histLns.clear();
-	histFileName = name;
-	startRevList(histFileName, true);
+	startRevList(fh->fileName, fh);
 }
 
 void Git::getFileFilter(SCRef path, QMap<QString, bool>& shaMap) {
 
 	shaMap.clear();
-	QRegExp rx(path, false, true);
-	RevFileMap::const_iterator it;
+	QRegExp rx(path, false, true); // not case sensitive and with wildcard
 	loop(StrVect, itr, revOrder) {
-		it = revsFiles.find(*itr);
-		if (it == revsFiles.constEnd())
+		RevFile* rf = revsFiles[*itr];
+		if (rf == NULL)
 			continue;
+
 		// case insensitive, wildcard search
-		for (uint i = 0; i < (*it).names.count(); ++i)
-			if (rx.search(filePath(*it, i)) != -1) {
-				shaMap.insert(it.key(), true);
+		for (uint i = 0; i < rf->names.count(); ++i)
+			if (rx.search(filePath(*rf, i)) != -1) {
+				shaMap.insert(*itr, true);
 				break;
 			}
 	}
@@ -791,7 +854,7 @@ bool Git::getPatchFilter(SCRef exp, bool isRegExp, QMap<QString, bool>& shaMap)
 	if (buf.isEmpty())
 		return true;
 
-	QString runCmd("git-diff-tree -r -s --stdin "), runOutput;
+	QString runCmd("git diff-tree -r -s --stdin "), runOutput;
 	if (isRegExp)
 		runCmd.append("--pickaxe-regex ");
 
@@ -808,12 +871,12 @@ bool Git::getPatchFilter(SCRef exp, bool isRegExp, QMap<QString, bool>& shaMap)
 
 bool Git::resetCommits(int parentDepth) {
 
-	QString runCmd("git-reset --soft HEAD~");
+	QString runCmd("git reset --soft HEAD~");
 	runCmd.append(QString::number(parentDepth));
 	return run(runCmd);
 }
 
-bool Git::applyPatchFile(SCRef patchPath, bool commit, bool fold, bool dropping) {
+bool Git::applyPatchFile(SCRef patchPath, bool commit, bool fold, bool sign) {
 
 	const QString quotedPath(QUOTE_CHAR + patchPath + QUOTE_CHAR);
 
@@ -823,9 +886,9 @@ bool Git::applyPatchFile(SCRef patchPath, bool commit, bool fold, bool dropping)
 
 		return run("stg import --mail " + quotedPath);
 	}
-	QString runCmd("git-am -k -u --3way ");
+	QString runCmd("git am -k -u --3way ");
 
-	if (testFlag(SIGN_PATCH_F) && !dropping)
+	if (testFlag(SIGN_PATCH_F) && sign)
 		runCmd.append("--signoff ");
 
 	return run(runCmd + quotedPath);
@@ -837,11 +900,13 @@ bool Git::formatPatch(SCList shaList, SCRef dirPath, SCRef remoteDir) {
 	QSettings settings;
 	const QString FPArgs = settings.readEntry(APP_KEY + FPATCH_ARGS_KEY, "");
 
-	QString runCmd("git-format-patch");
+	QString runCmd("git format-patch");
 	if (testFlag(NUMBERS_F) && !remote)
 		runCmd.append(" -n");
+
 	if (remote)
 		runCmd.append(" --keep-subject");
+
 	runCmd.append(" -o " + dirPath);
 	if (!FPArgs.isEmpty())
 		runCmd.append(" " + FPArgs);
@@ -856,7 +921,7 @@ bool Git::formatPatch(SCList shaList, SCRef dirPath, SCRef remoteDir) {
 	bool ret = false;
 	loopList(it, shaList) { // shaList is ordered by newest to oldest
 		const QString cmd(runCmd + QString::number(n) + " " +
-				*it + QString::fromLatin1("^..") + *it);
+		                  *it + QString::fromLatin1("^..") + *it);
 		n++;
 		ret = run(cmd);
 		if (!ret)
@@ -871,7 +936,7 @@ bool Git::updateIndex(SCList selFiles) {
 	if (selFiles.empty())
 		return true;
 
-	QString runCmd("git-update-index --add --remove --replace -- ");
+	QString runCmd("git update-index --add --remove --replace -- ");
 	runCmd.append(selFiles.join(" "));
 	return run(runCmd);
 }
@@ -900,40 +965,42 @@ bool Git::commitFiles(SCList selFiles, SCRef msg) {
 		cmtOptions.append(" -v");
 
 	// extract not selected files already updated
-	// in index, we don't want to commit them so we need
-	// to revert the index update with git-read-tree HEAD
-	QStringList notSelCachedFiles(getOtherFiles(selFiles, true));
+	// in index, save them to restore at the end
+	QStringList notSelInIndexFiles(getOtherFiles(selFiles, optOnlyInIndex));
 
-	// extract selected NOT to be deleted files because
-	// git-commit doesn't work with them
+	// extract selected NOT to be deleted files to
+	// later feed git commit. Files to be deleted
+	// should avoid going through 'git commit'
 	QStringList selNotDelFiles;
-	const RevFile* files = getFiles(ZERO_SHA);
+	const RevFile* files = getFiles(ZERO_SHA); // files != NULL
 	loopList(it, selFiles) {
 		int idx = findFileIndex(*files, *it);
-		if (!files->status[idx].endsWith(DELETED))
+		if (!files->statusCmp(idx, DELETED))
 			selNotDelFiles.append(*it);
 	}
-	// test if we need a git-read-tree to temporary
-	// remove not selected file's changes from index
-	if (!notSelCachedFiles.empty())
-		if (!run("git-read-tree --reset HEAD"))
+	// test if we need a git read-tree to temporary
+	// remove not selected files from index
+	if (!notSelInIndexFiles.empty())
+		if (!run("git read-tree --reset HEAD"))
 			return false;
 
-	// before to commit we have to update index with selected files
-	// because git-commit doesn't uses --add flag
+	// before to commit we have to update index with all
+	// the selected files because git commit doesn't
+	// use --add flag
 	updateIndex(selFiles);
 
-	// now we can commit, the script will update index with
-	// selected files (only for modify) for us
-	QString runCmd("git-commit -i" + cmtOptions + " -F " + msgFile);
+	// now we can commit, 'git commit' will update index
+	// with selected (not to be deleted) files for us
+	QString runCmd("git commit -i" + cmtOptions + " -F " + msgFile);
 	runCmd.append(" " + selNotDelFiles.join(" "));
 	if (!run(runCmd))
 		return false;
 
-	// re-update index with not selected files
-	if (!notSelCachedFiles.empty())
-		if (!updateIndex(notSelCachedFiles))
+	// finally restore not selected files in index
+	if (!notSelInIndexFiles.empty())
+		if (!updateIndex(notSelInIndexFiles))
 			return false;
+
 	QDir dir(workDir);
 	dir.remove(msgFile);
 	return true;
@@ -942,27 +1009,25 @@ bool Git::commitFiles(SCList selFiles, SCRef msg) {
 bool Git::mkPatchFromIndex(SCRef msg, SCRef patchFile) {
 
 	QString runOutput;
-	if (!run("git-diff-index --cached -p HEAD", &runOutput))
+	if (!run("git diff-index --cached -p HEAD", &runOutput))
 		return false;
 
-	SCRef patch("Subject: " + msg + "\n---\n" + runOutput);
-	if (!writeToFile(patchFile, patch))
-		return false;
-
-	return true;
+	const QString patch("Subject: " + msg + "\n---\n" + runOutput);
+	return writeToFile(patchFile, patch);
 }
 
-const QStringList Git::getOtherFiles(SCList selFiles, bool onlyCached) {
+const QStringList Git::getOtherFiles(SCList selFiles, bool onlyInIndex) {
 
-	const RevFile* files = getFiles(ZERO_SHA);
+	const RevFile* files = getFiles(ZERO_SHA); // files != NULL
 	QStringList notSelFiles;
 	for (uint i = 0; i < files->names.count(); ++i) {
 		SCRef fp = filePath(*files, i);
-		if (selFiles.find(fp) == selFiles.constEnd()) // not selected...
-			if (!onlyCached)
+		if (selFiles.find(fp) == selFiles.constEnd()) { // not selected...
+			if (!onlyInIndex)
 				notSelFiles.append(fp);
-			else if (files->status[i].startsWith(CACHED_FILE))
+			else if (files->isInIndex(i))
 				notSelFiles.append(fp);
+		}
 	}
 	return notSelFiles;
 }
@@ -978,21 +1043,21 @@ void Git::restoreFiles(SCList selFiles, SCRef workDir, SCRef ext) {
 
 	QDir d(workDir);
 	loopList(it, selFiles)
-		d.rename(*it + ext, *it); // overwrites any exsistant file
+		d.rename(*it + ext, *it); // overwrites any existent file
 }
 
 void Git::removeDeleted(SCList selFiles) {
 
 	QDir dir(workDir);
-	const RevFile* files = getFiles(ZERO_SHA);
+	const RevFile* files = getFiles(ZERO_SHA); // files != NULL
 	loopList(it, selFiles) {
 		int idx = findFileIndex(*files, *it);
-		if (files->status[idx].endsWith(DELETED))
+		if (files->statusCmp(idx, DELETED))
 			dir.remove(*it);
 	}
 }
 
-bool Git::stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool add) {
+bool Git::stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool fold) {
 
 	// here the deal is to create a patch with the diffs between the
 	// updated index and HEAD, then resetting the index and working
@@ -1005,11 +1070,11 @@ bool Git::stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool add) {
 	// we have selected modified files in selFiles, we still need
 	// to know the not selected but modified files and, among
 	// theese the cached ones to proper restore state at the end.
-	QStringList notSelFiles = getOtherFiles(selFiles, false);
-	QStringList notSelCachedFiles = getOtherFiles(selFiles, true);
+	QStringList notSelFiles = getOtherFiles(selFiles, !optOnlyInIndex);
+	QStringList notSelInIndexFiles = getOtherFiles(selFiles, optOnlyInIndex);
 
 	// update index with selected files
-	if (!run("git-read-tree --reset HEAD"))
+	if (!run("git read-tree --reset HEAD"))
 		goto error;
 	if (!updateIndex(selFiles))
 		goto error;
@@ -1023,13 +1088,13 @@ bool Git::stgCommit(SCList selFiles, SCRef msg, SCRef patchName, bool add) {
 	removeFiles(notSelFiles, workDir, extNS); // to restore at the end
 
 	// checkout index to have a clean tree
-	if (!run("git-read-tree --reset HEAD"))
+	if (!run("git read-tree --reset HEAD"))
 		goto error;
-	if (!run("git-checkout-index -q -f -u -a"))
+	if (!run("git checkout-index -q -f -u -a"))
 		goto rollback;
 
 	// finally import/fold the patch
-	if (add) {
+	if (fold) {
 		// update patch message before to fold so to use refresh only as a rename tool
 		if (!msg.isEmpty()) {
 			if (!run("stg refresh --message \"" + msg.stripWhiteSpace() + "\""))
@@ -1056,7 +1121,7 @@ exit:
 	// it is safe to call restore() also if back-up files don't
 	// exsist, so we can 'goto exit' from anywhere.
 	restoreFiles(notSelFiles, workDir, extNS);
-	updateIndex(notSelCachedFiles);
+	updateIndex(notSelInIndexFiles);
 	QDir dir(workDir);
 	dir.remove(patchFile);
 	loopList(it, selFiles)
@@ -1067,9 +1132,9 @@ exit:
 bool Git::makeTag(SCRef sha, SCRef tagName, SCRef msg) {
 
 	if (msg.isEmpty())
-		return run("git-tag " + tagName + " " + sha);
+		return run("git tag " + tagName + " " + sha);
 
-	return run("git-tag -m \"" + msg + "\" " + tagName + " " + sha);
+	return run("git tag -m \"" + msg + "\" " + tagName + " " + sha);
 }
 
 bool Git::deleteTag(SCRef sha) {
@@ -1078,8 +1143,8 @@ bool Git::deleteTag(SCRef sha) {
 	if (!r || !r->isTag)
 		return false;
 
-	// r->tag can contain more then one tag, separated by '\n'
-	return run("git-tag -d " + r->tag.section('\n', 0, 0));
+	// r->tag could contain more then one tag, separated by '\n'
+	return run("git tag -d " + r->tag.section('\n', 0, 0));
 }
 
 bool Git::stgPush(SCRef sha) {
@@ -1088,6 +1153,7 @@ bool Git::stgPush(SCRef sha) {
 	bool ret = run("stg push " + quotedPath);
 	if (!ret)
 		run("stg push --undo");
+
 	return ret;
 }
 
@@ -1128,9 +1194,7 @@ bool Git::readFromFile(SCRef fileName, QString& data) {
 		return false;
 	}
 	QTextStream stream(&file);
-	while (!stream.atEnd())
-		data.append(stream.readLine()).append('\n');
-
+	data = stream.read();
 	file.close();
 	return true;
 }
