@@ -12,6 +12,7 @@
 #include <qeventloop.h>
 #include <qregexp.h>
 #include <qtextcodec.h>
+#include <qobjectlist.h>
 #include "exceptionmanager.h"
 #include "rangeselectimpl.h"
 #include "cache.h"
@@ -24,29 +25,35 @@
 
 using namespace QGit;
 
-const QString Git::getArgs(bool askForRange, bool* quit) {
+const QString Git::getArgs(QString* range, bool askForRange, bool* quit) {
 
 	static bool startup = true; // it's OK to be unique among qgit windows
-	if (startup)
+	if (startup) {
+		*range = "";
 		for (int i = 1; i < qApp->argc(); i++) {
-			// in arguments with spaces double quotes are
-			// stripped by Qt, so re-add them.
+			// in arguments with spaces double quotes
+			// are stripped by Qt, so re-add them
 			QString arg(qApp->argv()[i]);
 			if (arg.contains(' '))
-				arg.prepend('"').append('"');
-			rangeSelectArgs.append(arg).append(' ');
+				arg.prepend('\"').append('\"');
+
+			range->append(arg + ' ');
 		}
-	bool showRangeSelect = testFlag(RANGE_SELECT_F);
-	if (showRangeSelect && askForRange && (rangeSelectArgs.isEmpty() || !startup)) {
-		rangeSelectArgs = "";
-		RangeSelectImpl rs(rangeSelectArgs, getTagNames(), par, rangeSelectOptions, quit);
+	}
+	if (    askForRange
+	    &&  testFlag(RANGE_SELECT_F)
+	    && (!startup || range->isEmpty())) {
+
+		RangeSelectImpl rs(par, range, getTagNames(), quit);
 		rs.exec(); // modal execution
 		if (*quit)
-			return QString();
+			return "";
 	}
 	startup = false;
 	QString runOutput;
-	run(QString("git-rev-parse --default HEAD ") + rangeSelectArgs, &runOutput);
+	if (!run("git rev-parse --default HEAD " + *range, &runOutput))
+		return "";
+
 	return runOutput.replace('\n', " ");
 }
 
@@ -56,7 +63,7 @@ const QString Git::getBaseDir(bool* changed, SCRef wd, bool* ok, QString* gd) {
 	QString runOutput, tmp(workDir);
 	workDir = wd;
 	errorReportingEnabled = false;
-	bool ret = run("git-rev-parse --git-dir", &runOutput); // run under newWorkDir
+	bool ret = run("git rev-parse --git-dir", &runOutput); // run under newWorkDir
 	errorReportingEnabled = true;
 	workDir = tmp;
 	runOutput = runOutput.stripWhiteSpace();
@@ -66,15 +73,15 @@ const QString Git::getBaseDir(bool* changed, SCRef wd, bool* ok, QString* gd) {
 			*ok = false;
 		return wd;
 	}
-	// 'git-rev-parse --git-dir' output could be a dir relative
-	// to working dir (as ex .git) or an absolute path.
+	// 'git rev-parse --git-dir' output could be a relative
+	// to working dir (as ex .git) or an absolute path
  	QDir d(runOutput.startsWith("/") ? runOutput : wd + "/" + runOutput);
 	*changed = (d.absPath() != gitDir);
 	if (gd)
 		*gd = d.absPath();
-	d.cdUp();
 	if (ok)
 		*ok = true;
+	d.cdUp();
 	return d.absPath();
 }
 
@@ -86,29 +93,31 @@ bool Git::getRefs() {
 	refsStillToFindMasterCopy.clear();
 
 	// check for a StGIT stack
-	isStGIT = false;
 	QDir d(gitDir);
 	if (d.exists("patches")) { // early skip
 		errorReportingEnabled = false;
-		isStGIT = run("stg unapplied"); // slow command, check if stg bin is in path
+		isStGIT = run("stg unapplied"); // slow command, check for stg bin
 		errorReportingEnabled = true;
-	}
+	} else
+		isStGIT = false;
+
 	// check for a merge and read current branch sha
 	isMergeHead = d.exists("MERGE_HEAD");
-	if (!run("git-rev-parse HEAD", &currentBranchSHA))
+	if (!run("git rev-parse HEAD", &currentBranchSHA))
 		return false;
+
 	currentBranchSHA = currentBranchSHA.stripWhiteSpace();
 
 	// read refs, normally unsorted
 	QString runOutput;
-	if (!run("git-peek-remote " + gitDir, &runOutput))
+	if (!run("git peek-remote " + gitDir, &runOutput))
 		return false;
 
 	QStringList rLst(QStringList::split('\n', runOutput));
 	loopList(it, rLst) {
 
 		SCRef refSha = (*it).left(40);
-		SCRef refName = (*it).right((*it).length() - 41);
+		SCRef refName = (*it).mid(41);
 
 		if (refName.startsWith("refs/patches/")) {
 			// StGIT patches should not be added to refs,
@@ -136,13 +145,13 @@ bool Git::getRefs() {
 				tagsSHA.first() = refSha;
 
 			} else {
-				tags.prepend(refName.right(refName.length() - 10));
+				tags.prepend(refName.mid(10));
 				tagsSHA.prepend(refSha);
 			}
 			continue;
 		}
 		if (refName.startsWith("refs/heads/")) {
-			heads.append(refName.right(refName.length() - 11));
+			heads.append(refName.mid(11));
 			headsSHA.append(refSha);
 			continue;
 		}
@@ -205,6 +214,7 @@ bool Git::getStGITPatches() {
 	QString branch;
 	if (!run("stg branch", &branch))
 		return false;
+
 	branch = branch.stripWhiteSpace();
 
 	// get patch names and status of current branch
@@ -219,7 +229,7 @@ bool Git::getStGITPatches() {
 	const QStringList pl = QStringList::split('\n', runOutput);
 	loopList(it, pl) { // keep stg order for unapplied
 		SCRef st = (*it).left(1);
-		SCRef pn = (*it).right((*it).length() - 2);
+		SCRef pn = (*it).mid(2);
 		if (st == "+" || st == ">")
 			al.append(pn);
 		else
@@ -268,7 +278,7 @@ const Rev* Git::fakeWorkDirRev(SCRef parent, SCRef log, SCRef longLog, int idx)
 const QStringList Git::getOthersFiles() {
 // add files exsistant in working directory but not in git archive
 
-	QString runCmd("git-ls-files --others ");
+	QString runCmd("git ls-files --others ");
 	QSettings settings;
 	QString exFile(settings.readEntry(APP_KEY + EX_KEY, EX_DEF));
 	QString exPerDir(settings.readEntry(APP_KEY + EX_PER_DIR_KEY, EX_PER_DIR_DEF));
@@ -287,71 +297,103 @@ const QStringList Git::getOthersFiles() {
 void Git::getDiffIndex() {
 
 	QString status;
-	if (!run("git-status", &status)) // git-status refreshes the index, run as first
+	if (!run("git status", &status)) // git status refreshes the index, run as first
 		return;
 
 	QString diffIndex;
-	if (!run("git-diff-index HEAD", &diffIndex))
+	if (!run("git diff-index HEAD", &diffIndex))
 		return;
 
 	// now mockup a RevFile
-	RevFileMap::iterator itf;
-	itf = revsFiles.insert(ZERO_SHA, RevFile());
-	parseDiffFormat(*itf, diffIndex);
+	revsFiles.insert(ZERO_SHA, new RevFile());
+	RevFile* rf = revsFiles[ZERO_SHA];
+	parseDiffFormat(*rf, diffIndex);
 
 	const QStringList otherFiles(getOthersFiles()); // get unknown files
 	if (!otherFiles.isEmpty()) {
 		loopList(it, otherFiles) {
-			appendFileName(*itf, *it);
-			(*itf).status.append(UNKNOWN);
-			(*itf).mergeParent.append(1);
+			appendFileName(*rf, *it);
+			rf->status.append(UNKNOWN);
+			rf->mergeParent.append(1);
 		}
 	}
 	revOrder.append(ZERO_SHA);
 	unknownFiles = (otherFiles.count() > 0);
-	nothingToCommit = (itf.data().names.count() == otherFiles.count());
+	nothingToCommit = (rf->names.count() == otherFiles.count());
 	const QString log(nothingToCommit ? "Nothing to commit" : "Working dir changes");
 
 	// then mockup the corresponding Rev
 	QString runOutput;
-	if (!run("git-rev-parse --default HEAD", &runOutput))
+	if (!run("git rev-parse --default HEAD", &runOutput))
 		return;
 
 	const QString parent(runOutput.section('\n', 0, 0));
 	revs.insert(ZERO_SHA, fakeWorkDirRev(parent, log, status, revs.count()));
 
 	// check for files already updated in cache, we will
-	// save this information as a prefix in status field
+	// save this information in status third field
 	RevFile cachedFiles;
-	if (!run("git-diff-index --cached HEAD", &runOutput))
+	if (!run("git diff-index --cached HEAD", &runOutput))
 		return;
 
 	parseDiffFormat(cachedFiles, runOutput);
 
-	for (uint i = 0; i < (*itf).status.count(); i++) {
+	for (uint i = 0; i < rf->status.count(); i++) {
 
-		SCRef t = (findFileIndex(cachedFiles, filePath(*itf, i)) == -1) ?
-				NO_CACHED_FILE : CACHED_FILE;
-
-		(*itf).status[i].prepend(t);
+		SCRef t = (findFileIndex(cachedFiles, filePath(*rf, i)) == -1) ? NOT_IN_INDEX :
+		                                                                 IN_INDEX;
+		rf->status[i].append(",," + t); // third field of status string
 	}
 	// finally send it to GUI
-	emit newRevsAdded(revOrder);
+	emit newRevsAdded(NULL, revOrder);
 }
 
 void Git::parseDiffFormatLine(RevFile& rf, SCRef line, int parNum) {
 
-	if (line[1] == ':') { // it's a merge
+	if (line[1] == ':') { // it's a combined merge
+
+		// TODO rename/copy is not suported for combined merges
 		appendFileName(rf, line.section('\t', -1));
 		rf.status.append(line.section('\t', -2, -1).right(1));
 		rf.mergeParent.append(parNum);
 	} else { // faster parsing in normal case
-		appendFileName(rf, line.mid(99, line.length() - 99));
-		rf.status.append(line.mid(97, 1));
-		rf.mergeParent.append(parNum);
+
+		if (line[98] == '\t') {
+			appendFileName(rf, line.mid(99));
+			rf.status.append(line.at(97));
+			rf.mergeParent.append(parNum);
+		} else
+			// it's a rename or a copy, we are not in fast path now!
+			setStatus(rf, line.mid(97), parNum);
 	}
 }
 
+void Git::setStatus(RevFile& rf, SCRef rowSt, int parNum) {
+
+	QStringList sl = QStringList::split('\t', rowSt);
+	if (sl.count() == 3) {
+
+		// we want store extra info with format "origFile --> destFile (Rxx%)"
+		// but git give us something like "Rxx\t<origFile>\t<destFile>"
+		SCRef origFile = sl[1];
+		SCRef destFile = sl[2];
+		QString info("," + origFile + " --> " + destFile + " (" + sl[0] + "%),,");
+
+		// simulate new file
+		appendFileName(rf, destFile);
+		rf.mergeParent.append(parNum);
+		rf.status.append(NEW + info);
+
+		// simulate deleted orig file only in case of rename
+		if (sl[0].at(0) == RENAMED) {
+			appendFileName(rf, origFile);
+			rf.mergeParent.append(parNum);
+			rf.status.append(DELETED + info);
+		}
+	} else
+		dbp("ASSERT in setStatus, unexpected status string %1", rowSt);
+}
+
 void Git::parseDiffFormat(RevFile& rf, SCRef buf) {
 
 	int parNum = 1, startPos = 0, endPos = buf.find('\n');
@@ -366,59 +408,49 @@ void Git::parseDiffFormat(RevFile& rf, SCRef buf) {
 	}
 }
 
-bool Git::startParseProc(SCRef initCmd) {
+bool Git::startParseProc(SCRef initCmd, FileHistory* fh) {
 
-	loadedTagNames.clear();
-	loadedBranchNames.clear();
+	if (fh == NULL) {
+		loadedTagNames.clear();
+		loadedBranchNames.clear();
+	}
+	DataLoader* dl = new DataLoader(this, fh); // auto-deleted when done
 
-	DataLoader* dl = new DataLoader(this, false); // auto-deleted when done
+	connect(this, SIGNAL(cancelLoading(const FileHistory*)),
+	        dl, SLOT(on_cancel(const FileHistory*)));
 
-	connect(dl, SIGNAL(newDataReady()), this, SLOT(on_newDataReady()));
-	connect(dl, SIGNAL(loaded(ulong,const QTime&,bool,const QString&,const QString&)),
-		this, SLOT(on_loaded(ulong,const QTime&,bool,const QString&,const QString&)));
+	connect(dl, SIGNAL(newDataReady(const FileHistory*)),
+	        this, SLOT(on_newDataReady(const FileHistory*)));
+
+	connect(dl, SIGNAL(loaded(const FileHistory*, ulong, int,
+	        bool, const QString&, const QString&)), this,
+	        SLOT(on_loaded(const FileHistory*, ulong, int,
+	        bool, const QString&, const QString&)));
 
 	return dl->start(initCmd, workDir);
 }
 
-bool Git::startHistoryProc(SCRef initCmd) {
-
-	DataLoader* dl = new DataLoader(this, true); // auto-deleted when done
-
-	// only one history loader can be active at a given time,
-	// this means that current Git class cannot handle multiple
-	// file history windows.
-	// a history loader may start while a previous one is canceling
-	// but still running.
-	connect(this, SIGNAL(cancelHistLoading()), dl, SLOT(on_cancel()));
-	connect(dl, SIGNAL(newDataReady()), this, SLOT(on_newHistDataReady()));
-	connect(dl, SIGNAL(loaded(ulong,const QTime&,bool,const QString&,const QString&)),
-	this, SLOT(on_histLoaded(ulong,const QTime&,bool,const QString&,const QString&)));
-
-	return dl->start(initCmd, workDir);
-}
-
-bool Git::startRevList(SCRef args, bool historyProc) {
+bool Git::startRevList(SCRef args, FileHistory* fh) {
 
 	refsStillToFind = refsStillToFindMasterCopy;
-	QString initCmd("git-rev-list --header --topo-order --boundary --parents ");
-	if (historyProc) {
+	QString initCmd("git rev-list --header --topo-order --boundary --parents ");
+	if (fh)
 		initCmd.append("--remove-empty --all -- ");
-		return startHistoryProc(initCmd + args);
-	}
-	return startParseProc(initCmd + args);
+
+	return startParseProc(initCmd + args, fh);
 }
 
 bool Git::startUnappliedList() {
 
-	// WARNING: with this command git-rev-list could send spurious
+	// WARNING: with this command git rev-list could send spurious
 	// revs so we need some filter out logic during loading
-	QString initCmd("git-rev-list --header --parents ");
+	QString initCmd("git rev-list --header --parents ");
 	initCmd.append(unAppliedSHA.join(" "));
 	initCmd.append(QString::fromLatin1(" ^HEAD"));
-	return startParseProc(initCmd);
+	return startParseProc(initCmd, NULL);
 }
 
-void Git::stop(bool notSaveCache) {
+bool Git::stop(bool saveCache) {
 // normally called when changing directory or closing
 
 	EM_RAISE(exGitStopped);
@@ -427,23 +459,31 @@ void Git::stop(bool notSaveCache) {
 
 	EM_BEFORE_PROCESS_EVENTS;
 
-	while ((runningProcesses > 0) && qApp->hasPendingEvents()) {
+	// wait until all processes are exited. Note that we
+	// could have been called while in run(), in this case we
+	// exit from stop() while still in run() and corresponding
+	// QProcess is still not deleted
+	while (runningProcesses > 0) {
 		usleep(20000);
 		qApp->processEvents(QEventLoop::ExcludeUserInput);
 	}
 	EM_AFTER_PROCESS_EVENTS;
 
-	if (cacheNeedsUpdate && !notSaveCache) {
+	if (cacheNeedsUpdate && saveCache) {
 		cacheNeedsUpdate = false;
 		if (!filesLoadingCurSha.isEmpty()) // we are in the middle of a loading
 			revsFiles.remove(filesLoadingCurSha); // remove partial data
 
 		if (!revsFiles.isEmpty()) {
 			POST_MSG("Saving cache. Please wait...");
-			if (!cache->update(workDir, revsFiles, dirNamesVec, fileNamesVec))
+			if (!cache->save(workDir, revsFiles, dirNamesVec, fileNamesVec))
 				qDebug("ERROR unable to update cache");
 		}
 	}
+	QObjectList* l = this->queryList("QProcess");
+	bool allDeleted = (l->count() == 0);
+	delete l;
+	return allDeleted;
 }
 
 void Git::clearRevs() {
@@ -503,7 +543,7 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 				qDebug("WARNING: no tags or heads found");
 
 			POST_MSG("");
-			args = getArgs(askForRange, quit); // must be called with refs loaded
+			args = getArgs(&curRange, askForRange, quit); // must be called with refs loaded
 			if (*quit) {
 				EM_REMOVE(exGitStopped);
 				return false;
@@ -560,17 +600,12 @@ bool Git::init(SCRef wd, bool askForRange, QStringList* filterList, bool* quit)
 	}
 }
 
-void Git::on_newDataReady() {
+void Git::on_newDataReady(const FileHistory* fh) {
 
-	emit newRevsAdded(revOrder);
+	emit newRevsAdded(fh, fh ? fh->revOrder : revOrder);
 }
 
-void Git::on_newHistDataReady() {
-
-	emit newHistRevsAdded(histRevOrder);
-}
-
-void Git::on_loaded(ulong byteSize, const QTime& loadTime,
+void Git::on_loaded(const FileHistory* fh, ulong byteSize, int loadTime,
                     bool normalExit, SCRef cmd, SCRef errorDesc) {
 
 	if (!errorDesc.isEmpty()) {
@@ -579,18 +614,19 @@ void Git::on_loaded(ulong byteSize, const QTime& loadTime,
 	}
 	if (normalExit) { // do not send anything if killed
 
-		emit newRevsAdded(revOrder);
+		on_newDataReady(fh);
 
 		if (!loadingUnAppliedPatches) {
 
 			uint kb = byteSize / 1024;
 			QString tmp(QString("Loaded %1 revisions (%2 KB), time elapsed: %3 ms")
-			                    .arg(revs.count()).arg(kb).arg(loadTime.elapsed()));
+			                    .arg(revs.count()).arg(kb).arg(loadTime));
 
-			emit loadCompleted(tmp);
+			emit loadCompleted(fh, tmp);
 
-			// check for revs with no loaded files out of fast path
-			QTimer::singleShot(10, this, SLOT(loadFileNames()));
+			if (fh == NULL)
+				// check for revs with no loaded files out of fast path
+				QTimer::singleShot(10, this, SLOT(loadFileNames()));
 		}
 	}
 	if (loadingUnAppliedPatches)
@@ -602,7 +638,7 @@ void Git::loadFileNames() {
 
 	QString diffTreeBuf;
 	loop(StrVect, it, revOrder) {
-		if (!revsFiles.contains(*it)) {
+		if (!revsFiles.find(*it)) {
 			const Rev* c = revLookup(*it);
 			if (c->parentsCount() == 1) // skip initials and merges
 				diffTreeBuf.append(*it).append('\n');
@@ -610,28 +646,15 @@ void Git::loadFileNames() {
 	}
 	if (!diffTreeBuf.isEmpty()) {
 		filesLoadingPending = filesLoadingCurSha = "";
-		const QString runCmd("git-diff-tree -r -c --stdin");
+		const QString runCmd("git diff-tree -r -C --stdin");
 		runAsync(runCmd, this, diffTreeBuf);
 	}
 	indexTree();
 }
 
-void Git::on_histLoaded(ulong, const QTime&, bool normalExit,
-                        SCRef cmd, SCRef errorDesc) {
+void Git::addChunk(FileHistory* fh, SCRef buffer) {
 
-	if (!errorDesc.isEmpty()) {
-		MainExecErrorEvent* e = new MainExecErrorEvent(cmd, errorDesc);
-		QApplication::postEvent(par, e);
-	}
-	if (normalExit) { // do not send anything if killed
-		emit newHistRevsAdded(histRevOrder);
-		emit histLoadCompleted();
-	}
-}
-
-void Git::addChunk(bool addToHistory, SCRef buffer) {
-
-	RevMap& r = (addToHistory) ? histRevs : revs;
+	RevMap& r = (fh ? fh->revs : revs);
 
 	// take in account boundaries
 	SCRef sha = (buffer[0] == '-') ? buffer.mid(1, 40) : buffer.left(40);
@@ -654,7 +677,7 @@ void Git::addChunk(bool addToHistory, SCRef buffer) {
 	}
 	if (r.find(sha)) {
 		// StGIT unapplied patches could be sent again by
-		// git-rev-list as example if called with --all option.
+		// git rev-list as example if called with --all option.
 		if (r[sha]->isUnApplied)
 			return;
 		dbp("ASSERT: addChunk sha <%1> already received", sha);
@@ -662,27 +685,27 @@ void Git::addChunk(bool addToHistory, SCRef buffer) {
 
 	r.insert(sha, new Rev(buffer, r.count())); // only here we create a new rev
 
-	Rev* c = const_cast<Rev*>(revLookup(sha, addToHistory));
-	updateRefs(*c, addToHistory);
+	Rev* c = const_cast<Rev*>(revLookup(sha, fh));
+	updateRefs(*c, fh);
 
 	// updateLanes() is called too late, after loadingUnAppliedPatches
 	// has been reset so update the lanes now.
 	if (loadingUnAppliedPatches)
 		c->lanes.append(UNAPPLIED);
 
-	if (addToHistory) {
+	if (fh) {
 		if (r.count() == 1)
-			copyDiffIndex(histRevs, histFileName);
-		histRevOrder.append(sha); // used to append commits in proper order
+			copyDiffIndex(fh);
+		fh->revOrder.append(sha); // used to append commits in proper order
 	} else
 		revOrder.append(sha);
 }
 
-void Git::copyDiffIndex(RevMap& histRevs, SCRef histFileName) {
+void Git::copyDiffIndex(FileHistory* fh) {
 // try to add ZERO_SHA to history. Must be called with only one rev
 // in histRevs that will be used as the parent.
 
-	if (!histRevOrder.isEmpty() || (histRevs.count() != 1)) {
+	if (!fh->revOrder.isEmpty() || (fh->revs.count() != 1)) {
 		qDebug("ASSERT in copyDiffIndex: called with wrong context");
 		return;
 	}
@@ -691,40 +714,32 @@ void Git::copyDiffIndex(RevMap& histRevs, SCRef histFileName) {
 		return;
 
 	const RevFile* files = getFiles(ZERO_SHA);
-	bool found = false;
-	for (uint i = 0; i < files->names.count(); ++i) {
-		SCRef fp = filePath(*files, i);
-		if (fp == histFileName) {
-			found = true;
-			break;
-		}
-	}
-	if (!found)
+	if (!files || findFileIndex(*files, fh->fileName) == -1)
 		return;
 
 	// insert a custom ZERO_SHA rev with proper parent
-	QDictIterator<Rev> it(histRevs); // only one item
+	QDictIterator<Rev> it(fh->revs); // only one item
 	QString parent(it.currentKey());
-	histRevs.insert(ZERO_SHA, fakeWorkDirRev(parent, "Working dir changes",
-	                                         "dummy", histRevs.count()));
-	histRevOrder.append(ZERO_SHA);
+	fh->revs.insert(ZERO_SHA, fakeWorkDirRev(parent, "Working dir changes",
+	                                         "dummy", fh->revs.count()));
+	fh->revOrder.append(ZERO_SHA);
 }
 
-void Git::setLane(SCRef sha, bool isHistory) {
+void Git::setLane(SCRef sha, FileHistory* fh) {
 
-	Lanes& l = (isHistory) ? histLns : lns;
-	uint i = (isHistory) ? firstHistFreeLane : firstFreeLane;
-	const QValueVector<QString>& shaVec((isHistory) ? histRevOrder : revOrder);
+	Lanes& l = (fh ? fh->lns : lns);
+	uint i = (fh ? fh->firstFreeLane : firstFreeLane);
+	const QValueVector<QString>& shaVec(fh ? fh->revOrder : revOrder);
 	for ( ; i < shaVec.count(); ++i) {
-		Rev* r = const_cast<Rev*>(revLookup(shaVec[i], isHistory));
+		Rev* r = const_cast<Rev*>(revLookup(shaVec[i], fh));
 		if (r->lanes.count() == 0)
 			updateLanes(*r, l);
 
 		if (shaVec[i] == sha)
 			break;
 	}
-	if (isHistory)
-		firstHistFreeLane = ++i;
+	if (fh)
+		fh->firstFreeLane = ++i;
 	else
 		firstFreeLane = ++i;
 }
@@ -784,7 +799,7 @@ void Git::on_procDataReady(const QString& fileChunk) {
 	else
 		filesLoadingPending.append(fileChunk); // add to previous half lines
 
-	RevFileMap::iterator rf(revsFiles.find(filesLoadingCurSha));
+	RevFile* rf = revsFiles[filesLoadingCurSha];
 	int nextEOL = filesLoadingPending.find('\n');
 	int lastEOL = -1;
 	while (nextEOL != -1) {
@@ -792,8 +807,10 @@ void Git::on_procDataReady(const QString& fileChunk) {
 		SCRef line(filesLoadingPending.mid(lastEOL + 1, nextEOL - lastEOL - 1));
 		if (line.constref(0) != ':') {
 			SCRef sha = line.left(40);
-			if (rf == revsFiles.end() || sha != rf.key()) { // new commit
-				rf = revsFiles.insert(sha, RevFile());
+			if (rf == NULL || sha != filesLoadingCurSha) { // new commit
+				revsFiles.insert(sha, new RevFile());
+				rf = revsFiles[sha];
+				filesLoadingCurSha = sha;
 				cacheNeedsUpdate = true;
 			} else
 				dbp("ASSERT: repeated sha %1 in file names loading", sha);
@@ -805,16 +822,13 @@ void Git::on_procDataReady(const QString& fileChunk) {
 	}
 	if (lastEOL != -1)
 		filesLoadingPending.remove(0, lastEOL + 1);
-
-	if (rf != revsFiles.end())
-		filesLoadingCurSha = rf.key();
 }
 
 void Git::appendFileName(RevFile& rf, SCRef name) {
 
 	int idx = name.findRev('/') + 1;
 	SCRef dr = name.left(idx);
-	SCRef nm = name.right(name.length() - idx);
+	SCRef nm = name.mid(idx);
 
 	StrSet::iterator it(dirNames.find(dr));
 	if (it == dirNames.end()) {
@@ -831,7 +845,7 @@ void Git::appendFileName(RevFile& rf, SCRef name) {
 	rf.names.append(it.data());
 }
 
-void Git::updateRefs(Rev& c, bool addToHistory) {
+void Git::updateRefs(Rev& c, FileHistory* fh) {
 
 	SCRef sha(c.sha());
 
@@ -861,7 +875,7 @@ void Git::updateRefs(Rev& c, bool addToHistory) {
 		while (itSha != tagsSHA.constEnd()) {
 			if (*itSha == sha) {
 				c.tag.append(*itNames + "\n");
-				if (!addToHistory)
+				if (fh == NULL)
 					loadedTagNames.append(*itNames);
 			}
 			++itSha;
@@ -880,7 +894,7 @@ void Git::updateRefs(Rev& c, bool addToHistory) {
 		while (itSha != headsSHA.constEnd()) {
 			if (*itSha == sha) {
 				c.branch.append(*itNames + "\n");
-				if (!addToHistory)
+				if (fh == NULL)
 					loadedBranchNames.append(*itNames);
 			}
 			++itSha;
@@ -954,10 +968,6 @@ void Git::mergeNearTags(bool down, Rev* p, const Rev* r, const QMap<QPair<uint,
 	if (!down && (p->ancRefsMaster == r_ancRefsMaster || r_ancRefsMaster == -1))
 		return;
 
-	if (dm.isEmpty()) {
-		dbs("ASSERT descendantMap is empty");
-		return;
-	}
 	// we want the nearest tag only, so remove any tag
 	// that is ancestor of any other tag in p U r
 	SCRef sha1 = down ? revOrder[p->descRefsMaster] : revOrder[p->ancRefsMaster];
@@ -977,7 +987,7 @@ void Git::mergeNearTags(bool down, Rev* p, const Rev* r, const QMap<QPair<uint,
 			}
 			QPair<uint, uint> key = qMakePair((uint)src2[s2], (uint)src1[s1]);
 
-			if (!dm.contains(key)) {
+			if (!dm.contains(key)) { // could be empty if all tags are indipendent
 				add = true; // could be an indipendent path
 				continue;
 			}
@@ -1011,7 +1021,8 @@ void Git::indexTree() {
 	QMap<QPair<uint, uint>, bool> descMap;
 	QMap<uint, QValueVector<int> > descVect;
 
-	// walk down the tree, compute chidlren and nearest descendants
+	// walk down the tree from latest to oldest,
+	// compute chidlren and nearest descendants
 	for (uint i = 0; i < revOrder.count(); i++) {
 
 		const Rev* r = revLookup(revOrder[i]);
@@ -1046,7 +1057,7 @@ void Git::indexTree() {
 					p->descRefsMaster = r->isTag ? r->orderIdx :
 					                               r->descRefsMaster;
 				else
-					mergeNearTags(true, p, r, descMap);
+					mergeNearTags(optGoDown, p, r, descMap);
 			}
 		}
 	}
@@ -1067,7 +1078,7 @@ void Git::indexTree() {
 				if (c->ancRefsMaster == -1)
 					c->ancRefsMaster = r->isTag ? r->orderIdx:r->ancRefsMaster;
 				else
-					mergeNearTags(false, c, r, descMap);
+					mergeNearTags(!optGoDown, c, r, descMap);
 			}
 		}
 	}
@@ -1116,4 +1127,5 @@ void Rev::indexData() const {
 		sLogLen = d.length() - idx;
 		lLogLen = 0;
 	}
+	indexed = true;
 }
