@@ -13,9 +13,8 @@
 
 class Lanes {
 public:
-	Lanes() {};
-	~Lanes() {};
-	bool isEmpty() { return lanes.empty(); };
+	Lanes() {}; // init() will setup us later, when data is available
+	bool isEmpty() { return typeVec.empty(); };
 	void init(const QString& expectedSha);
 	void clear();
 	bool isFork(const QString& sha, bool& isDiscontinuity);
@@ -31,21 +30,18 @@ public:
 	void afterBranch();
 	void afterApplied();
 	void nextParent(const QString& sha);
-	void getLanes(QValueVector<int> &ln);
+	void getLanes(QValueVector<int> &ln){ ln = typeVec; }; // O(1) vector is implicitly shared
 
 private:
-	int activeLane ;
-	struct Lane {
-		int type;
-		QString next;
-	};
-	QValueVector<Lane> lanes;
-	bool boundary;
-	int NODE, NODE_L, NODE_R;
-
-	int findNext(const QString& next, int pos);
+	int findNextSha(const QString& next, int pos);
 	int findType(int type, int pos);
 	int add(int type, const QString& next, int pos);
+
+	int activeLane ;
+	QValueVector<int> typeVec;
+	QValueVector<QString> nextShaVec;
+	bool boundary;
+	int NODE, NODE_L, NODE_R;
 };
 
 #endif
