@@ -22,8 +22,8 @@
 
 using namespace QGit;
 
-ListView::ListView(Domain* dm, Git* g, QListView* l, bool h) :
-                   QObject(dm), d(dm), git(g), lv(l), isHistory(h) {
+ListView::ListView(Domain* dm, Git* g, QListView* l, FileHistory* f) :
+                   QObject(dm), d(dm), git(g), lv(l), fh(f) {
 
 	st = &(d->st);
 	processedRevs = 0;
@@ -31,8 +31,8 @@ ListView::ListView(Domain* dm, Git* g, QListView* l, bool h) :
 	filterNextContextMenuRequest = false;
 	setupListView();
 
-	lv->setAcceptDrops(!isHistory);
-	lv->viewport()->setAcceptDrops(!isHistory);
+	lv->setAcceptDrops(fh == NULL);
+	lv->viewport()->setAcceptDrops(fh == NULL);
 	lv->viewport()->installEventFilter(this); // filter out some right clicks
 
 	if (testFlag(REL_DATE_F))
@@ -42,22 +42,26 @@ ListView::ListView(Domain* dm, Git* g, QListView* l, bool h) :
 
 	connect(lv, SIGNAL(currentChanged(QListViewItem*)),
 	        this, SLOT(on_currentChanged(QListViewItem*)));
+
 	connect(lv, SIGNAL(contentsMoving(int,int) ),
 	        this, SLOT(on_contentsMoving(int,int)));
+
 	connect(lv, SIGNAL(mouseButtonPressed(int,QListViewItem*,const QPoint&,int)),
 	        this, SLOT(on_mouseButtonPressed(int,QListViewItem*,const QPoint&,int)));
+
 	connect(lv, SIGNAL(clicked(QListViewItem*)),
 	        this, SLOT(on_clicked(QListViewItem*)));
+
 	connect(lv, SIGNAL(onItem(QListViewItem*)),
 	        this, SLOT(on_onItem(QListViewItem*)));
+
 	connect(lv, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&,int)),
 	        this, SLOT(on_contextMenuRequested(QListViewItem*)));
 }
 
 ListView::~ListView() {
 
-	if (isHistory)
-		git->cancelFileHistoryLoading(); // blocking
+	git->cancelDataLoading(fh); // blocking
 }
 
 void ListView::setupListView() {
@@ -65,7 +69,7 @@ void ListView::setupListView() {
 	lv->setItemMargin(0);
 	lv->setSorting(-1);
 
-	int adj = (isHistory) ? 0 : -1;
+	int adj = (fh) ? 0 : -1;
 
 	lv->setColumnWidthMode(GRAPH_COL, QListView::Manual);
 	lv->setColumnWidthMode(COMMIT_COL, QListView::Manual);
@@ -78,7 +82,7 @@ void ListView::setupListView() {
 	lv->setColumnWidth(AUTH_COL + adj, DEF_AUTH_COL_WIDTH);
 	lv->setColumnWidth(TIME_COL + adj, DEF_TIME_COL_WIDTH);
 
-	if (!isHistory) {
+	if (fh == NULL) {
 		lv->setColumnWidthMode(DUMMY_COL, QListView::Manual);
 		lv->hideColumn(DUMMY_COL);
 	}
@@ -98,9 +102,7 @@ void ListView::repaintAll(QFont& f) {
 
 void ListView::clear() {
 
-	if (isHistory)
-		git->cancelFileHistoryLoading(); // blocking
-
+	git->cancelDataLoading(fh);
 	lv->clear();
 	processedRevs = 0;
 	diffTarget = NULL; // avoid a dangling pointer
@@ -108,7 +110,7 @@ void ListView::clear() {
 
 void ListView::updateIdValues() {
 
-	if (!isHistory)
+	if (fh == NULL)
 		return;
 
 	int id = (int)lv->childCount();
@@ -133,7 +135,7 @@ void ListView::getSelectedItems(QStringList& selectedItems) {
 
 const QString ListView::getSha(int id) {
 
-	if (!isHistory)
+	if (fh == NULL)
 		return "";
 
 	// check to early skip common case of list mouse browsing
@@ -186,7 +188,7 @@ bool ListView::update() {
 		if (!newItem)
 			return false; // do not change anything in this case
 	}
-	if (!isHistory)
+	if (fh == NULL)
 		setHighlight(st->diffToSha());
 
 	if (shaChanged) {
@@ -201,16 +203,19 @@ bool ListView::update() {
 
 // ************************************ SLOTS ********************************
 
-void ListView::on_newRevsAdded(const QValueVector<QString>& shaVec) {
+void ListView::on_newRevsAdded(const FileHistory* f, const QValueVector<QString>& shaVec) {
+
+	if (f != fh)
+		return;
 
 	bool evenLine = !(lv->childCount() % 2);
 
 	if (lv->childCount() == 0)
 		lastItem = NULL;
 
-	for (uint i = processedRevs; i < shaVec.count(); ++i) {
+	for (uint i = processedRevs; i < shaVec.count(); i++) {
 		lastItem = new ListViewItem(lv, lastItem, git, shaVec[i], d->m()->pixmaps,
-		                            evenLine, d->m()->secs, isHistory);
+		                            evenLine, d->m()->secs, fh);
 		evenLine = !evenLine;
 	}
 	processedRevs = shaVec.count();
@@ -251,8 +256,9 @@ void ListView::on_onItem(QListViewItem*) {
 		if (!dragRevs.contains(ZERO_SHA)) {
 
 			const QString host(QString::fromLatin1("@") + d->m()->curDir);
-			for (uint i = 0; i < dragRevs.count(); i++)
-				dragRevs[i].append(host);
+			QStringList::iterator it(dragRevs.begin());
+			for ( ; it != dragRevs.end(); ++it)
+				(*it).append(host);
 
 			QDragObject* drObj = new QTextDrag(dragRevs.join("\n"), lv);
 			drObj->dragCopy(); // do NOT delete drObj. Blocking until drop event
@@ -330,6 +336,8 @@ bool ListView::eventFilter(QObject* obj, QEvent* ev) {
 bool ListView::filterRightButtonPressed(QMouseEvent* e) {
 
 	ListViewItem* item = static_cast<ListViewItem*>(lv->itemAt(e->pos()));
+	if (item == NULL)
+		return false;
 
 	if (e->state() == Qt::ControlButton) { // check for 'diff to' function
 
@@ -403,21 +411,21 @@ bool ListView::filterDropEvent(QDropEvent* e) {
 // ****************************** ListViewItem *****************************
 
 ListViewItem::ListViewItem(QListView* p, ListViewItem* a, Git* g, SCRef sha,
-	const QPtrVector<QPixmap>& pm, bool e, unsigned long t, bool h) :
+	const QPtrVector<QPixmap>& pm, bool e, unsigned long t, FileHistory* f) :
 	QListViewItem(p, a), pms(pm) {
 
 	git = g;
 	isEvenLine = e;
 	secs = t;
-	isHistory = h;
+	fh = f;
 	populated = isDiffTarget = isHighlighted = false;
 	setText(COMMIT_COL, sha); // item key, must be set from beginning
 }
 
 int ListViewItem::getLaneType(uint pos) const {
 
-	const Rev& c = *git->revLookup(text(COMMIT_COL), isHistory);
-	return (pos < c.lanes.count()) ? c.lanes[pos] : -1;
+	const Rev* r = git->revLookup(text(COMMIT_COL), fh); // cannot be NULL
+	return (pos < r->lanes.count() ? r->lanes[pos] : -1);
 }
 
 void ListViewItem::setDiffTarget(bool b) {
@@ -429,28 +437,24 @@ void ListViewItem::setDiffTarget(bool b) {
 void ListViewItem::paintCell(QPainter* p, const QColorGroup& cg,
                              int column, int width, int alignment) {
 	QColorGroup _cg(cg);
-	const Rev& r = *git->revLookup(text(COMMIT_COL), isHistory);
+	const Rev& c = *git->revLookup(text(COMMIT_COL), fh);
 
 	// lazy setup, only when visible
 	if (!populated)
-		setupData(r);
+		setupData(c);
 
 	// pixmap graph, separated from setupData to allow deleting
 	if (!pixmap(GRAPH_COL)) {
-		QPixmap* pm = getGraph(r);
+		QPixmap* pm = getGraph(c);
 		setPixmap(GRAPH_COL, *pm);
 		delete pm;
 	}
-
 	// adjust for annotation id column presence
-	int mycolumn = (isHistory) ? column : column + 1;
+	int mycolumn = (fh) ? column : column + 1;
 
-	// alternate background
-	if (isEvenLine && isInfoCol(mycolumn))
-		_cg.setColor(QColorGroup::Base, EVEN_LINE_COL);
-
-	if (!isEvenLine && isInfoCol(mycolumn))
-		_cg.setColor(QColorGroup::Base, ODD_LINE_COL);
+	// alternate background color
+	if (isInfoCol(mycolumn))
+		_cg.setColor(QColorGroup::Base, isEvenLine ? EVEN_LINE_COL : ODD_LINE_COL);
 
 	// tags, heads, refs and working dir colouring
 	if (mycolumn == LOG_COL) {
@@ -461,7 +465,7 @@ void ListViewItem::paintCell(QPainter* p, const QColorGroup& cg,
 			p->save();
 			p->setFont(f);
 		}
-		if (r.isDiffCache) {
+		if (c.isDiffCache) {
 			if (changedFiles(ZERO_SHA))
 				_cg.setColor(QColorGroup::Base, ORANGE);
 			else
@@ -478,23 +482,27 @@ void ListViewItem::paintCell(QPainter* p, const QColorGroup& cg,
 		p->restore();
 }
 
-void ListViewItem::addTextPixmap(SCRef text, const QColor& color) {
+void ListViewItem::addTextPixmap(SCRef text, const QColor& color, bool bold) {
 
-	int col = LOG_COL + ((isHistory) ? 0 : -1);
+	int col = LOG_COL + (fh ? 0 : -1);
 	QStringList sl = QStringList::split('\n', text);
 	loopList(it, sl) {
-		QPixmap* pm = doAddTextPixmap(*it, color, col);
+		QPixmap* pm = doAddTextPixmap(*it, color, col, bold);
 		setPixmap(col, *pm);
 		delete pm;
 	}
 }
 
-QPixmap* ListViewItem::doAddTextPixmap(SCRef text, const QColor& color, int col) {
+QPixmap* ListViewItem::doAddTextPixmap(SCRef text, const QColor& color, int col, bool bold) {
 
 	const QPixmap* oldPm = pixmap(col);
-	QFontMetrics fm(listView()->font());
+	QFont fnt(listView()->font());
+	if (bold)
+		fnt.setBold(true);
+
+	QFontMetrics fm(fnt);
 	int spacing = 2;
-	int pw = fm.boundingRect(text).width() + 2 * spacing;
+	int pw = fm.boundingRect(text).width() + 2 * (spacing + int(bold));
 	int ph = fm.height() + 1;
 	int ofs = oldPm ? oldPm->width() + 2 : 0;
 
@@ -509,7 +517,7 @@ QPixmap* ListViewItem::doAddTextPixmap(SCRef text, const QColor& color, int col)
 	}
 	p.setPen(Qt::black);
 	p.setBrush(color);
-	p.setFont(listView()->font());
+	p.setFont(fnt);
 	p.drawRect(ofs, 0, pw, ph);
 	p.drawText(ofs + spacing, fm.ascent(), text);
 	p.end();
@@ -518,14 +526,11 @@ QPixmap* ListViewItem::doAddTextPixmap(SCRef text, const QColor& color, int col)
 
 bool ListViewItem::changedFiles(SCRef c) {
 
-	bool changed = false;
 	const RevFile* f = git->getFiles(c);
-	for (uint i = 0; i < f->names.count(); ++i)
-		if (!f->status[i].endsWith(UNKNOWN)) {
-			changed = true;
-			break;
-		}
-	return changed;
+	for (uint i = 0; i < f->names.count(); i++)
+		if (!f->statusCmp(i, UNKNOWN))
+			return true;
+	return false;
 }
 
 void ListViewItem::setupData(const Rev& c) {
@@ -534,10 +539,10 @@ void ListViewItem::setupData(const Rev& c) {
 
 	// calculate lanes
 	if (c.lanes.count() == 0)
-		git->setLane(c.sha(), isHistory);
+		git->setLane(c.sha(), fh);
 
 	// set time/date column
-	int adj = (isHistory) ? 0 : -1;
+	int adj = (fh) ? 0 : -1;
 	if (c.sha() != ZERO_SHA) {
 		if (secs != 0) { // secs passed by MainImpl is 0 for absolute date
 			secs -= c.authorDate().toULong();
@@ -548,9 +553,10 @@ void ListViewItem::setupData(const Rev& c) {
 	setText(LOG_COL + adj, c.shortLog());
 	setText(AUTH_COL + adj, c.author());
 
-	if (c.isBranch || c.isCurrentBranch)
-		addTextPixmap(c.branch, c.isCurrentBranch ? Qt::green : DARK_GREEN);
-
+	if (c.isBranch || c.isCurrentBranch) {
+		QColor color(c.isCurrentBranch ? Qt::green : DARK_GREEN);
+		addTextPixmap(c.branch, color, c.isCurrentBranch);
+	}
 	if (c.isTag)
 		addTextPixmap(c.tag, Qt::yellow);
 
@@ -560,10 +566,10 @@ void ListViewItem::setupData(const Rev& c) {
 
 const QString ListViewItem::timeDiff(unsigned long secs) const {
 
-	uint days  = secs / (3600 * 24);
+	uint days  =  secs / (3600 * 24);
 	uint hours = (secs - days * 3600 * 24) / 3600;
 	uint min   = (secs - days * 3600 * 24 - hours * 3600) / 60;
-	uint sec   = secs - days * 3600 * 24 - hours * 3600 - min * 60;
+	uint sec   =  secs - days * 3600 * 24 - hours * 3600 - min * 60;
 	QString tmp;
 	if (days > 0)
 		tmp.append(QString::number(days) + "d ");
@@ -587,14 +593,14 @@ QPixmap* ListViewItem::getGraph(const Rev& c) {
 	QPixmap* pm = new QPixmap(pw * laneNum, ph);
 	pm->fill(ODD_LINE_COL); // faster then drawRect()
 	int mergeLane = -1;
-	for (uint j = 0; j < laneNum; ++j)
-		if (isMerge(lanes[j])) {
-			mergeLane = j;
+	for (uint i = 0; i < laneNum; i++)
+		if (isMerge(lanes[i])) {
+			mergeLane = i;
 			break;
 		}
 	QPainter p;
 	p.begin(pm);
-	for (uint i = 0; i < laneNum; ++i) {
+	for (uint i = 0; i < laneNum; i++) {
 
 		int ln = lanes[i], idx;
 		if (ln == EMPTY)
@@ -605,8 +611,8 @@ QPixmap* ListViewItem::getGraph(const Rev& c) {
 		else
 			idx = COLORS_NUM * (ln - 1);
 
-		int col = (isHead(ln) || isTail(ln) || isJoin(ln) ||
-				ln == CROSS_EMPTY) ? mergeLane : i;
+		int col = (   isHead(ln) || isTail(ln) || isJoin(ln)
+		           || ln == CROSS_EMPTY) ? mergeLane : i;
 
 		idx += col % COLORS_NUM;
 		p.drawPixmap(i * pw, 0, *pms[idx]);
@@ -621,11 +627,11 @@ QPixmap* ListViewItem::getGraph(const Rev& c) {
 
 // ***************** MainImpl related methods *********************
 
-#define P_OR	pw/2,ph/2
-#define P_0	pw,ph/2
-#define P_90	pw/2,0
-#define P_180	0,ph/2
-#define P_270	pw/2,ph
+#define P_OR  pw/2, ph/2
+#define P_0   pw,   ph/2
+#define P_90  pw/2, 0
+#define P_180 0,    ph/2
+#define P_270 pw/2, ph
 
 void MainImpl::setupPixmaps(int h) {
 
@@ -647,14 +653,13 @@ void MainImpl::setupPixmaps(int h) {
 	QBrush myWhiteBrush(ODD_LINE_COL, Qt::SolidPattern);
 	QColor colors[COLORS_NUM] = {Qt::black, Qt::red, DARK_GREEN, Qt::blue,
 	                             Qt::darkGray, BROWN, Qt::magenta, ORANGE};
-
 	pixmaps.clear();
 	pixmaps.resize(LANE_TYPES_NUM * COLORS_NUM);
 
 	for (int i = 0; i < LANE_TYPES_NUM * COLORS_NUM; i++) {
 
 		int type = (i / COLORS_NUM) + 1;
-		QColor myColorNum = colors[i % COLORS_NUM];
+		QColor myColorNum(colors[i % COLORS_NUM]);
 		QBrush myBrush(myColorNum, Qt::SolidPattern);
 
 		QPixmap* pm = new QPixmap(pw, ph);
