@@ -15,11 +15,12 @@ class QFont;
 class Git;
 class StateInfo;
 class Domain;
+class FileHistory;
 
 class ListViewItem: public QListViewItem {
 public:
 	ListViewItem(QListView* p, ListViewItem* a, Git* g, SCRef sha,
-	             const QPtrVector<QPixmap>& pm, bool e, unsigned long t, bool h);
+		     const QPtrVector<QPixmap>& pm, bool e, unsigned long t, FileHistory* f);
 
 	int getLaneType(uint pos) const;
 	void setDiffTarget(bool b);
@@ -32,21 +33,22 @@ public:
 private:
 	void setupData(const Rev& c);
 	QPixmap* getGraph(const Rev& c);
-	void addTextPixmap(SCRef text, const QColor& color);
-	QPixmap* doAddTextPixmap(SCRef text, const QColor& color, int col);
+	void addTextPixmap(SCRef text, const QColor& color, bool bold = false);
+	QPixmap* doAddTextPixmap(SCRef text, const QColor& color, int col, bool bold);
 	const QString timeDiff(unsigned long secs) const;
 	bool changedFiles(SCRef c);
 
 	Git* git;
+	FileHistory* fh;
 	const QPtrVector<QPixmap>& pms;
 	unsigned long secs;
-	bool populated, isEvenLine, isHighlighted, isHistory, isDiffTarget;
+	bool populated, isEvenLine, isHighlighted, isDiffTarget;
 };
 
 class ListView: public QObject {
 Q_OBJECT
 public:
-	ListView(Domain* d, Git* g, QListView* l, bool h);
+	ListView(Domain* d, Git* g, QListView* l, FileHistory* f);
 	~ListView();
 	void clear();
 	const QString getSha(int id);
@@ -64,7 +66,7 @@ signals:
 	void contextMenu(const QString&, int);
 
 public slots:
-	void on_newRevsAdded(const QValueVector<QString>& shaVec);
+	void on_newRevsAdded(const FileHistory* fh, const QValueVector<QString>& shaVec);
 
 protected:
 	bool eventFilter(QObject* obj, QEvent* ev);
@@ -89,7 +91,7 @@ private:
 	Git* git;
 	QListView* lv;
 	StateInfo* st;
-	bool isHistory;
+	FileHistory* fh;
 	ListViewItem* lastItem; // QListView::lastItem() is slow
 	ListViewItem* diffTarget; // cannot use QGuardedPtr, not QObject inherited
 	uint processedRevs;
