@@ -6,6 +6,7 @@
 	Copyright: See COPYING file that comes with this distribution
 
 */
+#include <qtimer.h>
 #include <qlineedit.h>
 #include <qtextbrowser.h>
 #include <qlistview.h>
@@ -37,9 +38,11 @@
 #include "help.h"
 #include "helpbase.h"
 #include "consoleimpl.h"
-#include "macroimpl.h"
+#include "customactionimpl.h"
 #include "settingsimpl.h"
+#include "revbase.h"
 #include "filebase.h"
+#include "patchbase.h"
 #include "common.h"
 #include "git.h"
 #include "listview.h"
@@ -102,6 +105,15 @@ MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
 	longLogRE.setMinimal(true);
 	longLogRE.setCaseSensitive(false);
 
+	// set-up tab view
+	delete tabWdg->currentPage(); // cannot be done in Qt Designer
+	rv = new RevsView(this, git);
+	pushButtonCloseTab->reparent(tabWdg, QPoint());
+	tabWdg->setCornerWidget(pushButtonCloseTab);
+
+	// set-up tree view
+	treeView->hide();
+
 	// set-up menu for recent visited repositories
 	connect(File, SIGNAL(activated(int)), this, SLOT(on_openRecent_activated(int)));
 	recentRepoMenuPos = 0;
@@ -109,14 +121,14 @@ MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
 		recentRepoMenuPos++;
 	doUpdateRecentRepoMenu("");
 
-	// set-up menu for macros
-	connect(Actions, SIGNAL(activated(int)), this, SLOT(on_macro_activated(int)));
+	// set-up menu for custom actions
+	connect(Actions, SIGNAL(activated(int)), this, SLOT(on_customAction_activated(int)));
 	QSettings set;
 	QStringList sl = QStringList::split(",", set.readEntry(APP_KEY + MCR_LIST_KEY, ""));
-	doUpdateMacroMenu(sl);
+	doUpdateCustomActionMenu(sl);
 
 	// create light and dark colors for alternate background
-	QColor l(listViewLog->paletteBackgroundColor());
+	QColor l(rv->tab()->listViewLog->paletteBackgroundColor());
 	QColor d(int(l.red() * 0.97), int(l.green() * 0.97), int(l.blue() * 0.97));
 	QGit::ODD_LINE_COL = l;
 	QGit::EVEN_LINE_COL = d;
@@ -127,57 +139,33 @@ MainImpl::MainImpl(const QString& cd, QWidget* p, const char *name) :
 	int wd = lineEditSHA->fontMetrics().boundingRect(tmp).width();
 	lineEditSHA->setMinimumWidth(wd);
 
-	// set-up tree view
-	treeView->hide();
-
-	// set-up tab view
-	revTab = tabWdg->page(0);
-	patchTab = tabWdg->page(1);
-	fileTab = new TabFile(this);
-	tabWdg->addTab(fileTab, "File");
-
-	pushButtonCloseTab->reparent(tabWdg, QPoint());
-	tabWdg->setCornerWidget(pushButtonCloseTab);
-	tabWdg->removePage(patchTab);
-	tabWdg->removePage(fileTab);
-
-	rv = new RevsView(this, git);
-
 	pixmaps.setAutoDelete(true);
 	adjustFontSize(0); // default is system-wide setting
 
-	connect(git, SIGNAL(newRevsAdded(const QValueVector<QString>&)),
-	        this, SLOT(on_newRevsAdded()));
+	connect(git, SIGNAL(newRevsAdded(const FileHistory*, const QValueVector<QString>&)),
+	        this, SLOT(on_newRevsAdded(const FileHistory*, const QValueVector<QString>&)));
 
 	// connect cross-domain update signals
-	connect(listViewLog, SIGNAL(doubleClicked(QListViewItem*)),
-             this, SLOT(on_listViewLog_doubleClicked(QListViewItem*)));
-	connect(listBoxFiles, SIGNAL(doubleClicked(QListBoxItem*)),
-             this, SLOT(on_fileList_doubleClicked(QListBoxItem*)));
-	connect(listBoxFiles_2, SIGNAL(doubleClicked(QListBoxItem*)),
+	connect(rv->tab()->listViewLog, SIGNAL(doubleClicked(QListViewItem*)),
+	        this, SLOT(on_listViewLog_doubleClicked(QListViewItem*)));
+
+	connect(rv->tab()->listBoxFiles, SIGNAL(doubleClicked(QListBoxItem*)),
 	        this, SLOT(on_fileList_doubleClicked(QListBoxItem*)));
+
 	connect(treeView, SIGNAL(doubleClicked(QListViewItem*)),
 	        this, SLOT(on_treeView_doubleClicked(QListViewItem*)));
-	connect(fileTab->histListView, SIGNAL(doubleClicked(QListViewItem*)),
-	        this, SLOT(on_histListView_doubleClicked(QListViewItem*)));
 
-	// some stuff must be done outside c'tor
+	// MainImpl c'tor is called before to enter event loop,
+	// but some stuff requires event loop to init properly
 	startUpDir = (cd.isEmpty()) ? QDir::current().absPath() : cd;
-	QTimer::singleShot(10, this, SLOT(afterCtorInit()));
+	QTimer::singleShot(10, this, SLOT(initWithEventLoopActive()));
 }
 
-void MainImpl::afterCtorInit() {
+void MainImpl::initWithEventLoopActive() {
 
 	setRepository(startUpDir, false, false);
 }
 
-void MainImpl::buttonGroupDiff_clicked(int diffType) {
-// due to a strange issue we have to do this trampoline
-// instead of connect the signal directly with PatchView slot
-
-	rv->patchView->on_buttonGroupDiff_clicked(diffType);
-}
-
 void MainImpl::lineEditSHA_returnPressed() {
 
 	rv->st.setSha(lineEditSHA->text());
@@ -266,6 +254,7 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 	// we need to guard against reentrancy here.
 	if (setRepositoryBusy)
 		return;
+
 	setRepositoryBusy = true;
 
 	EM_RAISE(exSetRepositoryCalled);
@@ -276,7 +265,7 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 		bool archiveChanged;
 		curDir = git->getBaseDir(&archiveChanged, newDir);
 
-		git->stop(!archiveChanged); // this stops all pending processing
+		git->stop(archiveChanged); // this stops all pending processing
 
 		if (archiveChanged && refresh)
 			dbs("ASSERT in setRepository: different dir with no range select");
@@ -284,10 +273,9 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 		// now we can clear all our data
 		setCaption(curDir + " - QGit");
 		rv->clear(refresh && keepSelection);
-		if (fv && archiveChanged) {
-			tabWdg->removePage(fileTab);
-			delete fv;
-		}
+		if (archiveChanged)
+			emit closeAllTabs();
+
 		// disable all actions
 		updateGlobalActions(false);
 		updateContextActions("", "", false, false);
@@ -297,10 +285,10 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 
 		if (testFlag(REL_DATE_F)) {
 			secs = QDateTime::currentDateTime().toTime_t();
-			listViewLog->setColumnText(TIME_COL + adj, "Last Change");
+			rv->tab()->listViewLog->setColumnText(TIME_COL + adj, "Last Change");
 		} else {
 			secs = 0;
-			listViewLog->setColumnText(TIME_COL + adj, "Author Date");
+			rv->tab()->listViewLog->setColumnText(TIME_COL + adj, "Author Date");
 		}
 		if (ActFilterTree->isOn())
 			setCaption(caption() + " - FILTER ON < " +
@@ -313,11 +301,9 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 
 		bool quit;
 		bool ret = git->init(curDir, !refresh, filterList, &quit);
-		if (quit) {
-			EM_REMOVE(exExiting);
-			ActClose->activate();
-			return;
-		}
+		if (quit)
+			goto exit;
+
 		if (!ret)
 			statusBar()->message("Not a git archive");
 
@@ -330,10 +316,15 @@ void MainImpl::setRepository(SCRef newDir, bool refresh, bool keepSelection,
 		ActCheckWorkDir->setOn(testFlag(DIFF_INDEX_F));
 		updateGlobalActions(ret);
 
+exit:
 		setRepositoryBusy = false;
-
 		EM_REMOVE(exExiting);
 
+		if (quit && !startUpDir.isEmpty())
+			ActClose->activate();
+
+		startUpDir = ""; // one shot
+
 	} catch (int i) {
 		EM_REMOVE(exExiting);
 
@@ -354,21 +345,22 @@ void MainImpl::updateGlobalActions(bool b) {
 	ActCheckWorkDir->setEnabled(b);
 	ActViewRev->setEnabled(b);
 	ActViewDiff->setEnabled(b);
+	ActViewDiffNewTab->setEnabled(b && firstTab<PatchView>());
 	ActShowTree->setEnabled(b);
 	ActMailApplyPatch->setEnabled(b);
 	ActMailFormatPatch->setEnabled(b);
 
-	revTab->setEnabled(b);
-	patchTab->setEnabled(b);
+	rv->setEnabled(b);
 }
 
 void MainImpl::updateContextActions(SCRef newRevSha, SCRef newFileName,
                                     bool isDir, bool found) {
 
 	bool pathActionsEnabled = !newFileName.isEmpty();
-	bool fileActionsEnabled = pathActionsEnabled && !isDir;
+	bool fileActionsEnabled = (pathActionsEnabled && !isDir);
 
 	ActViewFile->setEnabled(fileActionsEnabled);
+	ActViewFileNewTab->setEnabled(fileActionsEnabled && firstTab<FileView>());
 	ActExternalDiff->setEnabled(fileActionsEnabled);
 	ActSaveFile->setEnabled(fileActionsEnabled);
 	ActFilterTree->setEnabled(pathActionsEnabled || ActFilterTree->isOn());
@@ -404,14 +396,13 @@ void MainImpl::on_histListView_doubleClicked(QListViewItem* item) {
 
 void MainImpl::on_fileList_doubleClicked(QListBoxItem* item) {
 
-	if (rv->st.isMerge() && (item == listBoxFiles->firstItem() ||
-	    item == listBoxFiles_2->firstItem()))
+	if (item && rv->st.isMerge() && item->prev() == 0)
 		return;
 
-	if (item && item->listBox() == listBoxFiles && ActViewDiff->isEnabled())
+	if (item && item->listBox() == rv->tab()->listBoxFiles && ActViewDiff->isEnabled())
 		ActViewDiff->activate();
 
-	if (item && item->listBox() == listBoxFiles_2 && ActViewFile->isEnabled())
+	if (item && item->listBox() != rv->tab()->listBoxFiles && ActViewFile->isEnabled())
 		ActViewFile->activate();
 }
 
@@ -423,16 +414,20 @@ void MainImpl::on_treeView_doubleClicked(QListViewItem* item) {
 
 void MainImpl::pushButtonCloseTab_clicked() {
 
-	switch (tabType(tabWdg->currentPage())) {
+	int curPos = tabWdg->currentPageIndex();
+	QObject* t;
+	switch (currentTabType(&t)) {
 	case TAB_REV:
 		break;
 	case TAB_PATCH:
-		tabWdg->removePage(patchTab);
-		rv->setPatchView(false);
+		delete t;
+		emit tabClosed(curPos);
+		ActViewDiffNewTab->setEnabled(ActViewDiff->isEnabled() && firstTab<PatchView>());
 		break;
 	case TAB_FILE:
-		tabWdg->removePage(fileTab);
-		delete fv;
+		delete t;
+		emit tabClosed(curPos);
+		ActViewFileNewTab->setEnabled(ActViewFile->isEnabled() && firstTab<FileView>());
 		break;
 	default:
 		dbs("ASSERT in pushButtonCloseTab_clicked: unknown current page");
@@ -442,36 +437,55 @@ void MainImpl::pushButtonCloseTab_clicked() {
 
 void MainImpl::ActViewRev_activated() {
 
-	tabWdg->setCurrentPage(tabWdg->indexOf(revTab));
-	if (fv) {
-		rv->st = fv->st;
+	QObject* t;
+	if (currentTabType(&t) == TAB_FILE) {
+		rv->st = (static_cast<FileView*>(t))->st;
 		UPDATE_DOMAIN(rv);
 	}
+	tabWdg->setCurrentPage(rv->tabPos());
 }
 
 void MainImpl::ActViewFile_activated() {
 
+	openFileTab(firstTab<FileView>());
+}
+
+void MainImpl::ActViewFileNewTab_activated() {
+
+	openFileTab();
+}
+
+void MainImpl::openFileTab(FileView* fv) {
+
 	if (!fv) {
-		fv = new FileView(this, git, fileTab);
-		tabWdg->addTab(fileTab, "");
+		fv = new FileView(this, git);
+
+		connect(fv->tab()->histListView, SIGNAL(doubleClicked(QListViewItem*)),
+		        this, SLOT(on_histListView_doubleClicked(QListViewItem*)));
+
+		connect(this, SIGNAL(closeAllTabs()), fv, SLOT(on_closeAllTabs()));
+
+		ActViewFileNewTab->setEnabled(ActViewFile->isEnabled());
 	}
-	tabWdg->setCurrentPage(tabWdg->indexOf(fileTab));
+	tabWdg->setCurrentPage(fv->tabPos());
 	fv->st = rv->st;
 	UPDATE_DOMAIN(fv);
 }
 
 void MainImpl::ActViewDiff_activated() {
 
-	bool isFilePage = (tabWdg->currentPageIndex() == tabWdg->indexOf(fileTab));
-	if (!rv->patchView) {
-		rv->setPatchView(true);
-		tabWdg->insertTab(patchTab, "Patch", tabWdg->indexOf(revTab) + 1);
-	}
-	tabWdg->setCurrentPage(tabWdg->indexOf(patchTab));
-	if (isFilePage) {
-		rv->st = fv->st;
+	QObject* t;
+	if (currentTabType(&t) == TAB_FILE) {
+		rv->st = (static_cast<FileView*>(t))->st;
 		UPDATE_DOMAIN(rv);
 	}
+	rv->viewPatch(false);
+	ActViewDiffNewTab->setEnabled(true);
+}
+
+void MainImpl::ActViewDiffNewTab_activated() {
+
+	rv->viewPatch(true);
 }
 
 bool MainImpl::eventFilter(QObject* obj, QEvent* ev) {
@@ -495,7 +509,10 @@ bool MainImpl::eventFilter(QObject* obj, QEvent* ev) {
 
 // ******************************* Filter ******************************
 
-void MainImpl::on_newRevsAdded() {
+void MainImpl::on_newRevsAdded(const FileHistory* fh, const QValueVector<QString>&) {
+
+	if (fh)
+		return;
 
 	if (toolButtonFilter->isOn())
 		toolButtonFilter_toggled(true); // filter again on new arrived data
@@ -504,8 +521,8 @@ void MainImpl::on_newRevsAdded() {
 		toolButtonBold_toggled(true); // filter again on new arrived data
 
 	// first rev could be a StGIT unapplied patch so check more then once
-	if (!ActCommit->isEnabled() && !git->isCommittingMerge() &&
-	   (!git->isNothingToCommit() || git->isUnknownFiles()))
+	if (   (!git->isNothingToCommit() || git->isUnknownFiles())
+	    && !ActCommit->isEnabled() && !git->isCommittingMerge())
 		ActCommit_setEnabled(true);
 }
 
@@ -538,23 +555,28 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 	QMap<QString, bool> shaMap;
 	bool descNeedsUpdate, patchNeedsUpdate, isRegExp;
 	descNeedsUpdate = patchNeedsUpdate = isRegExp = false;
-	int colNum = 0, idx = cmbSearch->currentItem();
+	int idx = cmbSearch->currentItem(), colNum = 0;
 	if (isOn) {
-		if (idx == 0) {
+		switch (idx) {
+		case 0:
 			colNum = LOG_COL;
 			shortLogRE.setPattern(filter);
 			descNeedsUpdate = true;
-		}
-		if (idx == 1) {
+			break;
+		case 1:
 			colNum = LOG_MSG_COL;
 			longLogRE.setPattern(filter);
 			descNeedsUpdate = true;
-		}
-		if (idx == 2)
+			break;
+		case 2:
 			colNum = AUTH_COL;
-		if (idx == 3)
+			break;
+		case 3:
 			colNum = COMMIT_COL;
-		if (idx == 4 || idx == 5 || idx == 6) {
+			break;
+		case 4:
+		case 5:
+		case 6:
 			colNum = SHA_MAP_COL;
 			QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 			EM_PROCESS_EVENTS; // to let paint wait cursor
@@ -570,16 +592,17 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 				patchNeedsUpdate = (shaMap.count() > 0);
 			}
 			QApplication::restoreOverrideCursor();
+			break;
 		}
 	} else {
+		patchNeedsUpdate = ((idx == 5) || (idx == 6));
 		descNeedsUpdate = (!shortLogRE.isEmpty() || !longLogRE.isEmpty());
 		shortLogRE.setPattern("");
 		longLogRE.setPattern("");
-		patchNeedsUpdate = ((idx == 5) || (idx == 6));
 	}
-	QListViewItemIterator it(listViewLog);
 	bool evenLine = false;
 	int visibleCnt = 0;
+	QListViewItemIterator it(rv->tab()->listViewLog);
 	while (it.current()) {
 		ListViewItem* item = static_cast<ListViewItem*>(it.current());
 		if (isOn) {
@@ -602,9 +625,10 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 		}
 		++it;
 	}
-	if (descNeedsUpdate)
-		textBrowserDesc->setText(git->getDesc(rv->st.sha(), shortLogRE, longLogRE));
-
+	if (descNeedsUpdate) {
+		SCRef d(git->getDesc(rv->st.sha(), shortLogRE, longLogRE));
+		rv->tab()->textBrowserDesc->setText(d);
+	}
 	if (patchNeedsUpdate)
 		emit highlightPatch(isOn ? filter : "", isRegExp);
 
@@ -613,7 +637,8 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 		                          "button to remove the filter").arg(visibleCnt));
 		statusBar()->message(tmp);
 	} else {
-		listViewLog->ensureItemVisible(listViewLog->currentItem());
+		QListView* lv = rv->tab()->listViewLog;
+		lv->ensureItemVisible(lv->currentItem());
 		statusBar()->clear();
 	}
 	scrollListView(1); // hack to force a repaint
@@ -622,12 +647,12 @@ void MainImpl::filterList(bool isOn, bool onlyHighlight) {
 
 bool MainImpl::passFilter(ListViewItem* item, SCRef filter, int colNum,
                           const QMap<QString, bool>& shaMap) {
-	QString field;
-	if (colNum == SHA_MAP_COL) {
+
+	if (colNum == SHA_MAP_COL)
 		// in this case shaMap contains all good sha to search for
-		field = item->text(COMMIT_COL);
-		return shaMap.contains(field);
-	}
+		return shaMap.contains(item->text(COMMIT_COL));
+
+	QString field;
 	if (colNum != LOG_MSG_COL) {
 		int adj = (colNum != COMMIT_COL) ? -1 : 0;
 		field = item->text(colNum + adj);
@@ -636,18 +661,24 @@ bool MainImpl::passFilter(ListViewItem* item, SCRef filter, int colNum,
 		const Rev& c = *git->revLookup(item->text(COMMIT_COL));
 		if (colNum == LOG_COL)
 			field = c.shortLog();
-		if (colNum == AUTH_COL)
+		else if (colNum == AUTH_COL)
 			field = c.author();
-		if (colNum == LOG_MSG_COL)
+		else if (colNum == LOG_MSG_COL)
 			field = c.longLog();
 	}
 	// wildcard search, case insensitive
-	int idx = field.find(QRegExp(filter, false, true));
-	return (idx != -1);
+	return (field.find(QRegExp(filter, false, true)) != -1);
 }
 
 void MainImpl::customEvent(QCustomEvent* e) {
 
+	BaseEvent* de = dynamic_cast<BaseEvent*>(e);
+	if (de == NULL) {
+		MainBase::customEvent(e);
+		return;
+	}
+	SCRef data = de->data();
+
 	switch (e->type()) {
 	case ERROR_EV: {
 		QApplication::setOverrideCursor(QCursor(Qt::ArrowCursor));
@@ -659,50 +690,104 @@ void MainImpl::customEvent(QCustomEvent* e) {
 		QApplication::restoreOverrideCursor(); }
 		break;
 	case MSG_EV:
-		statusBar()->message(((MessageEvent*)e)->data());
+		statusBar()->message(data);
 		break;
 	case POPUP_LIST_EV:
-		doContexPopup(((DeferredPopupEvent*)e)->data());
+		doContexPopup(data);
 		break;
 	case POPUP_FILE_EV:
 	case POPUP_TREE_EV:
-		doFileContexPopup(((DeferredPopupEvent*)e)->data(), e->type());
+		doFileContexPopup(data, e->type());
 		break;
 	default:
 		qDebug("ASSERT in MainImpl::customEvent unhandled event %i", e->type());
 		break;
 	}
-	MainBase::customEvent(e);
 }
 
-int MainImpl::tabType(QWidget* tb) {
+int MainImpl::currentTabType(QObject** t) {
 
-	int idx = tabWdg->indexOf(tb);
-
-	if (idx == tabWdg->indexOf(revTab))
+	*t = NULL;
+	int curPos = tabWdg->currentPageIndex();
+	if (curPos == rv->tabPos()) {
+		*t = rv;
 		return TAB_REV;
-
-	if (idx == tabWdg->indexOf(patchTab))
+	}
+	QPtrList<PatchView>* l = getTabs<PatchView>(curPos);
+	if (l->count() > 0) {
+		*t = l->first();
+		delete l;
 		return TAB_PATCH;
-
-	if (idx == tabWdg->indexOf(fileTab))
+	}
+	delete l;
+	QPtrList<FileView>* l2 = getTabs<FileView>(curPos);
+	if (l2->count() > 0) {
+		*t = l2->first();
+		delete l2;
 		return TAB_FILE;
+	}
+	if (l2->count() > 0)
+		dbs("ASSERT in tabType file not found");
 
-	return 0;
+	delete l2;
+	return -1;
 }
 
-void MainImpl::tabWdg_currentChanged(QWidget*) {
+template<class X> QPtrList<X>* MainImpl::getTabs(int tabPos) {
+
+	X dummy;
+	QObjectList* l = this->queryList(dummy.className());
+	QPtrList<X>* ret = new QPtrList<X>;
+	for (QObject* item = l->first(); item; item = l->next()) {
+
+		X* x = static_cast<X*>(item);
+		if (tabPos == -1 || x->tabPos() == tabPos)
+			ret->append(x);
+	}
+	delete l;
+	return ret; // 'ret' must be deleted by caller
+}
+
+template<class X> X* MainImpl::firstTab(int startPos) {
+
+	int minVal = 99, firstVal = 99;
+	X* min = NULL;
+	X* first = NULL;
+	QPtrList<X>* l = getTabs<X>();
+	for (X* d = l->first(); d; d = l->next()) {
+
+		if (d->tabPos() < minVal) {
+			minVal = d->tabPos();
+			min = d;
+		}
+		if (d->tabPos() < firstVal && d->tabPos() > startPos) {
+			firstVal = d->tabPos();
+			first = d;
+		}
+	}
+	delete l;
+	return first ? first : min;
+}
+
+void MainImpl::tabWdg_currentChanged(QWidget* w) {
+
+	if (w == NULL)
+		return;
 
 	// set correct focus for keyboard browsing
-	switch (tabType(tabWdg->currentPage())) {
+	QObject* t;
+	switch (currentTabType(&t)) {
 	case TAB_REV:
-		listViewLog->setFocus();
+		static_cast<RevsView*>(t)->tab()->listViewLog->setFocus();
+		pushButtonCloseTab->setEnabled(false);
 		break;
 	case TAB_PATCH:
-		textEditDiff->setFocus();
+		static_cast<PatchView*>(t)->tab()->textEditDiff->setFocus();
+		pushButtonCloseTab->setEnabled(true);
 		break;
 	case TAB_FILE:
-		fileTab->histListView->setFocus();
+		static_cast<FileView*>(t)->tab()->histListView->setFocus();
+		pushButtonCloseTab->setEnabled(true);
 		break;
 	default:
 		dbs("ASSERT in tabWdg_currentChanged: unknown current page");
@@ -752,39 +837,41 @@ void MainImpl::accelActivated(int id) {
 		scrollTextEdit(1);
 		break;
 	case KEY_R:
-		tabWdg->setCurrentPage(tabWdg->indexOf(revTab));
+		tabWdg->setCurrentPage(rv->tabPos());
 		break;
 	case KEY_P:
-		tabWdg->setCurrentPage(tabWdg->indexOf(patchTab));
-		break;
-	case KEY_F:
-		tabWdg->setCurrentPage(tabWdg->indexOf(fileTab));
+	case KEY_F:{
+		int cp = tabWdg->currentPageIndex();
+		Domain* d = (id == KEY_P) ? static_cast<Domain*>(firstTab<PatchView>(cp)) :
+		                            static_cast<Domain*>(firstTab<FileView>(cp));
+		if (d)
+			tabWdg->setCurrentPage(d->tabPos()); }
 		break;
 	}
 }
 
 void MainImpl::setupAccelerator(QAccel* accel) {
 
-	accel->insertItem(Key_Up, KEY_UP);
-	accel->insertItem(Key_I, KEY_UP);
-	accel->insertItem(Key_Down, KEY_DOWN);
-	accel->insertItem(SHIFT+Key_Up, SHIFT_KEY_UP);
+	accel->insertItem(Key_Up,         KEY_UP);
+	accel->insertItem(Key_I,          KEY_UP);
+	accel->insertItem(Key_Down,       KEY_DOWN);
+	accel->insertItem(Key_N,          KEY_DOWN);
+	accel->insertItem(Key_K,          KEY_DOWN);
+	accel->insertItem(Key_Left,       KEY_LEFT);
+	accel->insertItem(Key_Right,      KEY_RIGHT);
+	accel->insertItem(SHIFT+Key_Up,   SHIFT_KEY_UP);
 	accel->insertItem(SHIFT+Key_Down, SHIFT_KEY_DOWN);
-	accel->insertItem(Key_N, KEY_DOWN);
-	accel->insertItem(Key_K, KEY_DOWN);
-	accel->insertItem(Key_Left, KEY_LEFT);
-	accel->insertItem(Key_Right, KEY_RIGHT);
-	accel->insertItem(CTRL+Key_Plus, CTRL_PLUS);
+	accel->insertItem(CTRL+Key_Plus,  CTRL_PLUS);
 	accel->insertItem(CTRL+Key_Minus, CTRL_MINUS);
-	accel->insertItem(Key_U, KEY_U);
-	accel->insertItem(Key_D, KEY_D);
-	accel->insertItem(Key_Delete, KEY_DELETE);
-	accel->insertItem(Key_B, KEY_B);
-	accel->insertItem(Key_Backspace, KEY_BCKSPC);
-	accel->insertItem(Key_Space, KEY_SPACE);
-	accel->insertItem(Key_R, KEY_R);
-	accel->insertItem(Key_P, KEY_P);
-	accel->insertItem(Key_F, KEY_F);
+	accel->insertItem(Key_U,          KEY_U);
+	accel->insertItem(Key_D,          KEY_D);
+	accel->insertItem(Key_Delete,     KEY_DELETE);
+	accel->insertItem(Key_B,          KEY_B);
+	accel->insertItem(Key_Backspace,  KEY_BCKSPC);
+	accel->insertItem(Key_Space,      KEY_SPACE);
+	accel->insertItem(Key_R,          KEY_R);
+	accel->insertItem(Key_P,          KEY_P);
+	accel->insertItem(Key_F,          KEY_F);
 
 	connect(accel, SIGNAL(activated(int)), this, SLOT(accelActivated(int)));
 }
@@ -794,17 +881,19 @@ void MainImpl::goMatch(int delta) {
 	if (!toolButtonBold->isOn())
 		return;
 
-	QListViewItemIterator it(listViewLog->currentItem());
+	QListViewItemIterator it(rv->tab()->listViewLog->currentItem());
 	if (delta > 0)
 		++it;
 	else
 		--it;
+
 	while (it.current()) {
 		ListViewItem* item = static_cast<ListViewItem*>(it.current());
 		if (item->highlighted()) {
-			listViewLog->clearSelection();
-			listViewLog->setCurrentItem(item);
-			listViewLog->ensureItemVisible(listViewLog->currentItem());
+			QListView* lv = rv->tab()->listViewLog;
+			lv->clearSelection();
+			lv->setCurrentItem(item);
+			lv->ensureItemVisible(lv->currentItem());
 			return;
 		}
 		if (delta > 0)
@@ -817,15 +906,16 @@ void MainImpl::goMatch(int delta) {
 QTextEdit* MainImpl::getCurrentTextEdit() {
 
 	QTextEdit* te = NULL;
-	switch (tabType(tabWdg->currentPage())) {
+	QObject* t;
+	switch (currentTabType(&t)) {
 	case TAB_REV:
-		te = textBrowserDesc;
+		te = static_cast<RevsView*>(t)->tab()->textBrowserDesc;
 		break;
 	case TAB_PATCH:
-		te = textEditDiff;
+		te = static_cast<PatchView*>(t)->tab()->textEditDiff;
 		break;
 	case TAB_FILE:
-		te = fileTab->textEditFile;
+		te = static_cast<FileView*>(t)->tab()->textEditFile;
 		break;
 	default:
 		break;
@@ -839,7 +929,6 @@ void MainImpl::scrollTextEdit(int delta) {
 	if (!te)
 		return;
 
-	// now scroll by delta
 	int h = te->visibleHeight();
 	int ls = te->fontMetrics().lineSpacing();
 	if (delta == 1 || delta == -1) {
@@ -852,12 +941,13 @@ void MainImpl::scrollTextEdit(int delta) {
 void MainImpl::scrollListView(int delta) {
 
 	QWidget* lv = NULL;
-	switch (tabType(tabWdg->currentPage())) {
+	QObject* t;
+	switch (currentTabType(&t)) {
 	case TAB_REV:
-		lv = listViewLog;
+		lv = static_cast<RevsView*>(t)->tab()->listViewLog;
 		break;
 	case TAB_FILE:
-		lv = fileTab->histListView;
+		lv = static_cast<FileView*>(t)->tab()->histListView;
 		break;
 	default:
 		lv = qApp->focusWidget();
@@ -876,12 +966,13 @@ void MainImpl::scrollListView(int delta) {
 void MainImpl::adjustFontSize(int delta) {
 // font size is adjusted on a 'per instance' base and only on list views
 
-	QFont f(listViewLog->font());
+	QListView* lv = rv->tab()->listViewLog; // we cannot use a different list view. Wrong sizes!
+	QFont f(lv->font());
 	f.setPointSize(f.pointSize() + delta);
 
 	// little hack to read new item height
-	listViewLog->setFont(f); // we cannot use a different list view. Wrong sizes!
-	QListViewItem* item = new QListViewItem(listViewLog);
+	lv->setFont(f);
+	QListViewItem* item = new QListViewItem(lv);
 	int h = item->height();
 	delete item;
 
@@ -889,8 +980,10 @@ void MainImpl::adjustFontSize(int delta) {
 
 	// clear main view graphs
 	rv->listViewLog->repaintAll(f);
-	if (fv)
-		fv->histListView->repaintAll(f);
+	QPtrList<FileView>* l = getTabs<FileView>();
+	for (FileView* item = l->first(); item; item = l->next())
+		item->histListView->repaintAll(f);
+	delete l;
 }
 
 // ****************************** Menu *********************************
@@ -921,7 +1014,7 @@ void MainImpl::updateRecentRepoMenu(SCRef newEntry) {
 			w->doUpdateRecentRepoMenu(newEntry);
 		++it;
 	}
-	delete list; // list is created using new by topLevelWidgets()
+	delete list;
 }
 
 void MainImpl::doUpdateRecentRepoMenu(SCRef newEntry) {
@@ -958,27 +1051,30 @@ void MainImpl::doContexPopup(SCRef sha) {
 
 	// we need to use popup() to be non blocking and we need a
 	// global scope because we use a signal/slot connection
-	if (contextMenu)
-		delete contextMenu;
-	if (contextSubMenu)
-		delete contextSubMenu;
-
-	contextMenu = new QPopupMenu();
-	contextSubMenu = new QPopupMenu();
+	delete contextMenu;
+	delete contextSubMenu;
+	contextMenu = new QPopupMenu(this);
+	contextSubMenu = new QPopupMenu(this);
 	connect(contextMenu, SIGNAL(activated(int)), this, SLOT(on_goRef_activated(int)));
 	connect(contextSubMenu, SIGNAL(activated(int)), this, SLOT(on_goRef_activated(int)));
 
-	bool isRevPage = (tabWdg->currentPageIndex() == tabWdg->indexOf(revTab));
-	bool isPatchPage = (tabWdg->currentPageIndex() == tabWdg->indexOf(patchTab));
-	bool isFilePage = (tabWdg->currentPageIndex() == tabWdg->indexOf(fileTab));
+	QObject* t;
+	int tt = currentTabType(&t);
+	bool isRevPage = (tt == TAB_REV);
+	bool isPatchPage = (tt == TAB_PATCH);
+	bool isFilePage = (tt == TAB_FILE);
 
 	if (!isFilePage && ActCheckWorkDir->isEnabled())
 		ActCheckWorkDir->addTo(contextMenu);
+
 	contextMenu->insertSeparator();
 
 	if (!isPatchPage && ActViewDiff->isEnabled())
 		ActViewDiff->addTo(contextMenu);
 
+	if (isRevPage && ActViewDiffNewTab->isEnabled())
+		ActViewDiffNewTab->addTo(contextMenu);
+
 	if (isFilePage && ActViewRev->isEnabled())
 		ActViewRev->addTo(contextMenu);
 
@@ -1000,7 +1096,7 @@ void MainImpl::doContexPopup(SCRef sha) {
 			ActPop->addTo(contextMenu);
 
 		const QStringList& bn(git->getBranchNames());
-		const QStringList& tn(git->getTagNames(true));
+		const QStringList& tn(git->getTagNames(Git::optOnlyLoaded));
 		if (bn.empty() && tn.empty()) {
 			contextMenu->exec(QCursor::pos());
 			return;
@@ -1040,9 +1136,10 @@ void MainImpl::doFileContexPopup(SCRef fileName, int type) {
 
 	QPopupMenu contextMenu;
 
-	bool isRevPage = (tabWdg->currentPageIndex() == tabWdg->indexOf(revTab));
-	bool isPatchPage = (tabWdg->currentPageIndex() == tabWdg->indexOf(patchTab));
-
+	QObject* t;
+	int tt = currentTabType(&t);
+	bool isRevPage = (tt == TAB_REV);
+	bool isPatchPage = (tt == TAB_PATCH);
 	bool isDir = rv->treeView->isDir(fileName);
 
 	if (type == POPUP_FILE_EV)
@@ -1052,6 +1149,9 @@ void MainImpl::doFileContexPopup(SCRef fileName, int type) {
 	if (!isDir && ActViewFile->isEnabled())
 		ActViewFile->addTo(&contextMenu);
 
+	if (!isDir && ActViewFileNewTab->isEnabled())
+		ActViewFileNewTab->addTo(&contextMenu);
+
 	if (!isRevPage && (type == POPUP_FILE_EV) && ActViewRev->isEnabled())
 		ActViewRev->addTo(&contextMenu);
 
@@ -1080,19 +1180,23 @@ void MainImpl::on_goRef_activated(int id) {
 void MainImpl::ActSplitView_activated() {
 
 	bool hide;
-	switch (tabType(tabWdg->currentPage())) {
-	case TAB_REV:
-		hide = textBrowserDesc->isVisible();
-		textBrowserDesc->setHidden(hide);
-		listBoxFiles->setHidden(hide);
+	QObject* t;
+	switch (currentTabType(&t)) {
+	case TAB_REV: {
+		RevsView* rv = static_cast<RevsView*>(t);
+		hide = rv->tab()->textBrowserDesc->isVisible();
+		rv->tab()->textBrowserDesc->setHidden(hide);
+		rv->tab()->listBoxFiles->setHidden(hide); }
 		break;
-	case TAB_PATCH:
-		hide = textBrowserDesc_2->isVisible();
-		textBrowserDesc_2->setHidden(hide);
+	case TAB_PATCH: {
+		PatchView* pv = static_cast<PatchView*>(t);
+		hide = pv->tab()->textBrowserDesc->isVisible();
+		pv->tab()->textBrowserDesc->setHidden(hide); }
 		break;
-	case TAB_FILE:
-		hide = fileTab->histListView->isVisible();
-		fileTab->histListView->setHidden(hide);
+	case TAB_FILE: {
+		FileView* fv = static_cast<FileView*>(t);
+		hide = fv->tab()->histListView->isVisible();
+		fv->tab()->histListView->setHidden(hide); }
 		break;
 	default:
 		dbs("ASSERT in ActSplitView_activated: unknown current page");
@@ -1120,13 +1224,10 @@ void MainImpl::ActSaveFile_activated() {
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-	QString fileData;
-	git->getFile(rv->st.fileName(), rv->st.sha(), NULL, &fileData);
+	if (!git->saveFile(rv->st.fileName(), rv->st.sha(), fileName))
+		statusBar()->message("Unable to save " + fileName);
 
 	QApplication::restoreOverrideCursor();
-
-	if (!git->writeToFile(fileName, fileData))
-		statusBar()->message("Unable to save " + fileName);
 }
 
 void MainImpl::on_openRecent_activated(int id) {
@@ -1176,10 +1277,10 @@ void MainImpl::ActRefresh_activated() {
 
 void MainImpl::ActMailFormatPatch_activated() {
 
-	if (listViewLog->childCount() == 0)
+	if (rv->tab()->listViewLog->childCount() == 0)
 		return;
 
-	if (listViewLog->currentItem() == NULL) {
+	if (rv->tab()->listViewLog->currentItem() == NULL) {
 		statusBar()->message("At least one selected revision needed");
 		return;
 	}
@@ -1242,7 +1343,7 @@ void MainImpl::ActMailApplyPatch_activated() {
 
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
-	if (git->applyPatchFile(f.absFilePath(), commit, fold, false) && !commit)
+	if (git->applyPatchFile(f.absFilePath(), commit, fold, Git::optSign) && !commit)
 		git->resetCommits(1);
 
 	QApplication::restoreOverrideCursor();
@@ -1265,51 +1366,50 @@ void MainImpl::ActSettings_activated() {
 	setView->exec(); // modal exec
 
 	// update ActCheckWorkDir if necessary
-	bool diffIndex = (testFlag(DIFF_INDEX_F) != 0);
-	if (ActCheckWorkDir->isOn() != diffIndex)
+	if (ActCheckWorkDir->isOn() != testFlag(DIFF_INDEX_F))
 		ActCheckWorkDir->toggle();
 }
 
-void MainImpl::ActMacroSetup_activated() {
+void MainImpl::ActCustomActionSetup_activated() {
 
-	MacroImpl* macroView = new MacroImpl(); // has WDestructiveClose
-	connect(this, SIGNAL(closeAllWindows()), macroView, SLOT(close()));
-	connect(macroView, SIGNAL(macroListChanged(const QStringList&)),
-	        this, SLOT(on_macroListChanged(const QStringList&)));
+	CustomActionImpl* ca = new CustomActionImpl(); // has WDestructiveClose
+	connect(this, SIGNAL(closeAllWindows()), ca, SLOT(close()));
+	connect(ca, SIGNAL(listChanged(const QStringList&)),
+	        this, SLOT(on_customActionListChanged(const QStringList&)));
 
-	macroView->show();
+	ca->show();
 }
 
-void MainImpl::on_macroListChanged(const QStringList& macroList) {
+void MainImpl::on_customActionListChanged(const QStringList& list) {
 
 	// update menu of all windows
-	QWidgetList* list = QApplication::topLevelWidgets();
-	QWidgetListIt it(*list);
+	QWidgetList* l = QApplication::topLevelWidgets();
+	QWidgetListIt it(*l);
 	while (it.current() != 0) {
 		MainImpl* w = dynamic_cast<MainImpl*>(it.current());
 		if (w)
-			w->doUpdateMacroMenu(macroList);
+			w->doUpdateCustomActionMenu(list);
 		++it;
 	}
-	delete list; // list is created using new by topLevelWidgets()
+	delete l; // list is created using new by topLevelWidgets()
 }
 
-void MainImpl::doUpdateMacroMenu(const QStringList& macroList) {
+void MainImpl::doUpdateCustomActionMenu(const QStringList& list) {
 
-	while (Actions->idAt(1) != -1) // clears menu
+	while (Actions->idAt(1) != -1) // clear menu
 		Actions->removeItemAt(1);
 
-	if (macroList.isEmpty())
+	if (list.isEmpty())
 		return;
 
 	Actions->insertSeparator();
-	loopList(it, macroList)
+	loopList(it, list)
 		Actions->insertItem(*it);
 }
 
-void MainImpl::on_macro_activated(int id) {
+void MainImpl::on_customAction_activated(int id) {
 
-	const QString name = Actions->text(id);
+	const QString name(Actions->text(id));
 	if (name == "Setup actions...")
 		return;
 
@@ -1317,10 +1417,10 @@ void MainImpl::on_macro_activated(int id) {
 	QStringList sl = QStringList::split(",", set.readEntry(APP_KEY + MCR_LIST_KEY, ""));
 
 	if (sl.findIndex(name) == -1) {
-		dbp("ASSERT in on_macro_Activated, action %1 not found", name);
+		dbp("ASSERT in on_customAction_activated, action %1 not found", name);
 		return;
 	}
-	const QString header = "Macro " + name + "/";
+	const QString header("Macro " + name + "/");
 	QString cmdArgs;
 
 	if (testFlag(MCR_CMD_LINE_F, header)) {
@@ -1331,24 +1431,24 @@ void MainImpl::on_macro_activated(int id) {
 		if (!ok)
 			return;
 	}
-	QString cmd = set.readEntry(APP_KEY + header + MCR_TEXT_KEY, "");
+	SCRef cmd = set.readEntry(APP_KEY + header + MCR_TEXT_KEY, "");
 	if (cmd.isEmpty())
 		return;
 
 	ConsoleImpl* c = new ConsoleImpl(name, git); // has WDestructiveClose
 	connect(this, SIGNAL(closeAllWindows()), c, SLOT(close()));
-	connect(c, SIGNAL(macro_exited(const QString&)),
-	        this, SLOT(on_macro_exited(const QString&)));
+	connect(c, SIGNAL(customAction_exited(const QString&)),
+		this, SLOT(on_customAction_exited(const QString&)));
 
 	if (c->start(cmd, cmdArgs))
 		c->show();
 }
 
-void MainImpl::on_macro_exited(const QString& name) {
+void MainImpl::on_customAction_exited(const QString& name) {
 
-	const QString header = "Macro " + name + "/";
+	const QString header("Macro " + name + "/");
 	if (testFlag(MCR_REFRESH_F, header))
-		QTimer::singleShot(10, this, SLOT(refreshRepo())); // must be out of handler
+		QTimer::singleShot(10, this, SLOT(refreshRepo())); // outside of event handler
 }
 
 void MainImpl::ActCommit_activated() {
@@ -1362,13 +1462,13 @@ void MainImpl::ActCommit_activated() {
 
 void MainImpl::ActCommit_setEnabled(bool b) {
 
-	// Pop and push commands fail if there are local changes,
-	// i.e. commit is enabled, so in this case we disable ActPop and ActPush
-	ActCommit->setEnabled(b);
+	// pop and push commands fail if there are local changes,
+	// so in this case we disable ActPop and ActPush
 	if (b) {
 		ActPush->setEnabled(false);
 		ActPop->setEnabled(false);
 	}
+	ActCommit->setEnabled(b);
 }
 
 void MainImpl::slotCommitViewerClosed() {
@@ -1377,7 +1477,7 @@ void MainImpl::slotCommitViewerClosed() {
 		changesCommitted = false;
 		refreshRepo(false);
 	} else {
-		QListViewItem* item = listViewLog->findItem(ZERO_SHA, COMMIT_COL);
+		QListViewItem* item = rv->tab()->listViewLog->findItem(ZERO_SHA, COMMIT_COL);
 		ActCommit_setEnabled(item != NULL && !git->isCommittingMerge());
 		statusBar()->message("Failed to commit changes");
 	}
@@ -1385,13 +1485,14 @@ void MainImpl::slotCommitViewerClosed() {
 
 void MainImpl::ActTag_activated() {
 
-	bool ok;
 	int adj = -1; // hack to correctly map col numbers in main view
-	QString tag(listViewLog->currentItem()->text(LOG_COL + adj));
+	QString tag(rv->tab()->listViewLog->currentItem()->text(LOG_COL + adj));
+	bool ok;
 	tag = QInputDialog::getText("Make tag - QGit", "Enter tag name:",
 	                            QLineEdit::Normal, tag, &ok, this);
 	if (!ok || tag.isEmpty())
 		return;
+
 	QString tmp(tag.simplifyWhiteSpace());
 	if (tag != tmp.remove(' ')) {
 		QMessageBox::warning(this, "Make tag - QGit",
@@ -1437,33 +1538,28 @@ void MainImpl::ActPush_activated() {
 
 	QStringList selectedItems;
 	rv->listViewLog->getSelectedItems(selectedItems);
-	bool ok = true;
-	for (uint i = 0; i < selectedItems.count(); i++)
+	for (uint i = 0; i < selectedItems.count(); i++) {
 		if (!git->isUnapplied(selectedItems[i])) {
-			ok = false;
-			break;
+			statusBar()->message("Please, select only unapplied patches");
+			return;
 		}
-	if (!ok) {
-		statusBar()->message("Please, select only unapplied patches");
-		return;
 	}
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-	ok = true;
-	QString sha;
+	bool ok = true;
 	for (uint i = 0; i < selectedItems.count(); i++) {
 		const QString tmp(QString("Pushing patch %1 of %2")
 		                  .arg(i+1).arg(selectedItems.count()));
 		statusBar()->message(tmp);
-		sha = selectedItems[selectedItems.count() - i - 1];
+		SCRef sha = selectedItems[selectedItems.count() - i - 1];
 		if (!git->stgPush(sha)) {
+			statusBar()->message("Failed to push patch " + sha);
 			ok = false;
 			break;
 		}
 	}
 	if (ok)
 		statusBar()->clear();
-	else
-		statusBar()->message("Failed to push patch " + sha);
+
 	QApplication::restoreOverrideCursor();
 	refreshRepo(false);
 }
@@ -1473,7 +1569,7 @@ void MainImpl::ActPop_activated() {
 	QStringList selectedItems;
 	rv->listViewLog->getSelectedItems(selectedItems);
 	if (selectedItems.count() > 1) {
-		statusBar()->message("Please, select just one revision");
+		statusBar()->message("Please, select one revision only");
 		return;
 	}
 	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
@@ -1495,14 +1591,8 @@ void MainImpl::ActFilterTree_toggled(bool b) {
 
 		rv->treeView->getTreeSelectedItems(selectedItems);
 		if (selectedItems.count() == 0) {
-			if (rv->st.fileName().isEmpty()) {
-				dbs("ASSERT tree filter action activated "
-				    "with no selected items");
-				return;
-			}
-			// tree could be empty also with exsistant curFileName
-			// as example when ZERO_SHA is selected
-			selectedItems.append(rv->st.fileName());
+			dbs("ASSERT tree filter action activated with no selected items");
+			return;
 		}
 		statusBar()->message("Filter view on " + selectedItems.join(" "));
 		setRepository(curDir, true, true, &selectedItems);
@@ -1542,17 +1632,20 @@ void MainImpl::ActFind_activated() {
 	if (!te)
 		return;
 
-	QString def;
-	if (te->hasSelectedText())
+	QString def(textToFind);
+	if (te->hasSelectedText()) {
+		TextFormat tf = te->textFormat();
+		te->setTextFormat(Qt::PlainText); // we want text without formatting tags
 		def = te->selectedText().section('\n', 0, 0);
-
+		te->setTextFormat(tf);
+	}
 	bool ok;
 	QString str(QInputDialog::getText("Find text - QGit", "Text to find:",
 	                                  QLineEdit::Normal, def, &ok, this));
 	if (!ok || str.isEmpty())
 		return;
 
-	textToFind = str;
+	textToFind = str; // update with valid data only
 	ActFindNext_activated();
 }
 
@@ -1581,8 +1674,8 @@ void MainImpl::closeEvent(QCloseEvent* ce) {
 	// lastWindowClosed() signal is emitted by close(), after sending
 	// closeEvent(), so we need to close _here_ all secondary windows before
 	// the close() method checks for lastWindowClosed flag to avoid missing
-	// the signal and stay in the main loop forever, because this signal is
-	// connected to qApp->quit()
+	// the signal and stay in the main loop forever, because lastWindowClosed()
+	// signal is connected to qApp->quit()
 	//
 	// note that we cannot rely on setting 'this' parent in secondary windows
 	// because when close() is called childern are still alive and, finally,
@@ -1591,12 +1684,17 @@ void MainImpl::closeEvent(QCloseEvent* ce) {
 	emit closeAllWindows();
 
 	hide();
-	delete fv;
-	delete rv;
 
 	EM_RAISE(exExiting);
-	git->stop(false); // also save cache
-
+	if (!git->stop(Git::optSaveCache)) {
+		// not all processes have been deleted, there is
+		// still some run() not returned somewhere, it is
+		// not safe to delete run() callers objects now
+		QTimer::singleShot(100, this, SLOT(ActClose_activated()));
+		return;
+	}
+	emit closeAllTabs();
+	delete rv;
 	MainBase::closeEvent(ce);
 }
 
