@@ -15,6 +15,8 @@
 #include "common.h"
 #include "mainbase.h"
 
+class QListViewItem;
+class QListBoxItem;
 class Git;
 class Rev;
 class Domain;
@@ -30,6 +32,7 @@ class PatchView;
 class FileView;
 class RevsView;
 class TabFile;
+class FileHistory;
 
 class MainImpl : public MainBase {
 Q_OBJECT
@@ -39,26 +42,27 @@ public:
 signals:
 	void highlightPatch(const QString&, bool);
 	void closeAllWindows();
+	void closeAllTabs();
+	void tabClosed(int tabPos);
 
 public slots:
-	void buttonGroupDiff_clicked(int); // workaround: direct connection does not work
 	void tabWdg_currentChanged(QWidget*);
-	void on_newRevsAdded();
+	void on_newRevsAdded(const FileHistory*, const QValueVector<QString>&);
 
 protected:
 	void customEvent(QCustomEvent* e);
 
 private slots:
-	void afterCtorInit();
+	void initWithEventLoopActive();
 	void refreshRepo(bool setCurRevAfterLoad = true);
 	void on_listViewLog_doubleClicked(QListViewItem*);
 	void on_fileList_doubleClicked(QListBoxItem*);
 	void on_treeView_doubleClicked(QListViewItem*);
 	void on_histListView_doubleClicked(QListViewItem* item);
-	void on_macroListChanged(const QStringList& macroList);
+	void on_customActionListChanged(const QStringList& list);
 	void on_openRecent_activated(int id);
-	void on_macro_activated(int id);
-	void on_macro_exited(const QString& name);
+	void on_customAction_activated(int id);
+	void on_customAction_exited(const QString& name);
 	void on_goRef_activated(int id);
 	void slotCommitViewerClosed();
 	void lineEditSHA_returnPressed();
@@ -73,7 +77,9 @@ private slots:
 	void ActFindNext_activated();
 	void ActViewRev_activated();
 	void ActViewFile_activated();
+	void ActViewFileNewTab_activated();
 	void ActViewDiff_activated();
+	void ActViewDiffNewTab_activated();
 	void ActExternalDiff_activated();
 	void ActSplitView_activated();
 	void ActOpenRepo_activated();
@@ -90,7 +96,7 @@ private slots:
 	void ActPop_activated();
 	void ActClose_activated();
 	void ActExit_activated();
-	void ActMacroSetup_activated();
+	void ActCustomActionSetup_activated();
 	void ActCheckWorkDir_toggled(bool);
 	void ActShowTree_toggled(bool);
 	void ActFilterTree_toggled(bool);
@@ -102,13 +108,14 @@ private:
 	friend class FileView;
 	friend class RevsView;
 	friend class ListView;
+	friend class PatchView;
 
 	bool eventFilter(QObject* obj, QEvent* ev);
 	void updateGlobalActions(bool b);
 	void updateContextActions(SCRef newRevSha, SCRef newFileName, bool isDir, bool found);
 	void setupPixmaps(int h);
 	void setupAccelerator(QAccel* accel);
-	int tabType(QWidget* tb);
+	int currentTabType(QObject** t);
 	void filterList(bool isOn, bool onlyHighlight);
 	bool passFilter(ListViewItem* i, SCRef f, int cn, const QMap<QString,bool>& sm);
 	void setRepository(SCRef wd, bool r, bool ks, QStringList* fl = NULL);
@@ -118,7 +125,7 @@ private:
 	void updateCommitMenu(bool isStGITStack);
 	void updateRecentRepoMenu(SCRef newEntry = "");
 	void doUpdateRecentRepoMenu(SCRef newEntry);
-	void doUpdateMacroMenu(const QStringList& macroList);
+	void doUpdateCustomActionMenu(const QStringList& list);
 	void ActCommit_setEnabled(bool b);
 	void doContexPopup(SCRef sha);
 	void doFileContexPopup(SCRef fileName, int type);
@@ -128,17 +135,16 @@ private:
 	void goMatch(int delta);
 	bool askApplyPatchParameters(bool* commit, bool* fold);
 	QTextEdit* getCurrentTextEdit();
+	template<class X> QPtrList<X>* getTabs(int tabPos = -1);
+	template<class X> X* firstTab(int startPos = -1);
+	void openFileTab(FileView* fv = NULL);
 
 	EM_DECLARE(exSetRepositoryCalled);
 	EM_DECLARE(exExiting);
 
 	Git* git;
 	RevsView* rv;
-	QGuardedPtr<FileView> fv;
 	QAccel* accel;
-	QWidget* revTab;
-	QWidget* patchTab;
-	QGuardedPtr<TabFile> fileTab;
 
 	// curDir is the repository working dir, could be different from qgit running
 	// directory QDir::current(). Note that qgit could be run from subdirectory
@@ -164,9 +170,11 @@ class ExternalDiffProc : public QProcess {
 Q_OBJECT
 public:
 	ExternalDiffProc(const QStringList& args, QObject* p) : QProcess(args, p) {
+
 		connect(this, SIGNAL(processExited()), this, SLOT(exit()));
 	}
 	~ExternalDiffProc() {
+
 		tryTerminate();
 		removeFiles();
 	}
@@ -175,6 +183,7 @@ public slots:
 
 private:
 	void removeFiles() {
+
 		if (!arguments().empty()) {
 			QDir d; // remove temporary files to diff on
 			d.remove(arguments()[1]);
