@@ -13,7 +13,7 @@
 #include "common.h"
 #include "myprocess.h"
 
-MyProcess::MyProcess(QObject *go, Git* g, const QString& wd, bool err) : QProcess(go) {
+MyProcess::MyProcess(QObject *go, Git* g, const QString& wd, bool err) : QProcess(g) {
 
 	guiObject = go;
 	git = g;
@@ -33,7 +33,6 @@ bool MyProcess::runAsync(SCRef rc, QObject* rcv, SCRef buf) {
 	if (!launchMe(runCmd, buf))
 		return false; // caller will delete us
 
-	git->incRunningProcesses();
 	return true;
 }
 
@@ -50,15 +49,17 @@ bool MyProcess::runSync(SCRef rc, QString* ro, QObject* rcv, SCRef buf) {
 	if (!launchMe(runCmd, buf))
 		return false;
 
-	git->incRunningProcesses();
 	busy = true; // we have to wait here until we exit
 
 	EM_BEFORE_PROCESS_EVENTS;
 
 	while (busy) {
-		isRunning();
-		usleep(20000); // suspend 20ms to let OS reschedule
+		// without this filter on patch does not work
+		// FIXME definitely needs more understanding
 		qApp->processEvents();
+
+		usleep(20000); // suspend 20ms to let OS reschedule
+		isRunning();
 	}
 
 	EM_AFTER_PROCESS_EVENTS;
@@ -68,11 +69,11 @@ bool MyProcess::runSync(SCRef rc, QString* ro, QObject* rcv, SCRef buf) {
 
 void MyProcess::setupSignals() {
 
-	connect(git, SIGNAL(cancelAllProcesses()), this, SLOT(cancel()));
-	connect(this, SIGNAL(readyReadStdout()), this, SLOT(myReadFromStdout()));
-	connect(this, SIGNAL(processExited()), this, SLOT(myProcExited()));
+	connect(git, SIGNAL(cancelAllProcesses()), this, SLOT(on_cancel()));
+	connect(this, SIGNAL(readyReadStdout()), this, SLOT(on_readyReadStdout()));
+	connect(this, SIGNAL(processExited()), this, SLOT(on_processExited()));
 	if (receiver != NULL) {
-		connect(this, SIGNAL(readyReadStderr()), this, SLOT(myReadFromStderr()));
+		connect(this, SIGNAL(readyReadStderr()), this, SLOT(on_readyReadStderr()));
 		connect(this, SIGNAL(procDataReady(const QString&)),
 		        receiver, SLOT(on_procDataReady(const QString&)));
 		connect(this, SIGNAL(eof()), receiver, SLOT(on_eof()));
@@ -81,6 +82,9 @@ void MyProcess::setupSignals() {
 
 void MyProcess::sendErrorMsg(bool notStarted) {
 
+	if (!errorReportingEnabled)
+		return;
+
 	QString errorDesc(readStderr());
 	if (notStarted)
 		errorDesc = QString::fromAscii("Unable to start the process!");
@@ -95,14 +99,16 @@ bool MyProcess::launchMe(SCRef runCmd, SCRef buf) {
 	const QStringList sl(splitArgList(runCmd));
 	setArguments(sl);
 	setWorkingDirectory(workDir);
+	git->incRunningProcesses();
 	bool ok = launch(buf);
-	if (!ok)
+	if (!ok) {
+		git->decRunningProcesses();
 		sendErrorMsg(true);
-
+	}
 	return ok;
 }
 
-void MyProcess::myReadFromStdout() {
+void MyProcess::on_readyReadStdout() {
 // workaround pipe buffer size limit. In case of big output, buffer
 // can became full and an hang occurs, so we read all data as soon
 // as possible and store it in runOutput
@@ -118,7 +124,7 @@ void MyProcess::myReadFromStdout() {
 		runOutput->append(tmp);
 }
 
-void MyProcess::myReadFromStderr() {
+void MyProcess::on_readyReadStderr() {
 
 	if (receiver == NULL) {
 		dbs("ASSERT in myReadFromStderr: NULL receiver");
@@ -131,14 +137,14 @@ void MyProcess::myReadFromStderr() {
 	emit procDataReady(tmp); // redirect to stdout
 }
 
-void MyProcess::myProcExited() {
+void MyProcess::on_processExited() {
 
 	if (!canceling) { // no more noise after cancel
 
 		if (receiver)
 			emit eof();
 
-		if (errorReportingEnabled && (!normalExit() || canReadLineStderr())) {
+		if (!normalExit() || canReadLineStderr()) {
 			exitStatus = false;
 			sendErrorMsg();
 		}
@@ -152,7 +158,7 @@ void MyProcess::myProcExited() {
 	git->decRunningProcesses();
 }
 
-void MyProcess::cancel() {
+void MyProcess::on_cancel() {
 
 	canceling = true;
 	QProcess::tryTerminate();
@@ -176,7 +182,7 @@ const QStringList MyProcess::splitArgList(SCRef cmd) {
 	const QString sepList("#%&!?"); // separator candidates
 	uint i = 0;
 	while (cmd.contains(sepList[i]) && i < sepList.length())
-		++i;
+		i++;
 
 	if (i == sepList.length()) {
 		dbs("ASSERT no unique separator found");
@@ -197,14 +203,16 @@ const QStringList MyProcess::splitArgList(SCRef cmd) {
 	// and should be removed before to feed QProcess
 	newCmd.remove(QGit::QUOTE_CHAR);
 
-	//QProcess::setArguments doesn't want extra quotes
-	//for arguments with spaces, so remove them.
-	newCmd.replace(sepChar + '\"', sepChar);
-	newCmd.replace(sepChar + '\'', sepChar);
-	newCmd.replace('\"' + sepChar, sepChar);
-	newCmd.replace('\'' + sepChar, sepChar);
-
-	return QStringList::split(sepChar, newCmd);
+	// QProcess::setArguments doesn't want quote
+	// delimited arguments, so remove trailing quotes
+	QStringList sl = QStringList::split(sepChar, newCmd);
+	QStringList::iterator it(sl.begin());
+	for ( ; it != sl.end(); ++it) {
+		if (((*it).left(1) == "\"" && (*it).right(1) == "\"") ||
+		   ((*it).left(1) == "\'" && (*it).right(1) == "\'"))
+			*it = (*it).mid(1, (*it).length() - 2);
+	}
+	return sl;
 }
 
 void MyProcess::restoreSpaces(QString& newCmd, SCRef sepChar) {
@@ -212,7 +220,7 @@ void MyProcess::restoreSpaces(QString& newCmd, SCRef sepChar) {
 
 	QString quoteChar;
 	bool replace = false;
-	for (uint i = 0; i < newCmd.length(); ++i) {
+	for (uint i = 0; i < newCmd.length(); i++) {
 
 		const QChar& c = newCmd[i];
 
