@@ -26,10 +26,12 @@ signals:
 	void eof();
 
 public slots:
-	void cancel();
-	void myReadFromStdout();
-	void myReadFromStderr();
-	void myProcExited();
+	void on_cancel();
+
+private slots:
+	void on_readyReadStdout();
+	void on_readyReadStderr();
+	void on_processExited();
 
 private:
 	void setupSignals();
