@@ -12,9 +12,14 @@
 #include <qsyntaxhighlighter.h>
 #include <qradiobutton.h>
 #include <qbuttongroup.h>
+#include <qtabwidget.h>
 #include "common.h"
 #include "git.h"
 #include "domain.h"
+#include "mainimpl.h"
+#include "revdesc.h"
+#include "filelist.h"
+#include "patchbase.h"
 #include "patchview.h"
 
 // diff types options, corresponds to buttons id
@@ -42,8 +47,20 @@ public:
 		case '-':
 			myColor = Qt::red;
 			break;
+		case 'c':
 		case 'd':
-			if (text.startsWith("diff --git a/"))
+		case 'i':
+		case 'n':
+		case 'o':
+		case 'r':
+		case 's':
+			if (   text.startsWith("diff --git a/")
+			    || text.startsWith("copy ")
+			    || text.startsWith("index ")
+			    || text.startsWith("new ")
+			    || text.startsWith("old ")
+			    || text.startsWith("rename ")
+			    || text.startsWith("similarity "))
 				myColor = Qt::darkBlue;
 			else if (combinedLenght > 0 && text.startsWith("diff --combined"))
 				myColor = Qt::darkBlue;
@@ -80,63 +97,83 @@ private:
 	uint combinedLenght;
 };
 
-PatchView::PatchView(Domain* dm, Git* g, QTextEdit* p, QLineEdit* l, QRadioButton* r) :
-                     QObject(dm), d(dm), git(g), textEditDiff(p),lineEditDiff(l),radioButtonSha(r) {
+PatchView::PatchView(MainImpl* mi, Git* g) : Domain(mi, g) {
 
-	diffHighlighter = new DiffHighlighter(this, textEditDiff);
+	patchTab = new TabPatch(m());
+	patchTab->textBrowserDesc->setDomain(this);
+	m()->tabWdg->addTab(patchTab, "&Patch");
+	tabPosition = m()->tabWdg->count() - 1;
+
+	listBoxFiles = new ListBoxFiles(this, git, patchTab->listBoxFiles);
+	diffHighlighter = new DiffHighlighter(this, patchTab->textEditDiff);
 	seekTarget = diffLoaded = false;
 	pickAxeRE.setMinimal(true);
 	pickAxeRE.setCaseSensitive(false);
 
-	st = &(d->st);
+	connect(patchTab->lineEditDiff, SIGNAL(returnPressed()),
+	        this, SLOT(on_lineEditDiff_returnPressed()));
 
-	connect(lineEditDiff, SIGNAL(returnPressed()), this, SLOT(on_lineEditDiff_returnPressed()));
+	connect(patchTab->buttonGroupDiff, SIGNAL(clicked(int)),
+	        this, SLOT(on_buttonGroupDiff_clicked(int)));
+
+	connect(listBoxFiles, SIGNAL(contextMenu(const QString&, int)),
+	        this, SLOT(on_contextMenu(const QString&, int)));
 }
 
 PatchView::~PatchView() {
 
+	if (!parent())
+		return;
+
 	git->cancelProcess(proc);
 	delete diffHighlighter;
+	delete listBoxFiles;
+	delete patchTab;
 }
 
-void PatchView::clear() {
+void PatchView::clear(bool complete) {
 
+	if (complete) {
+		st.clear();
+		patchTab->textBrowserDesc->clear();
+		listBoxFiles->clear();
+	}
+	patchTab->textEditDiff->clear();
 	matches.clear();
 	diffLoaded = false;
-	textEditDiff->clear();
 	seekTarget = !target.isEmpty();
 	partialParagraphs = "";
 }
 
-void PatchView::centerOn(const QString& t) {
+void PatchView::centerOnFileHeader(const QString& fileName) {
 
-	target = t;
+	target = fileName;
+	bool combined = (st.isMerge() && !st.allMergeFiles());
+	git->formatPatchFileHeader(&target, st.sha(), st.diffToSha(), combined, st.allMergeFiles());
 	seekTarget = !target.isEmpty();
 	if (seekTarget)
 		centerTarget();
 }
 
-void PatchView::on_fileSelected(const QString& text) {
-// center diffViewer on selected file name
+void PatchView::on_contextMenu(const QString& data, int type) {
 
-	bool combined = (st->isMerge() && !st->allMergeFiles());
-	QString prefix((combined) ? "diff --combined " : "diff --git a/");
-	centerOn(prefix + text);
+	if (isLinked()) // skip if not linked to main view
+		Domain::on_contextMenu(data, type);
 }
 
 void PatchView::centerTarget() {
 
-	textEditDiff->setCursorPosition(0, 0);
-	if (!textEditDiff->find(target, true, true)) // updates cursor position
+	patchTab->textEditDiff->setCursorPosition(0, 0);
+	if (!patchTab->textEditDiff->find(target, true, true)) // updates cursor position
 		return;
 
 	// target found
 	seekTarget = false;
 	int para, index;
-	textEditDiff->getCursorPosition(&para, &index);
-	QPoint p = textEditDiff->paragraphRect(para).topLeft();
-	textEditDiff->setContentsPos(p.x(), p.y());
-	textEditDiff->removeSelection();
+	patchTab->textEditDiff->getCursorPosition(&para, &index);
+	QPoint p = patchTab->textEditDiff->paragraphRect(para).topLeft();
+	patchTab->textEditDiff->setContentsPos(p.x(), p.y());
+	patchTab->textEditDiff->removeSelection();
 }
 
 void PatchView::centerMatch(uint id) {
@@ -144,14 +181,14 @@ void PatchView::centerMatch(uint id) {
 	if (matches.count() <= id)
 		return;
 
-	textEditDiff->setSelection(matches[id].paraFrom, matches[id].indexFrom,
-	                           matches[id].paraTo, matches[id].indexTo);
+	patchTab->textEditDiff->setSelection(matches[id].paraFrom, matches[id].indexFrom,
+	                                     matches[id].paraTo, matches[id].indexTo);
 }
 
 void PatchView::on_procDataReady(const QString& data) {
 
-	int X = textEditDiff->contentsX();
-	int Y = textEditDiff->contentsY();
+	int X = patchTab->textEditDiff->contentsX();
+	int Y = patchTab->textEditDiff->contentsY();
 
 	bool targetInNewChunk = false;
 	if (seekTarget)
@@ -163,18 +200,18 @@ void PatchView::on_procDataReady(const QString& data) {
 	// so we append only \n terminating text
 	//
 	// NOTE: last char of diff data MUST always be '\n' for this to work
-	int pos = data.findRev('\n');
-	if (pos == -1) {
+	int pos = data.findRev('\n') + 1;
+	if (pos == 0) {
 		partialParagraphs.append(data);
 		return;
 	}
-	textEditDiff->append(partialParagraphs + data.left(pos + 1));
-	partialParagraphs = data.right(data.length() - pos - 1);
+	patchTab->textEditDiff->append(partialParagraphs + data.left(pos));
+	partialParagraphs = data.mid(pos);
 	if (targetInNewChunk)
 		centerTarget();
 	else {
-		textEditDiff->setContentsPos(X, Y);
-		textEditDiff->sync();
+		patchTab->textEditDiff->setContentsPos(X, Y);
+		patchTab->textEditDiff->sync();
 	}
 }
 
@@ -200,7 +237,7 @@ void PatchView::computeMatches() {
 	if (pickAxeRE.isEmpty())
 		return;
 
-	SCRef txt = textEditDiff->text();
+	SCRef txt = patchTab->textEditDiff->text();
 	int pos, lastPos = 0, lastPara = 0;
 
 	// must be at the end to catch patterns across more the one chunk
@@ -248,10 +285,10 @@ void PatchView::on_highlightPatch(const QString& exp, bool re) {
 
 void PatchView::on_lineEditDiff_returnPressed() {
 
-	if (lineEditDiff->text().isEmpty())
+	if (patchTab->lineEditDiff->text().isEmpty())
 		return;
 
-	radioButtonSha->setChecked(true); // could be called by code
+	patchTab->radioButtonSha->setChecked(true); // could be called by code
 	on_buttonGroupDiff_clicked(DIFF_TO_SHA);
 }
 
@@ -265,7 +302,7 @@ void PatchView::on_buttonGroupDiff_clicked(int diffType) {
 		sha = "HEAD";
 		break;
 	case DIFF_TO_SHA:
-		sha = lineEditDiff->text();
+		sha = patchTab->lineEditDiff->text();
 		break;
 	}
 	if (sha == QGit::ZERO_SHA)
@@ -274,48 +311,99 @@ void PatchView::on_buttonGroupDiff_clicked(int diffType) {
 	// check for a ref name or an abbreviated form
 	normalizedSha = (sha.length() != 40 && !sha.isEmpty()) ? git->getRefSha(sha) : sha;
 
-	if (normalizedSha != st->diffToSha()) { // avoid looping
-		st->setDiffToSha(normalizedSha); // could be empty
-		UPDATE_DOMAIN(d);
+	if (normalizedSha != st.diffToSha()) { // avoid looping
+		st.setDiffToSha(normalizedSha); // could be empty
+		UPDATE();
 	}
 }
 
-void PatchView::update(bool force) {
-
-	if (st->sha().isEmpty())
-		return;
-
-	bool combined = (st->isMerge(true) && !st->allMergeFiles(true));
-	bool prevCombined = (st->isMerge(false) && !st->allMergeFiles(false));
-
-	bool isChanged = (   (st->sha(true) != st->sha(false))
-	                  || (st->diffToSha(true) != st->diffToSha(false))
-	                  || (combined != prevCombined));
-
-	if (!isChanged && !force)
-		return;
+void PatchView::updatePatch() {
 
 	git->cancelProcess(proc);
-	clear();
+	clear(false); // only patch content
 
+	bool combined = (st.isMerge() && !st.allMergeFiles());
 	if (combined) {
-		const Rev* r = git->revLookup(st->sha());
+		const Rev* r = git->revLookup(st.sha());
 		diffHighlighter->setCombinedLength(r->parentsCount());
 	} else
 		diffHighlighter->setCombinedLength(0);
 
-	if (normalizedSha != st->diffToSha()) { // note <(null> != <(empty)>
+	if (normalizedSha != st.diffToSha()) { // note <(null> != <(empty)>
 
-		if (!st->diffToSha().isEmpty()) {
-			lineEditDiff->setText(st->diffToSha());
+		if (!st.diffToSha().isEmpty()) {
+			patchTab->lineEditDiff->setText(st.diffToSha());
 			on_lineEditDiff_returnPressed();
 
 		} else if (!normalizedSha.isEmpty()) {
 			normalizedSha = "";
 			// we cannot uncheck radioButtonSha directly
 			// because "Parent" button will stay off
-			radioButtonSha->group()->find(0)->toggle();
+			patchTab->radioButtonSha->group()->find(0)->toggle();
 		}
 	}
-	proc = git->getDiff(st->sha(), this, st->diffToSha(), combined); // non blocking
+	proc = git->getDiff(st.sha(), this, st.diffToSha(), combined); // non blocking
+}
+
+bool PatchView::doUpdate() {
+
+	bool dataCleared = m()->lineEditSHA->text().isEmpty();
+	bool isFileChanged = (st.fileName(true) != st.fileName(false));
+	bool isChanged = (   (st.sha(true) != st.sha(false))
+	                  || (st.diffToSha(true) != st.diffToSha(false))
+	                  || (st.isMerge(true) != st.isMerge(false))
+	                  || (st.allMergeFiles(true) != st.allMergeFiles(false)));
+	try {
+		EM_REGISTER(m()->exSetRepositoryCalled);
+		EM_REGISTER(m()->exExiting);
+
+		const RevFile* files = NULL;
+		if (isChanged || dataCleared) {
+
+			if (!isLinked()) {
+				uint maxLen = 30;
+				QString caption(git->getShortLog(st.sha()));
+				if (caption.length() > maxLen)
+					caption = caption.left(maxLen - 3).
+					          stripWhiteSpace().append("...");
+
+				m()->tabWdg->changeTab(patchTab, caption);
+			}
+			SCRef d(git->getDesc(st.sha(), m()->shortLogRE, m()->longLogRE));
+			patchTab->textBrowserDesc->setText(d);
+			patchTab->textBrowserDesc->setCursorPosition(0, 0);
+
+			updatePatch();
+
+			files = git->getFiles(st.sha(), st.diffToSha(), st.allMergeFiles());
+		}
+		if (isFileChanged || isChanged || dataCleared) {
+
+			listBoxFiles->update(files);
+			if (!st.fileName().isEmpty())
+				centerOnFileHeader(st.fileName());
+		}
+		EM_REMOVE(m()->exExiting);
+		EM_REMOVE(m()->exSetRepositoryCalled);
+
+		return true;
+
+	} catch(int i) {
+
+		EM_REMOVE(m()->exExiting);
+		EM_REMOVE(m()->exSetRepositoryCalled);
+
+		if (EM_MATCH(i, m()->exSetRepositoryCalled, "update patch")) {
+			EM_CHECK_PENDING;
+			return false;
+		}
+		if (EM_MATCH(i, m()->exExiting, "update patch")) {
+			EM_CHECK_PENDING;
+			return false;
+		}
+		const QString info("Exception \'" + EM_DESC(i) + "\' "
+		                   "not handled in update patch...re-throw");
+		dbp("%1", info);
+		throw i;
+	}
 }
