@@ -15,13 +15,12 @@
 
 using namespace QGit;
 
-RangeSelectImpl::RangeSelectImpl(QString& args, const QStringList& tl, QWidget *p,
-                                 QString& opt, bool* q) : RangeSelectBase(p, 0, true) {
+RangeSelectImpl::RangeSelectImpl(QWidget* par, QString* r, const QStringList& tl,
+                                 bool* q) : RangeSelectBase(par, 0, true) {
+	range = r;
 	quit = q;
 	*quit = true; // if user press ESC everything works as expected
-	output = &args;
-	options = &opt;
-	QStringList tmp1(tl), tmp2; // tl is unsorted
+	QStringList tmp1(tl), tmp2; // tag list tl is unsorted
 	tmp1.sort();
 	loopList(it, tmp1)
 		tmp2.prepend(*it);
@@ -30,7 +29,6 @@ RangeSelectImpl::RangeSelectImpl(QString& args, const QStringList& tl, QWidget *
 	comboBoxTo->insertItem("HEAD", 0);
 	comboBoxFrom->insertStringList(tmp2);
 	comboBoxFrom->setFocus();
-	lineEditOptions->setText(opt);
 
 	int f = flags();
 	checkBoxDiffCache->setChecked(f & DIFF_INDEX_F);
@@ -39,53 +37,6 @@ RangeSelectImpl::RangeSelectImpl(QString& args, const QStringList& tl, QWidget *
 	checkBoxShowDialog->setChecked(f & RANGE_SELECT_F);
 }
 
-void RangeSelectImpl::pushButtonOk_clicked() {
-
-	if (comboBoxFrom->currentText().isEmpty())
-		*output = comboBoxTo->currentText();
-	else {
-		*output = comboBoxFrom->currentText();
-		output->append("..");
-		output->append(comboBoxTo->currentText());
-	}
-	*options = lineEditOptions->text();
-	output->prepend(*options + " ");
-	*output = output->stripWhiteSpace();
-	*quit = false;
-	close();
-}
-
-void RangeSelectImpl::checkBoxShowAll_toggled(bool b) {
-
-	QString opt = lineEditOptions->text();
-	if (b) {
-		if (!opt.contains("--all"))
-			opt = opt.append(" --all").stripWhiteSpace();
-	} else
-		opt = opt.remove("--all").stripWhiteSpace();
-
-	lineEditOptions->setText(opt);
-	setFlag(ALL_BRANCHES_F, opt.contains("--all"));
-}
-
-void RangeSelectImpl::checkBoxShowWholeHistory_toggled(bool b) {
-
-	if (b) {
-		fromTmp = comboBoxFrom->currentText();
-		toTmp = comboBoxTo->currentText();
-		comboBoxFrom->setCurrentText("");
-		comboBoxTo->setCurrentText("HEAD");
-		comboBoxFrom->setEnabled(false);
-		comboBoxTo->setEnabled(false);
-	} else {
-		comboBoxFrom->setCurrentText(fromTmp);
-		comboBoxTo->setCurrentText(toTmp);
-		comboBoxFrom->setEnabled(true);
-		comboBoxTo->setEnabled(true);
-	}
-	setFlag(WHOLE_HISTORY_F, b);
-}
-
 void RangeSelectImpl::checkBoxDiffCache_toggled(bool b) {
 
 	setFlag(DIFF_INDEX_F, b);
@@ -95,3 +46,44 @@ void RangeSelectImpl::checkBoxShowDialog_toggled(bool b) {
 
 	setFlag(RANGE_SELECT_F, b);
 }
+
+void RangeSelectImpl::pushButtonOk_clicked() {
+
+	*range = comboBoxFrom->currentText();
+	if (!range->isEmpty())
+		range->append("..");
+
+	range->append(comboBoxTo->currentText());
+	range->prepend(lineEditOptions->text() + " ");
+	*range = range->stripWhiteSpace();
+	*quit = false;
+	close();
+}
+
+void RangeSelectImpl::checkBoxShowAll_toggled(bool b) {
+
+	QString opt(lineEditOptions->text());
+	opt.remove("--all");
+	if (b)
+		opt.append(" --all");
+
+	lineEditOptions->setText(opt.stripWhiteSpace());
+	setFlag(ALL_BRANCHES_F, b);
+}
+
+void RangeSelectImpl::checkBoxShowWholeHistory_toggled(bool b) {
+
+	if (b) {
+		fromBckUp = comboBoxFrom->currentText();
+		toBckUp = comboBoxTo->currentText();
+		comboBoxFrom->setCurrentText("");
+		comboBoxTo->setCurrentText("HEAD");
+
+	} else {
+		comboBoxFrom->setCurrentText(fromBckUp);
+		comboBoxTo->setCurrentText(toBckUp);
+	}
+	comboBoxFrom->setEnabled(!b);
+	comboBoxTo->setEnabled(!b);
+	setFlag(WHOLE_HISTORY_F, b);
+}
