@@ -21,33 +21,36 @@
 #include <qaction.h>
 #include <qdragobject.h>
 #include <qtoolbutton.h>
+#include <qtabwidget.h>
 #include "common.h"
 #include "git.h"
 #include "domain.h"
 #include "treeview.h"
 #include "listview.h"
 #include "filelist.h"
+#include "revbase.h"
 #include "revdesc.h"
+#include "patchbase.h"
 #include "patchview.h"
 #include "mainimpl.h"
 #include "revsview.h"
 
 RevsView::RevsView(MainImpl* mi, Git* g) : Domain(mi, g) {
 
-	listViewLog = new ListView(this, git, m()->listViewLog, false);
-	textBrowserDesc = m()->textBrowserDesc;
-	textBrowserDesc->setDomain(this);
-	textBrowserDesc_2 = m()->textBrowserDesc_2;
-	textBrowserDesc_2->setDomain(this);
-	listBoxFiles = new ListBoxFiles(this, git, m()->listBoxFiles);
-	listBoxFiles_2 = new ListBoxFiles(this, git, m()->listBoxFiles_2);
+	revTab = new TabRev(m());
+	m()->tabWdg->addTab(revTab, "&Rev list");
+	tabPosition = m()->tabWdg->count() - 1;
+
+	listViewLog = new ListView(this, git, tab()->listViewLog, NULL);
+	tab()->textBrowserDesc->setDomain(this);
+	listBoxFiles = new ListBoxFiles(this, git, tab()->listBoxFiles);
 	treeView = new TreeView(this, git, m()->treeView);
 
-	connect(git, SIGNAL(newRevsAdded(const QValueVector<QString>&)),
-	        listViewLog, SLOT(on_newRevsAdded(const QValueVector<QString>&)));
+	connect(git, SIGNAL(newRevsAdded(const FileHistory*, const QValueVector<QString>&)),
+	listViewLog, SLOT(on_newRevsAdded(const FileHistory*, const QValueVector<QString>&)));
 
-	connect(git, SIGNAL(loadCompleted(const QString&)),
-	        this, SLOT(on_loadCompleted(const QString&)));
+	connect(git, SIGNAL(loadCompleted(const FileHistory*, const QString&)),
+		this, SLOT(on_loadCompleted(const FileHistory*, const QString&)));
 
 	connect(listViewLog, SIGNAL(lanesContextMenuRequested(const QStringList&,
 	        const QStringList&)), this, SLOT(on_lanesContextMenuRequested
@@ -58,16 +61,22 @@ RevsView::RevsView(MainImpl* mi, Git* g) : Domain(mi, g) {
 	        (const QString&, const QStringList&, const QString&)));
 
 	connect(listViewLog, SIGNAL(contextMenu(const QString&, int)),
-	        d, SLOT(on_contextMenu(const QString&, int)));
+	        this, SLOT(on_contextMenu(const QString&, int)));
 
 	connect(treeView, SIGNAL(contextMenu(const QString&, int)),
-	        d, SLOT(on_contextMenu(const QString&, int)));
+	        this, SLOT(on_contextMenu(const QString&, int)));
 
 	connect(listBoxFiles, SIGNAL(contextMenu(const QString&, int)),
-	        d, SLOT(on_contextMenu(const QString&, int)));
+	        this, SLOT(on_contextMenu(const QString&, int)));
+}
 
-	connect(listBoxFiles_2, SIGNAL(contextMenu(const QString&, int)),
-	        d, SLOT(on_contextMenu(const QString&, int)));
+RevsView::~RevsView() {
+
+	if (!parent())
+		return;
+
+	delete linkedPatchView;
+	delete revTab;
 }
 
 void RevsView::clear(bool keepState) {
@@ -76,57 +85,66 @@ void RevsView::clear(bool keepState) {
 		st.clear();
 
 	listViewLog->clear();
-	textBrowserDesc->clear();
-	textBrowserDesc_2->clear();
+	tab()->textBrowserDesc->clear();
 	listBoxFiles->clear();
-	listBoxFiles_2->clear();
 	treeView->clear();
 	updateLineEditSHA(true);
-	if (patchView)
-		patchView->clear();
+	if (linkedPatchView)
+		linkedPatchView->clear();
 }
 
-void RevsView::setPatchView(bool b) {
+void RevsView::setEnabled(bool b) {
 
-	if (b && patchView)
+	revTab->setEnabled(b);
+	if (linkedPatchView)
+		linkedPatchView->tab()->setEnabled(b);
+}
+
+void RevsView::viewPatch(bool newTab) {
+
+	if (!newTab && linkedPatchView) {
+		m()->tabWdg->setCurrentPage(linkedPatchView->tabPos());
 		return;
+	}
+	PatchView* pv = new PatchView(m(), git);
+	m()->tabWdg->setCurrentPage(pv->tabPos());
 
-	if (b) {
-		patchView = new PatchView(this, git, m()->textEditDiff,
-		                          m()->lineEditDiff, m()->radioButtonSha);
-
-		connect(listBoxFiles, SIGNAL(fileSelected(const QString&)),
-		        patchView, SLOT(on_fileSelected(const QString&)));
+	if (!newTab) { // linkedPatchView == NULL
+		linkedPatchView = pv;
+		linkDomain(linkedPatchView);
 
 		connect(m(), SIGNAL(highlightPatch(const QString&, bool)),
-		        patchView, SLOT(on_highlightPatch(const QString&, bool)));
+			linkedPatchView, SLOT(on_highlightPatch(const QString&, bool)));
 
-		patchView->update(true);
-		patchView->on_fileSelected(st.fileName());
-	} else
-		delete patchView;
+		connect(linkedPatchView->tab()->listBoxFiles, SIGNAL(doubleClicked(QListBoxItem*)),
+			m(), SLOT(on_fileList_doubleClicked(QListBoxItem*)));
+	}
+	connect(m(), SIGNAL(closeAllTabs()), pv, SLOT(on_closeAllTabs()));
+	pv->st = st;
+	UPDATE_DM_MASTER(pv);
 }
 
-void RevsView::on_loadCompleted(const QString& stats) {
+void RevsView::on_loadCompleted(const FileHistory* fh, const QString& stats) {
+
+	if (fh)
+		return;
 
 	if (st.sha().isEmpty()) { // point to first one in list
-		if (m()->listViewLog->firstChild()) {
-			SCRef firstRev(m()->listViewLog->firstChild()->text(QGit::COMMIT_COL));
+		if (tab()->listViewLog->firstChild()) {
+			SCRef firstRev(tab()->listViewLog->firstChild()->text(QGit::COMMIT_COL));
 			st.setSha(firstRev);
 			st.setSelectItem(true);
 		}
 	}
-	UPDATE_DOMAIN(d);
-
-	// queue message after UPDATE_DOMAIN()
-	QApplication::postEvent(d, new MessageEvent(stats));
+	UPDATE();
+	QApplication::postEvent(this, new MessageEvent(stats));
 }
 
 bool RevsView::doUpdate() {
 
 	try {
-		EM_REGISTER(exSetRepositoryCalled);
-		EM_REGISTER(exExiting);
+		EM_REGISTER(m()->exSetRepositoryCalled);
+		EM_REGISTER(m()->exExiting);
 
 		bool dataCleared = m()->lineEditSHA->text().isEmpty();
 		bool found = listViewLog->update();
@@ -143,12 +161,12 @@ bool RevsView::doUpdate() {
 			bool allChanged = (st.allMergeFiles(true) != st.allMergeFiles(false));
 
 			if (shaChanged || dataCleared) {
+
 				updateLineEditSHA();
 				SCRef d(git->getDesc(st.sha(), m()->shortLogRE, m()->longLogRE));
-				textBrowserDesc->setText(d);
-				textBrowserDesc->setCursorPosition(0, 0);
-				textBrowserDesc_2->setText(d);
-				textBrowserDesc_2->setCursorPosition(0, 0);
+				tab()->textBrowserDesc->setText(d);
+				tab()->textBrowserDesc->setCursorPosition(0, 0);
+
 				m()->statusBar()->message(git->getRevInfo(st.sha(), false));
 			}
 			const RevFile* files = NULL;
@@ -158,7 +176,6 @@ bool RevsView::doUpdate() {
 				files = git->getFiles(st.sha(), st.diffToSha(), st.allMergeFiles());
 
 			listBoxFiles->update(files);
-			listBoxFiles_2->update(files);
 
 			if (m()->treeView->isVisible())
 				treeView->update(); // blocking call
@@ -169,34 +186,36 @@ bool RevsView::doUpdate() {
 			}
 			// at the end update diffs that is the slowest and must be
 			// run after update of file list for 'diff to sha' to work
-			if (patchView)
-				patchView->update(dataCleared);
+			if (linkedPatchView) {
+				linkedPatchView->st = st;
+				UPDATE_DM_MASTER(linkedPatchView); // async call
+			}
 		}
-		if (!found && dataCleared && m()->listViewLog->currentItem()) {
+		if (!found && dataCleared && tab()->listViewLog->currentItem()) {
 			// we are in an inconsistent state: list view current item is
 			// not selected and secondary panes are empty.
 			// This could happen as example after removing a tree filter.
 			// At least populate secondary panes
-			SCRef firstRev(m()->listViewLog->currentItem()->text(QGit::COMMIT_COL));
+			SCRef firstRev(tab()->listViewLog->currentItem()->text(QGit::COMMIT_COL));
 			st.setSha(firstRev);
 			st.setSelectItem(false);
-			customEvent(new UpdateDomainEvent()); // will be queued immediately
+			customEvent(new UpdateDomainEvent(false)); // will be queued immediately
 		}
-		EM_REMOVE(exExiting);
-		EM_REMOVE(exSetRepositoryCalled);
+		EM_REMOVE(m()->exExiting);
+		EM_REMOVE(m()->exSetRepositoryCalled);
 
 		return found;
 
 	} catch(int i) {
 
-		EM_REMOVE(exExiting);
-		EM_REMOVE(exSetRepositoryCalled);
+		EM_REMOVE(m()->exExiting);
+		EM_REMOVE(m()->exSetRepositoryCalled);
 
-		if (EM_MATCH(i, exSetRepositoryCalled, "update views")) {
+		if (EM_MATCH(i, m()->exSetRepositoryCalled, "update views")) {
 			EM_CHECK_PENDING;
 			return false;
 		}
-		if (EM_MATCH(i, exExiting, "update main view")) {
+		if (EM_MATCH(i, m()->exExiting, "update main view")) {
 			EM_CHECK_PENDING;
 			return false;
 		}
@@ -252,7 +271,7 @@ void RevsView::on_lanesContextMenuRequested(SCList parents, SCList childs) {
 	int cc = (int)childs.count();
 	SCRef target((id < cc) ? childs[id] : parents[id - cc]);
 	st.setSha(target);
-	UPDATE_DOMAIN(d);
+	UPDATE();
 }
 
 void RevsView::on_droppedRevisions(const QString& sha, const QStringList& remoteRevs,
@@ -271,7 +290,6 @@ void RevsView::on_droppedRevisions(const QString& sha, const QStringList& remote
 		m()->statusBar()->message(tmp);
 		return;
 	}
-
 	bool commit, fold;
 	if (!m()->askApplyPatchParameters(&commit, &fold))
 		return;
@@ -302,7 +320,7 @@ void RevsView::on_droppedRevisions(const QString& sha, const QStringList& remote
 			       "in %s", dr.count(), QGit::PATCHES_DIR.latin1());
 			break;
 		}
-		bool ok = git->applyPatchFile(dr.absFilePath(dr[0]), commit, fold, true);
+		bool ok = git->applyPatchFile(dr.absFilePath(dr[0]), commit, fold, !Git::optSign);
 		dr.remove(dr[0]);
 		if (!ok)
 			break;
