@@ -8,6 +8,7 @@
 #define REVSVIEW_H
 
 #include <qguardedptr.h>
+#include "revbase.h" // needed by moc_* file to understand tab() function
 #include "exceptionmanager.h"
 #include "common.h"
 #include "domain.h"
@@ -26,39 +27,38 @@ class ListBoxFiles;
 class PatchView;
 class TreeView;
 class MainImpl;
+class FileHistory;
 
 class RevsView : public Domain {
 Q_OBJECT
 public:
 	RevsView(MainImpl* parent, Git* git);
+	~RevsView();
 	void clear(bool keepState);
-	void setPatchView(bool b);
+	void viewPatch(bool newTab);
+	void setEnabled(bool b);
+	virtual TabRev* tab() { return revTab; }
 
 private slots:
-	void on_loadCompleted(const QString& stats);
+	void on_loadCompleted(const FileHistory*, const QString& stats);
 	void on_lanesContextMenuRequested(const QStringList&, const QStringList&);
 	void on_droppedRevisions(const QString&, const QStringList&, const QString&);
 
+protected:
+	virtual bool doUpdate();
+
 private:
 	friend class MainImpl;
 
-	virtual bool doUpdate();
-
 	bool listBoxFiles_mouseMoveEvent();
 	bool listBoxFiles_mouseReleaseEvent();
 	void updateLineEditSHA(bool clear = false);
 
-	EM_DECLARE(exSetRepositoryCalled);
-	EM_DECLARE(exExiting);
-
-	QWidget* tabPage;
-	QGuardedPtr<PatchView> patchView;
+	TabRev* revTab;
+	QGuardedPtr<PatchView> linkedPatchView;
 	TreeView* treeView;
 	ListView* listViewLog;
-	RevDesc* textBrowserDesc;
-	RevDesc* textBrowserDesc_2;
 	ListBoxFiles* listBoxFiles;
-	ListBoxFiles* listBoxFiles_2;
 };
 
 #endif
