@@ -18,9 +18,9 @@
 #include "mainimpl.h"
 #include "treeview.h"
 
-static QPixmap* folderClosed = NULL;
-static QPixmap* folderOpen = NULL;
-static QPixmap* fileDefault = NULL;
+QPixmap* DirItem::folderClosed = NULL;
+QPixmap* DirItem::folderOpen = NULL;
+QPixmap* TreeView::fileDefault = NULL;
 
 // ******************************* FileItem ****************************
 
@@ -35,16 +35,12 @@ void FileItem::setPixmap(QPixmap* p) {
 
 const QPixmap* FileItem::pixmap(int i) const {
 
-	if (i > 0)
-		return 0;
-	return pix;
+	return (i > 0 ? NULL : pix);
 }
 
 QString FileItem::text(int column) const {
 
-	if (column == 0)
-		return name;
-	return "";
+	return (column == 0 ? name : "");
 }
 
 QString FileItem::fullName() {
@@ -156,26 +152,27 @@ TreeView::TreeView(Domain* dm, Git* g, QListView* lv) :
 	initMimePix();
 
 	connect(listView, SIGNAL(currentChanged(QListViewItem*)),
-	        this, SLOT(slotCurrentChanged(QListViewItem*)));
-	connect(listView, SIGNAL(contextMenuRequested(QListViewItem*,const QPoint&,int)),
-	        this, SLOT(contextMenuRequested(QListViewItem*,const QPoint&,int)));
+	        this, SLOT(on_currentChanged(QListViewItem*)));
+
+	connect(listView, SIGNAL(contextMenuRequested(QListViewItem*, const QPoint&, int)),
+	        this, SLOT(on_contextMenuRequested(QListViewItem*, const QPoint&, int)));
 }
 
 void TreeView::initMimePix() {
 
-	MainImpl* m = const_cast<MainImpl*>(d->m());
+	MainImpl* m = d->m(); // just a shortcut
 
 	setMimePix("#FOLDER_CLOSED", m->lblFolder->pixmap());
-	setMimePix("#FOLDER_OPEN", m->lblFolderOpen->pixmap());
-	setMimePix("#DEFAULT", m->lblFile->pixmap());
-	setMimePix("c", m->lblC->pixmap());
-	setMimePix("cpp", m->lblCpp->pixmap());
-	setMimePix("h", m->lblH->pixmap());
-	setMimePix("txt", m->lblTxt->pixmap());
-	setMimePix("sh", m->lblSh->pixmap());
-	setMimePix("perl", m->lblPl->pixmap());
-	setMimePix("pl", m->lblPl->pixmap());
-	setMimePix("py", m->lblPy->pixmap());
+	setMimePix("#FOLDER_OPEN",   m->lblFolderOpen->pixmap());
+	setMimePix("#DEFAULT",       m->lblFile->pixmap());
+	setMimePix("c",              m->lblC->pixmap());
+	setMimePix("cpp",            m->lblCpp->pixmap());
+	setMimePix("h",              m->lblH->pixmap());
+	setMimePix("txt",            m->lblTxt->pixmap());
+	setMimePix("sh",             m->lblSh->pixmap());
+	setMimePix("perl",           m->lblPl->pixmap());
+	setMimePix("pl",             m->lblPl->pixmap());
+	setMimePix("py",             m->lblPy->pixmap());
 
 	m->lblFolder->hide();
 	m->lblFolderOpen->hide();
@@ -189,7 +186,7 @@ void TreeView::initMimePix() {
 	m->lblPy->hide();
 }
 
-void TreeView::slotCurrentChanged(QListViewItem* item) {
+void TreeView::on_currentChanged(QListViewItem* item) {
 
 	if (item) {
 		SCRef fn = ((FileItem*)item)->fullName();
@@ -201,12 +198,10 @@ void TreeView::slotCurrentChanged(QListViewItem* item) {
 	}
 }
 
-void TreeView::contextMenuRequested(QListViewItem* item, const QPoint&,int) {
+void TreeView::on_contextMenuRequested(QListViewItem* item, const QPoint&,int) {
 
-	if (!item)
-		return;
-
-	emit contextMenu(fullName(item), POPUP_TREE_EV);
+	if (item)
+		emit contextMenu(fullName(item), POPUP_TREE_EV);
 }
 
 void TreeView::clear() {
@@ -231,24 +226,17 @@ bool TreeView::isDir(SCRef fileName) {
 	// if currentItem is NULL or is different from fileName
 	// return false, because treeview is not updated while
 	// not visible, so could be out of sync.
-	if (listView->currentItem() == NULL)
+	FileItem* item = static_cast<FileItem*>(listView->currentItem());
+	if (item == NULL || item->fullName() != fileName)
 		return false;
 
-	FileItem* item = (FileItem*)listView->currentItem();
-	if (item->fullName() != fileName)
-		return false;
-
-	DirItem* d = dynamic_cast<DirItem*>(listView->currentItem());
-	return (d != NULL);
+	return dynamic_cast<DirItem*>(item);
 }
 
 const QString TreeView::fullName(QListViewItem* item) {
 
-	if (item == NULL)
-		return QString();
-
 	FileItem* f = static_cast<FileItem*>(item);
-	return f->fullName();
+	return (item ? f->fullName() : "");
 }
 
 void TreeView::getTreeSelectedItems(QStringList& selectedItems) {
@@ -256,9 +244,9 @@ void TreeView::getTreeSelectedItems(QStringList& selectedItems) {
 	selectedItems.clear();
 	QListViewItemIterator it(listView);
 	while (it.current()) {
-		QListViewItem* item = it.current();
-		if (item->isSelected())
-			selectedItems.append(((FileItem*)item)->fullName());
+		FileItem* f = static_cast<FileItem*>(it.current());
+		if (f->isSelected())
+			selectedItems.append(f->fullName());
 		++it;
 	}
 }
@@ -267,16 +255,17 @@ void TreeView::setMimePix(SCRef ext, QPixmap* pix) {
 
 	// common cases first
 	if (ext == "#FOLDER_CLOSED")
-		folderClosed = pix;
-	if (ext == "#FOLDER_OPEN")
-		folderOpen = pix;
-	if (ext == "#DEFAULT")
+		DirItem::folderClosed = pix;
+
+	else if (ext == "#FOLDER_OPEN")
+		DirItem::folderOpen = pix;
+
+	else if (ext == "#DEFAULT")
 		fileDefault = pix;
 
-	//added extensions
-	if (mimePixMap.find(ext) != 0)
-		return;
-	mimePixMap.insert(ext, pix);
+	// set added extensions
+	if (!mimePixMap.find(ext))
+		mimePixMap.insert(ext, pix);
 }
 
 QPixmap* TreeView::mimePix(SCRef fileName) {
@@ -286,10 +275,7 @@ QPixmap* TreeView::mimePix(SCRef fileName) {
 		return fileDefault;
 
 	QPixmap* pix = mimePixMap.find(ext);
-	if (pix)
-		return pix;
-
-	return fileDefault;
+	return (pix ? pix : fileDefault);
 }
 
 void TreeView::setTreeName(SCRef treeName) {
@@ -322,8 +308,8 @@ void TreeView::update() {
 	if (st->sha().isEmpty())
 		return;
 
-	// qt emits currentChanged() signal when populating the
-	// list view, so we should ignore while here.
+	// qt emits currentChanged() signal when populating
+	// the list view, so we should ignore while here
 	ignoreCurrentChanged = true;
 
 	try {
@@ -332,43 +318,38 @@ void TreeView::update() {
 		QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
 
 		bool newTree = true;
-		DirItem* root = (DirItem*)(listView->firstChild());
+		DirItem* root = static_cast<DirItem*>(listView->firstChild());
 		if (root)
 			newTree = (root->treeSha != st->sha());
 
-		if (newTree && root && st->sha() != ZERO_SHA &&
-			root->treeSha != ZERO_SHA)
+		if (   newTree
+		    && root
+		    && st->sha() != ZERO_SHA
+		    && root->treeSha != ZERO_SHA)
 			// root->treeSha could reference a different sha from current
 			// one in case the tree is the same, i.e. has the same files.
 			// so we must use the state sha to call isSameFiles() and
 			// benefit from the early skip logic
-			newTree = !git->isSameFiles(st->sha(false), st->sha());
+			newTree = !git->isSameFiles(st->sha(false), st->sha(true));
 
-		if (newTree)
+		if (newTree) // ok, we really need to update the tree
 			setTree(st->sha());
+		else {
+			FileItem* f = static_cast<FileItem*>(listView->currentItem());
+			if (f && f->fullName() == st->fileName()) {
 
-		if (st->fileName().isEmpty()) {
-			if (newTree && oldRoot) {
-				while (oldRoot->itemBelow())
-					delete oldRoot->itemBelow();
-
-				delete oldRoot;
-				oldRoot = NULL;
-			}
-			ignoreCurrentChanged = false;
-			QApplication::restoreOverrideCursor();
-			EM_REMOVE(exTreeCleared);
-			return;
-		}
-		FileItem* cur = (FileItem*)listView->currentItem();
-		if (cur && !newTree) {
-			if (cur->fullName() == st->fileName()) {
-				ignoreCurrentChanged = false;
-				QApplication::restoreOverrideCursor();
-				EM_REMOVE(exTreeCleared);
+				restoreStuff();
 				return;
 			}
 		}
+		if (st->fileName().isEmpty()) {
+
+			if (newTree)
+				deleteOldTree();
+
+			restoreStuff();
+			return;
+		}
 		listView->setUpdatesEnabled(false);
 		QStringList lst(QStringList::split("/", st->fileName()));
 		QListViewItem* item(listView->firstChild());
@@ -381,35 +362,28 @@ void TreeView::update() {
 					// to be sure we need to check the names
 					if (st->fileName().startsWith(((FileItem*)item)->fullName())) {
 						item->setOpen(true);
-						break;
+						break; // from while loop only
 					}
 				}
 				item = item->itemBelow();
 			}
 		}
-		if (item == oldRoot)
-			item = NULL;
+		bool found = (item && item != oldRoot);
 
-		if (oldRoot != NULL) {
-			while (oldRoot->itemBelow())
-				delete oldRoot->itemBelow();
+		deleteOldTree();
 
-			delete oldRoot;
-			oldRoot = NULL;
-		}
-		if (item) {
+		if (found) {
 			listView->clearSelection();
 			listView->setSelected(item, true);
-			listView->setCurrentItem(item); // calls slotCurrentChanged()
+			listView->setCurrentItem(item); // calls on_currentChanged()
 			listView->ensureItemVisible(item);
 		}
 		listView->setUpdatesEnabled(true);
 		listView->triggerUpdate();
-		ignoreCurrentChanged = false;
-		QApplication::restoreOverrideCursor();
-		EM_REMOVE(exTreeCleared);
+		restoreStuff();
 
 	} catch (int i) {
+
 		EM_REMOVE(exTreeCleared);
 
 		if (EM_MATCH(i, exTreeCleared, "updating tree")) {
@@ -426,3 +400,19 @@ void TreeView::update() {
 		throw i;
 	}
 }
+
+void TreeView::restoreStuff() {
+
+	ignoreCurrentChanged = false;
+	QApplication::restoreOverrideCursor();
+	EM_REMOVE(exTreeCleared);
+}
+
+void TreeView::deleteOldTree() {
+
+	while (oldRoot && oldRoot->itemBelow())
+		delete oldRoot->itemBelow();
+
+	delete oldRoot;
+	oldRoot = NULL;
+}
